@@ -55,14 +55,20 @@ enum {
   ACTION_SHOW_PROFILER,
   ACTION_EXTRA_VIEW,
   ACTION_RACK_EDITOR,
-#define ACTION_HELP_FIRST   ACTION_HELP_FAQ
+#define ACTION_HELP_FIRST   ACTION_HELP_INDEX
+  ACTION_HELP_INDEX,
   ACTION_HELP_FAQ,
-  ACTION_HELP_KEYTABLE,
   ACTION_HELP_RELEASE_NOTES,
-  ACTION_HELP_GSL_PLAN,
   ACTION_HELP_QUICK_START,
+  ACTION_HELP_PLUGIN_DEVEL,
+  ACTION_HELP_DSP_ENGINE,
+  ACTION_HELP_DEVELOPMENT,
   ACTION_HELP_ABOUT,
 #define ACTION_HELP_LAST    ACTION_HELP_ABOUT
+  ACTION_URL_HELP_DESK,
+  ACTION_URL_BEAST_SITE,
+  ACTION_URL_ONLINE_SYNTHESIZERS,
+  ACTION_URL_ONLINE_DEMOS,
 };
 static const GxkStockAction file_open_actions[] = {
   { N_("_New"),                 "<ctrl>N",      N_("Create new project"),
@@ -73,81 +79,105 @@ static const GxkStockAction file_open_actions[] = {
     BST_ACTION_MERGE_PROJECT,   BST_STOCK_MERGE, },
   { N_("_Import MIDI..."),      "",             N_("Import a standard MIDI file into the current project"),
     BST_ACTION_IMPORT_MIDI,     BST_STOCK_OPEN, },
-  { N_("_Close"),               "<ctrl>W",      NULL,
+  { N_("_Close"),               "<ctrl>W",      N_("Close the project"),
     BST_ACTION_CLOSE_PROJECT,   BST_STOCK_CLOSE, },
 };
 static const GxkStockAction file_save_actions[] = {
-  { N_("_Save"),                NULL,           NULL,
+  { N_("_Save"),                NULL,           N_("Write project to disk"),
     BST_ACTION_SAVE_PROJECT,    BST_STOCK_SAVE, },
-  { N_("Save _As..."),          NULL,           NULL,
+  { N_("Save _As..."),          NULL,           N_("Write project to a specific file"),
     BST_ACTION_SAVE_PROJECT_AS, BST_STOCK_SAVE_AS, },
 };
 static const GxkStockAction file_epilog_actions[] = {
-  { N_("_Quit"),                "<ctrl>Q",      NULL,
+  { N_("_Quit"),                "<ctrl>Q",      N_("Close all windows and quit"),
     BST_ACTION_EXIT,            BST_STOCK_QUIT, },
 };
 static const GxkStockAction preference_actions[] = {
-  { N_("_Preferences..."),      NULL,           NULL,   ACTION_SHOW_PREFERENCES,    BST_STOCK_PREFERENCES, },
+  { N_("_Preferences..."),      NULL,           N_("Adjust overall program behaviour"),
+    ACTION_SHOW_PREFERENCES,    BST_STOCK_PREFERENCES, },
 };
 static const GxkStockAction rebuild_actions[] = {
   { N_("Rebuild"),              NULL,           NULL,   BST_ACTION_REBUILD, },
 };
 static const GxkStockAction about_actions[] = {
-  { N_("_About..."),            NULL,           NULL,   ACTION_HELP_ABOUT,          BST_STOCK_ABOUT },
+  { N_("_About..."),            NULL,           N_("Display developer and contributor credits"),
+    ACTION_HELP_ABOUT,          BST_STOCK_ABOUT },
 };
 static const GxkStockAction undo_actions[] = {
-  { N_("_Undo"),                "<ctrl>Z",      NULL,
+  { N_("_Undo"),                "<ctrl>Z",      N_("Undo the effect of the last action"),
     BST_ACTION_UNDO,            BST_STOCK_UNDO, },
-  { N_("_Redo"),                "<ctrl>R",      NULL,
+  { N_("_Redo"),                "<ctrl>Y",      N_("Redo the last undone action"),
     BST_ACTION_REDO,            BST_STOCK_REDO, },
 };
 static const GxkStockAction undo_dvl_actions[] = {
-  { N_("_Clear Undo"),          NULL,           NULL,
+  { N_("_Clear Undo"),          NULL,           N_("Delete the complete undo history"),
     BST_ACTION_CLEAR_UNDO,      BST_STOCK_CLEAR_UNDO, },
 };
 static const GxkStockAction dialog_actions[] = {
-  { N_("Procedure _Browser"),   NULL,           NULL,
+  { N_("Procedure _Browser"),   NULL,           N_("Display an overview of all procedures"),
     ACTION_SHOW_PROC_BROWSER, },
   { N_("Rack Editor"),          NULL,           NULL,
     ACTION_RACK_EDITOR, },
-  { N_("Profiler"),             NULL,           NULL,
+  { N_("Profiler"),             NULL,           N_("Display statistics and timing information"),
     ACTION_SHOW_PROFILER, },
-  { N_("New View"),             NULL,           NULL,
+  { N_("New View"),             NULL,           N_("Create an extra view of the project"),
     ACTION_EXTRA_VIEW, },
 };
 static const GxkStockAction playback_actions[] = {
-  { N_("_Play"),                "<ctrl>P",      NULL,
+  { N_("_Play"),                "<ctrl>P",      N_("Play or restart playback of the project"),
     BST_ACTION_START_PLAYBACK,  BST_STOCK_PLAY },
-  { N_("_Stop"),                "<ctrl>S",      NULL,
+  { N_("_Stop"),                "<ctrl>S",      N_("Stop playback of the project"),
     BST_ACTION_STOP_PLAYBACK,  BST_STOCK_STOP },
 };
 static const GxkStockAction project_actions[] = {
-  { N_("New Song"),             NULL,           NULL,
-    BST_ACTION_NEW_SONG,        BST_STOCK_NEW_SONG },
-  { N_("New Custom Synthesizer"), NULL,         NULL,
-    BST_ACTION_NEW_CSYNTH,      BST_STOCK_NEW_CSYNTH },
-  { N_("New MIDI Synthesizer"), NULL,           NULL,
-    BST_ACTION_NEW_MIDI_SYNTH,  BST_STOCK_NEW_MIDI_SYNTH },
-  { N_("Remove Song or Synthesizer"), NULL,     NULL,
+  { N_("New Song"),             NULL,           N_("Create a new song, consisting of a mixer, tracks, parts and notes"),
+    BST_ACTION_NEW_SONG,        BST_STOCK_MINI_SONG },
+  { N_("New Custom Synthesizer"), NULL,         N_("Create a new synthesizer mesh to be used as effect or instrument in songs"),
+    BST_ACTION_NEW_CSYNTH,      BST_STOCK_MINI_CSYNTH },
+  { N_("New MIDI Synthesizer"), NULL,           N_("Create a new MIDI synthesizer to control an instrument from external MIDI events"),
+    BST_ACTION_NEW_MIDI_SYNTH,  BST_STOCK_MINI_MIDI_SYNTH },
+  { N_("Remove Song or Synthesizer"), NULL,     N_("Remove the currently selected synthesizer (song)"),
     BST_ACTION_REMOVE_SYNTH,    BST_STOCK_REMOVE_SYNTH },
 };
 static const GxkStockAction library_files_actions[] = {
-  { N_("Load _Instrument"),     NULL,           NULL,
-    BST_ACTION_MERGE_INSTRUMENT, },
-  { N_("Load _Effect"),         NULL,           NULL,
-    BST_ACTION_MERGE_EFFECT, },
-  { N_("Save As Instrument"),   NULL,           NULL,
-    BST_ACTION_SAVE_INSTRUMENT, },
-  { N_("Save As Effect"),       NULL,           NULL,
-    BST_ACTION_SAVE_EFFECT, },
+  { N_("Load _Instrument..."),  NULL,           N_("Load synthesizer mesh from instruments folder"),
+    BST_ACTION_MERGE_INSTRUMENT,BST_STOCK_OPEN },
+  { N_("Load _Effect..."),      NULL,           N_("Load synthesizer mesh from effects folder"),
+    BST_ACTION_MERGE_EFFECT,    BST_STOCK_OPEN },
+  { N_("Save As Instrument..."),NULL,           N_("Save synthesizer mesh to instruments folder"),
+    BST_ACTION_SAVE_INSTRUMENT, BST_STOCK_SAVE_AS },
+  { N_("Save As Effect..."),    NULL,           N_("Save synthesizer mesh to effects folder"),
+    BST_ACTION_SAVE_EFFECT,     BST_STOCK_SAVE_AS },
 };
 static const GxkStockAction simple_help_actions[] = {
-  { N_("_Release Notes..."),    NULL,           NULL,   ACTION_HELP_RELEASE_NOTES,  BST_STOCK_DOC_NEWS },
-  { N_("Quick Start..."),       NULL,           NULL,   ACTION_HELP_QUICK_START,    BST_STOCK_HELP },
-  { N_("_FAQ..."),              NULL,           NULL,   ACTION_HELP_FAQ,            BST_STOCK_DOC_FAQ },
+  { N_("Document _Index..."),   NULL,           N_("Provide an overview of all BEAST documentation contents"),
+    ACTION_HELP_INDEX,          BST_STOCK_DOC_INDEX },
+  { N_("_Quick Start..."),      NULL,           N_("Provides an introduction about how to accomplish the most common tasks"),
+    ACTION_HELP_QUICK_START,    BST_STOCK_HELP },
+  { N_("_FAQ..."),              NULL,           N_("Frequently asked questions"),
+    ACTION_HELP_FAQ,            BST_STOCK_DOC_FAQ },
+  { N_("Online _Help Desk..."), NULL,           N_("Start a web browser pointing to the online help desk at the BEAST website"),
+    ACTION_URL_HELP_DESK,       BST_STOCK_ONLINE_HELP_DESK },
+  { N_("_BEAST Website..."),    NULL,           N_("Start a web browser pointing to the BEAST website"),
+    ACTION_URL_BEAST_SITE,      BST_STOCK_ONLINE_BEAST_SITE },
+#if 0
+  { N_("_Release Notes..."),    NULL,           N_("Notes and informations about this release cycle"),
+    ACTION_HELP_RELEASE_NOTES,  BST_STOCK_DOC_NEWS },
+  { N_("Developing Plugins..."),NULL,           N_("A guide to synthesis plugin development"),
+    ACTION_HELP_PLUGIN_DEVEL,   BST_STOCK_DOC_DEVEL },
+  { N_("DSP Engine..."),        NULL,           N_("Technical description of the multi-threaded synthesis engine innards"),
+    ACTION_HELP_DSP_ENGINE,     BST_STOCK_DOC_DEVEL },
+#endif
+  { N_("Development..."),       NULL,           N_("Provide an overview of development related topics and documents"),
+    ACTION_HELP_DEVELOPMENT,    BST_STOCK_DOC_DEVEL },
 };
-static const GxkStockAction devel_help_actions[] = {
-  { N_("GSL Engine..."),        NULL,           NULL,   ACTION_HELP_GSL_PLAN,       BST_STOCK_DOC_DEVEL },
+static const GxkStockAction online_synthesizers[] = {
+  { N_("Online Sound Archive..."),  NULL,        N_("Start a web browser pointing to the online sound archive"),
+    ACTION_URL_ONLINE_SYNTHESIZERS, BST_STOCK_ONLINE_SOUND_ARCHIVE },
+};
+static const GxkStockAction online_demos[] = {
+  { N_("Online Demos..."),          NULL,           N_("Start a web browser pointing to online demo songs"),
+    ACTION_URL_ONLINE_DEMOS,        BST_STOCK_ONLINE_SOUND_ARCHIVE },
 };
 
 
@@ -196,7 +226,7 @@ bst_app_init (BstApp *self)
                 "name", "BEAST-Application",
                 "allow_shrink", TRUE,
                 "allow_grow", TRUE,
-                "flags", GXK_DIALOG_STATUS_SHELL,
+                "flags", GXK_DIALOG_STATUS_BAR | GXK_DIALOG_IGNORE_ESCAPE, // | GXK_DIALOG_WINDOW_GROUP,
                 NULL);
   bst_app_register (self);
   self->box = gxk_radget_create ("beast", "application-box", NULL);
@@ -230,7 +260,9 @@ bst_app_init (BstApp *self)
                               NULL, app_action_check, app_action_exec);
   gxk_widget_publish_actions (self, "simple-help", G_N_ELEMENTS (simple_help_actions), simple_help_actions,
                               NULL, app_action_check, app_action_exec);
-  gxk_widget_publish_actions (self, "devel-help", G_N_ELEMENTS (devel_help_actions), devel_help_actions,
+  gxk_widget_publish_actions (self, "online-synthesizers", G_N_ELEMENTS (online_synthesizers), online_synthesizers,
+                              NULL, app_action_check, app_action_exec);
+  gxk_widget_publish_actions (self, "online-demos", G_N_ELEMENTS (online_demos), online_demos,
                               NULL, app_action_check, app_action_exec);
   /* Project utilities */
   cseq = bse_categories_match ("/Project/*");
@@ -332,22 +364,11 @@ bst_app_destroy (GtkObject *object)
 BstApp*
 bst_app_new (SfiProxy project)
 {
-  GdkGeometry geometry;
-  GtkWidget *widget;
-  BstApp *self;
-  
   g_return_val_if_fail (BSE_IS_PROJECT (project), NULL);
   
-  widget = gtk_widget_new (BST_TYPE_APP,
-                           "default_width", 640,
-                           "default_height", 512,
-                           NULL);
-  self = BST_APP (widget);
-  
-  geometry.min_width = 320;
-  geometry.min_height = 450;
-  gtk_window_set_geometry_hints (GTK_WINDOW (widget), NULL, &geometry, GDK_HINT_MIN_SIZE);
-  
+  BstApp *self = g_object_new (BST_TYPE_APP, NULL);
+  gxk_dialog_set_sizes (GXK_DIALOG (self), 500, 400, 950, 800);
+
   self->project = project;
   bse_item_use (self->project);
   bse_proxy_connect (self->project,
@@ -413,39 +434,76 @@ bst_app_get_current_super (BstApp *app)
   return 0;
 }
 
-void
-bst_app_reload_supers (BstApp *app)
+static gint
+proxy_rate_super (SfiProxy p)
 {
-  GtkWidget *old_page, *old_focus, *song_page = NULL, *synth_page = NULL;
+  if (BSE_IS_WAVE_REPO (p))
+    return 1;
+  else if (BSE_IS_SONG (p))
+    return 2;
+  else if (BSE_IS_MIDI_SYNTH (p))
+    return 4;
+  else if (BSE_IS_CSYNTH (p))
+    return 3;
+  return 5;
+}
+
+static gint
+proxyp_cmp_supers (gconstpointer v1,
+                   gconstpointer v2,
+                   gpointer      data)
+{
+  const SfiProxy *p1 = v1;
+  const SfiProxy *p2 = v2;
+  if (*p1 == *p2)
+    return 0;
+  return proxy_rate_super (*p1) - proxy_rate_super (*p2);
+}
+
+void
+bst_app_reload_supers (BstApp *self)
+{
+  GtkWidget *old_page, *old_focus, *first_unseen = NULL, *first_synth = NULL;
   GSList *page_list = NULL;
   GSList *slist;
   BseItemSeq *iseq;
   guint i;
   
-  g_return_if_fail (BST_IS_APP (app));
+  g_return_if_fail (BST_IS_APP (self));
   
-  old_focus = GTK_WINDOW (app)->focus_widget;
+  old_focus = GTK_WINDOW (self)->focus_widget;
   if (old_focus)
     gtk_widget_ref (old_focus);
-  old_page = app->notebook->cur_page ? gtk_notebook_current_widget (app->notebook) : NULL;
-  while (gtk_notebook_current_widget (app->notebook))
+  old_page = gtk_notebook_current_widget (self->notebook);
+
+  gtk_widget_hide (GTK_WIDGET (self->notebook));
+
+  while (gtk_notebook_current_widget (self->notebook))
     {
-      g_object_ref (gtk_notebook_current_widget (app->notebook));
-      page_list = g_slist_prepend (page_list, gtk_notebook_current_widget (app->notebook));
-      gtk_container_remove (GTK_CONTAINER (app->notebook), page_list->data);
+      g_object_ref (gtk_notebook_current_widget (self->notebook));
+      page_list = g_slist_prepend (page_list, gtk_notebook_current_widget (self->notebook));
+      gtk_container_remove (GTK_CONTAINER (self->notebook), page_list->data);
     }
   
-  iseq = bse_project_get_supers (app->project);
+  /* get supers */
+  iseq = bse_project_get_supers (self->project);
+  SfiRing *ring, *supers = NULL;
+  /* convert to ring */
   for (i = 0; i < iseq->n_items; i++)
+    supers = sfi_ring_append (supers, iseq->items + i);
+  /* sort supers */
+  supers = sfi_ring_sort (supers, proxyp_cmp_supers, NULL);
+  /* update shells */
+  for (ring = supers; ring; ring = sfi_ring_next (ring, supers))
     {
-      GtkWidget *label, *page = NULL;
-      GSList *node;
-      
-      if (!BST_DBG_EXT && bse_item_internal (iseq->items[i]))
+      SfiProxy *pp = ring->data, super = *pp;
+      if (bse_item_internal (super) && !BST_DBG_EXT)
         continue;
-      
+
+      GtkWidget *page = NULL;
+      GSList *node;
       for (node = page_list; node; node = node->next)
-        if (BST_SUPER_SHELL (node->data)->super == iseq->items[i])
+        if (BST_SUPER_SHELL (node->data)->super == super)
           {
             page = node->data;
             page_list = g_slist_remove (page_list, page);
@@ -453,37 +511,36 @@ bst_app_reload_supers (BstApp *app)
           }
       if (!page)
         {
-          page = g_object_new (BST_TYPE_SUPER_SHELL, "super", iseq->items[i], NULL);
+          page = g_object_new (BST_TYPE_SUPER_SHELL, "super", super, NULL);
           g_object_ref (page);
           gtk_object_sink (GTK_OBJECT (page));
+          if (!first_unseen)
+            first_unseen = page;
         }
-      if (page)
-        {
-          if (!song_page && BSE_IS_SONG (iseq->items[i]))
-            song_page = page;
-          else if (!synth_page && BSE_IS_SNET (iseq->items[i]))
-            synth_page = page;
-          label = g_object_new (GTK_TYPE_LABEL,
-                                "visible", TRUE,
-                                "width_request", BST_TAB_WIDTH ? BST_TAB_WIDTH : -1,
-                                NULL);
-          gtk_notebook_append_page (app->notebook, page, label);
-          gtk_notebook_set_tab_label_packing (app->notebook, page, FALSE, TRUE, GTK_PACK_START);
-          bst_super_shell_update_label (BST_SUPER_SHELL (page));
-          gtk_widget_unref (page);
-        }
+      if (!first_synth && BSE_IS_SNET (super))
+        first_synth = page;
+      GtkWidget *label = bst_super_shell_create_label (BST_SUPER_SHELL (page));
+      gtk_notebook_append_page (self->notebook, page, label);
+      gtk_notebook_set_tab_label_packing (self->notebook, page, FALSE, TRUE, GTK_PACK_START);
+      bst_super_shell_update_label (BST_SUPER_SHELL (page));
+      gtk_widget_unref (page);
     }
-  if (old_page && old_page->parent == GTK_WIDGET (app->notebook))
-    gxk_notebook_set_current_page_widget (app->notebook, old_page);
-  else if (song_page)
-    gxk_notebook_set_current_page_widget (app->notebook, song_page);
-  else if (synth_page)
-    gxk_notebook_set_current_page_widget (app->notebook, synth_page);
+  /* free ring */
+  sfi_ring_free (ring);
   
+  /* select/restore current page */
+  if (first_unseen && self->select_unseen_super)
+    gxk_notebook_set_current_page_widget (self->notebook, first_unseen);
+  else if (old_page && old_page->parent == GTK_WIDGET (self->notebook))
+    gxk_notebook_set_current_page_widget (self->notebook, old_page);
+  else if (first_synth)
+    gxk_notebook_set_current_page_widget (self->notebook, first_synth);
+  self->select_unseen_super = FALSE;
+  /* restore focus */
   if (old_focus)
     {
-      if (old_page && gtk_widget_is_ancestor (old_focus, old_page) &&
-          gtk_widget_get_toplevel (old_focus) == GTK_WIDGET (app))
+      if (gxk_widget_ancestry_viewable (old_focus) &&
+          gtk_widget_get_toplevel (old_focus) == GTK_WIDGET (self))
         gtk_widget_grab_focus (old_focus);
       gtk_widget_unref (old_focus);
     }
@@ -493,6 +550,8 @@ bst_app_reload_supers (BstApp *app)
       gtk_widget_unref (slist->data);
     }
   g_slist_free (page_list);
+
+  gtk_widget_show (GTK_WIDGET (self->notebook));
 }
 
 static gboolean
@@ -564,7 +623,7 @@ demo_play_song (gpointer data,
 {
   const gchar *file_name = demo_entries[callback_action - BST_ACTION_LOAD_DEMO_0000].file;
   SfiProxy project = bse_server_use_new_project (BSE_SERVER, file_name);
-  BseErrorType error = bse_project_restore_from_file (project, file_name);
+  BseErrorType error = bst_project_restore_from_file (project, file_name);
   if (error)
     bst_status_eprintf (error, _("Opening project `%s'"), file_name);
   else
@@ -743,9 +802,11 @@ app_action_exec (gpointer data,
       break;
     case BST_ACTION_MERGE_EFFECT:
       bst_file_dialog_popup_merge_effect (self, self->project);
+      self->select_unseen_super = TRUE;
       break;
     case BST_ACTION_MERGE_INSTRUMENT:
       bst_file_dialog_popup_merge_instrument (self, self->project);
+      self->select_unseen_super = TRUE;
       break;
     case BST_ACTION_SAVE_EFFECT:
       bst_file_dialog_popup_save_effect (self, self->project, bst_app_get_current_super (self));
@@ -754,18 +815,25 @@ app_action_exec (gpointer data,
       bst_file_dialog_popup_save_instrument (self, self->project, bst_app_get_current_super (self));
       break;
     case BST_ACTION_NEW_SONG:
+      bse_item_group_undo (self->project, "Create Song");
       proxy = bse_project_create_song (self->project, NULL);
+      bse_song_ensure_master_bus (proxy);
+      bse_item_ungroup_undo (self->project);
+      self->select_unseen_super = TRUE;
       break;
     case BST_ACTION_NEW_CSYNTH:
       proxy = bse_project_create_csynth (self->project, NULL);
+      self->select_unseen_super = TRUE;
       break;
     case BST_ACTION_NEW_MIDI_SYNTH:
       proxy = bse_project_create_midi_synth (self->project, NULL);
+      self->select_unseen_super = TRUE;
       break;
     case BST_ACTION_REMOVE_SYNTH:
       proxy = bst_app_get_current_super (self);
       if (BSE_IS_SNET (proxy) && !bse_project_is_active (self->project))
         bse_project_remove_snet (self->project, proxy);
+      self->select_unseen_super = FALSE;
       break;
     case BST_ACTION_CLEAR_UNDO:
       bse_project_clear_undo (self->project);
@@ -878,12 +946,12 @@ app_action_exec (gpointer data,
                              NULL);
       gtk_widget_queue_draw (GTK_WIDGET (self->notebook));
       break;
-    case ACTION_HELP_FAQ:
-      help_file = g_strconcat (BST_PATH_DOCS, "/faq.markup", NULL);
+    case ACTION_HELP_INDEX:
+      help_file = g_strconcat (BST_PATH_DOCS, "/beast-index.markup", NULL);
       help_title = g_strdup (help_file);
       goto HELP_DIALOG;
-    case ACTION_HELP_GSL_PLAN:
-      help_file = g_strconcat (BST_PATH_DOCS, "/engine-mplan.markup", NULL);
+    case ACTION_HELP_FAQ:
+      help_file = g_strconcat (BST_PATH_DOCS, "/faq.markup", NULL);
       help_title = g_strdup (help_file);
       goto HELP_DIALOG;
     case ACTION_HELP_QUICK_START:
@@ -894,6 +962,18 @@ app_action_exec (gpointer data,
       help_file = g_strconcat (BST_PATH_DOCS, "/release-notes.markup", NULL);
       help_title = g_strdup_printf (_("BEAST-%s Release Notes"), BST_VERSION);
       goto HELP_DIALOG;
+    case ACTION_HELP_DSP_ENGINE:
+      help_file = g_strconcat (BST_PATH_DOCS, "/engine-mplan.markup", NULL);
+      help_title = g_strdup (help_file);
+      goto HELP_DIALOG;
+    case ACTION_HELP_PLUGIN_DEVEL:
+      help_file = g_strconcat (BST_PATH_DOCS, "/plugin-devel.markup", NULL);
+      help_title = g_strdup (help_file);
+      goto HELP_DIALOG;
+    case ACTION_HELP_DEVELOPMENT:
+      help_file = g_strconcat (BST_PATH_DOCS, "/beast-index.markup#development", NULL);
+      help_title = g_strdup (help_file);
+      goto HELP_DIALOG;
     HELP_DIALOG:
       if (!bst_help_dialogs[action - ACTION_HELP_FIRST])
         {
@@ -906,10 +986,7 @@ app_action_exec (gpointer data,
                                                                          NULL,
                                                                          GXK_DIALOG_HIDE_ON_DELETE | GXK_DIALOG_DELETE_BUTTON,
                                                                          help_title, sctext);
-          g_object_set (bst_help_dialogs[action - ACTION_HELP_FIRST],
-                        "default_width", 560,
-                        "default_height", 640,
-                        NULL);
+          gxk_dialog_set_sizes (GXK_DIALOG (bst_help_dialogs[action - ACTION_HELP_FIRST]), 500, 400, 560, 640);
         }
       g_free (help_file);
       g_free (help_title);
@@ -919,6 +996,18 @@ app_action_exec (gpointer data,
     case ACTION_HELP_ABOUT:
       beast_show_about_box ();
       break;
+    case ACTION_URL_HELP_DESK:
+      gxk_show_url ("http://beast.gtk.org/wiki:HelpDesk");
+      break;
+    case ACTION_URL_BEAST_SITE:
+      gxk_show_url ("http://beast.gtk.org/");
+      break;
+    case ACTION_URL_ONLINE_SYNTHESIZERS:
+      gxk_show_url ("http://beast.gtk.org/browse-bse-files.html");
+      break;
+    case ACTION_URL_ONLINE_DEMOS:
+      gxk_show_url ("http://beast.gtk.org/browse-bse-files.html");
+      break;
     default:
       g_assert_not_reached ();
       break;
@@ -982,11 +1071,18 @@ app_action_check (gpointer data,
     case BST_ACTION_SAVE_INSTRUMENT:
       super = bst_app_get_current_super (self);
       return BSE_IS_CSYNTH (super) && !bse_project_is_active (self->project);
-    case ACTION_HELP_ABOUT:
-    case ACTION_HELP_FAQ:
-    case ACTION_HELP_GSL_PLAN:
+    case ACTION_HELP_INDEX:
     case ACTION_HELP_RELEASE_NOTES:
     case ACTION_HELP_QUICK_START:
+    case ACTION_HELP_FAQ:
+    case ACTION_HELP_DSP_ENGINE:
+    case ACTION_HELP_PLUGIN_DEVEL:
+    case ACTION_HELP_DEVELOPMENT:
+    case ACTION_HELP_ABOUT:
+    case ACTION_URL_HELP_DESK:
+    case ACTION_URL_BEAST_SITE:
+    case ACTION_URL_ONLINE_SYNTHESIZERS:
+    case ACTION_URL_ONLINE_DEMOS:
       return TRUE;
     case BST_ACTION_EXIT:
       /* abuse generic "Exit" update to sync Tools menu items */
