@@ -39,6 +39,9 @@ typedef enum {
   BST_QUANTIZE_TACT		= 65535
 } BstQuantizationType;
 
+
+typedef struct _BstKeyBinding BstKeyBinding;
+
 /* choose IDs that are unlikely to clash with category IDs */
 #define BST_GENERIC_ROLL_TOOL_FIRST	(G_MAXINT - 100000)
 typedef enum /*< skip >*/
@@ -86,8 +89,8 @@ typedef enum /*< skip >*/
 
 
 /* --- i18n and gettext helpers --- */
-#define _(str)	dgettext (NULL, str)
-#define T_(str)	dgettext (NULL, str)
+#define _DOMAIN (NULL)
+#define _(str)	dgettext (_DOMAIN, str)
 #define N_(str)	(str)
 
 
