@@ -68,13 +68,12 @@ bst_file_dialog_init (BstFileDialog *self)
   
   /* configure self */
   g_object_set (self,
-		"default_width", 500,
-		"default_height", 450,
 		"flags", (GXK_DIALOG_HIDE_ON_DELETE |
                           GXK_DIALOG_PRESERVE_STATE |
 			  GXK_DIALOG_POPUP_POS |
 			  GXK_DIALOG_MODAL),
 		NULL);
+  gxk_dialog_set_sizes (GXK_DIALOG (self), -1, -1, 500, 450);
   g_object_set (main_box,
 		"homogeneous", FALSE,
 		"spacing", 0,
@@ -104,7 +103,7 @@ bst_file_dialog_init (BstFileDialog *self)
   self->fpage = gxk_file_selection_split (self->fs, &bbox);
   g_object_ref (self->fpage);
   gtk_container_remove (GTK_CONTAINER (self->fpage->parent), self->fpage);
-  gxk_notebook_add_page (GTK_NOTEBOOK (self->notebook), self->fpage, "File Selection", TRUE);
+  gxk_notebook_append (GTK_NOTEBOOK (self->notebook), self->fpage, "File Selection", TRUE);
   g_object_unref (self->fpage);
   
   /* sample selection tree */
@@ -115,7 +114,7 @@ bst_file_dialog_init (BstFileDialog *self)
 			      "border_width", 5,
 			      "shadow_type", GTK_SHADOW_IN,
 			      NULL);
-  gxk_notebook_add_page (GTK_NOTEBOOK (self->notebook), self->spage, "Sample Selection", TRUE);
+  gxk_notebook_append (GTK_NOTEBOOK (self->notebook), self->spage, "Sample Selection", TRUE);
   self->file_store = bst_file_store_create ();
   smodel = gtk_tree_model_sort_new_with_model (self->file_store);
   self->tview = g_object_new (GTK_TYPE_TREE_VIEW,
@@ -156,7 +155,7 @@ bst_file_dialog_init (BstFileDialog *self)
 				 0.0, "Filename", NULL,
 				 NULL, self, G_CONNECT_SWAPPED);
   
-  /* pack seperator and buttons */
+  /* pack separator and buttons */
   gtk_box_pack_end (GTK_BOX (main_box), bbox, FALSE, TRUE, 0);
   gtk_box_pack_end (GTK_BOX (main_box),
 		    g_object_new (GTK_TYPE_HSEPARATOR,
@@ -418,7 +417,7 @@ bst_file_dialog_open_project (BstFileDialog *self,
 			      const gchar   *file_name)
 {
   SfiProxy project = bse_server_use_new_project (BSE_SERVER, file_name);
-  BseErrorType error = bse_project_restore_from_file (project, file_name);
+  BseErrorType error = bst_project_restore_from_file (project, file_name);
 
   if (error)
     bst_status_eprintf (error, _("Opening project `%s'"), file_name);
@@ -457,7 +456,7 @@ bst_file_dialog_merge_project (BstFileDialog *self,
 			       const gchar   *file_name)
 {
   SfiProxy project = bse_item_use (self->proxy);
-  BseErrorType error = bse_project_restore_from_file (project, file_name);
+  BseErrorType error = bst_project_restore_from_file (project, file_name);
 
   bst_status_eprintf (error, _("Merging project `%s'"), file_name);
 
@@ -595,7 +594,7 @@ bst_file_dialog_merge_effect (BstFileDialog *self,
                               const gchar   *file_name)
 {
   SfiProxy project = bse_item_use (self->proxy);
-  BseErrorType error = bse_project_restore_from_file (project, file_name);
+  BseErrorType error = bst_project_restore_from_file (project, file_name);
 
   bst_status_eprintf (error, _("Merging effect `%s'"), file_name);
 
@@ -671,7 +670,7 @@ bst_file_dialog_merge_instrument (BstFileDialog *self,
                                   const gchar   *file_name)
 {
   SfiProxy project = bse_item_use (self->proxy);
-  BseErrorType error = bse_project_restore_from_file (project, file_name);
+  BseErrorType error = bst_project_restore_from_file (project, file_name);
 
   bst_status_eprintf (error, _("Merging instrument `%s'"), file_name);
 
@@ -756,6 +755,36 @@ bst_file_dialog_load_wave (BstFileDialog *self,
   return TRUE;
 }
 
+GtkWidget*
+bst_file_dialog_create (void)
+{
+  BstFileDialog *self = g_object_new (BST_TYPE_FILE_DIALOG, NULL);
+  bst_file_dialog_set_mode (self, NULL,
+			    BST_FILE_DIALOG_SELECT_FILE,
+			    "File Selector", 0);
+  return GTK_WIDGET (self);
+}
+
+void
+bst_file_dialog_setup (GtkWidget        *widget,
+                       gpointer          parent_widget,
+                       const gchar      *title,
+                       const gchar      *search_path)
+{
+  BstFileDialog *self = BST_FILE_DIALOG (widget);
+  gchar *path;
+  bst_file_dialog_set_mode (self, parent_widget,
+                            BST_FILE_DIALOG_SELECT_FILE,
+                            title, 0);
+  g_free (self->search_path);
+  self->search_path = g_strdup (search_path);
+  self->search_filter = "*";
+  path = g_strconcat (self->search_path, G_DIR_SEPARATOR_S, self->search_filter, NULL);
+  gtk_file_selection_complete (self->fs, path);
+  g_free (path);
+  tree_viewable_changed (self);
+}
+
 typedef struct {
   BstFileDialogHandler handler;
   gpointer             data;
@@ -783,7 +812,7 @@ bst_file_dialog_set_handler (BstFileDialog    *self,
   BstFileDialogData *data = g_new0 (BstFileDialogData, 1);
 
   g_return_if_fail (GTK_WIDGET_VISIBLE (self));
-
+  
   data->handler = handler;
   data->data = handler_data;
   data->destroy = destroy;
