@@ -23,6 +23,7 @@
 #include "bstgconfig.h"
 #include "bstkeybindings.h"
 #include "bstskinconfig.h"
+#include "bstmsgabsorb.h"
 #include "bstusermessage.h"
 #include "bstparam.h"
 #include "bstpreferences.h"
@@ -53,6 +54,7 @@ static gboolean     arg_force_xkb = FALSE;
 static gboolean     register_core_plugins = TRUE;
 static gboolean     register_ladspa_plugins = TRUE;
 static gboolean     register_scripts = TRUE;
+static gboolean     may_auto_update_bse_rc_file = TRUE;
 
 
 /* --- functions --- */
@@ -85,7 +87,6 @@ main (int   argc,
   SfiRec *bseconfig;
   gchar *string;
   GSource *source;
-  gboolean save_rc_file = TRUE;
   guint i;
 
   /* initialize i18n */
@@ -114,7 +115,8 @@ main (int   argc,
   /* initialize Gtk+ and go into threading mode */
   bst_early_parse_args (&argc, &argv, bseconfig);
   gtk_init (&argc, &argv);
-  g_set_prgname ("beast");	/* overriding Gdk's program name */
+  g_set_prgname ("beast");	        /* overriding Gdk's program name */
+  g_set_application_name (_("BEAST"));  /* User visible name */       
   GDK_THREADS_ENTER ();
 
   /* initialize Gtk+ Extension Kit */
@@ -142,7 +144,8 @@ main (int   argc,
   _bst_init_params ();
   _bst_gconfig_init ();
   _bst_skin_config_init ();
-
+  _bst_msg_absorb_config_init ();
+  
   /* parse rc file */
   bst_splash_update_item (beast_splash, _("RC Files"));
   bst_preferences_load_rc_files();
@@ -164,7 +167,7 @@ main (int   argc,
   bse_init_async (&argc, &argv, bseconfig);
   sfi_rec_unref (bseconfig);
   sfi_glue_context_push (bse_init_glue_context ("BEAST"));
-  source = g_source_simple (G_PRIORITY_HIGH,
+  source = g_source_simple (GDK_PRIORITY_EVENTS, // G_PRIORITY_HIGH - 100,
 			    (GSourcePending) sfi_glue_context_pending,
 			    (GSourceDispatch) sfi_glue_context_dispatch,
 			    NULL, NULL, NULL);
@@ -292,7 +295,7 @@ main (int   argc,
 	    }
 	}
       project = bse_server_use_new_project (BSE_SERVER, argv[i]);
-      error = bse_project_restore_from_file (project, argv[i]);
+      error = bst_project_restore_from_file (project, argv[i]);
       
       if (!error)
 	{
@@ -322,9 +325,10 @@ main (int   argc,
 
   /* fire up release notes dialog
    */
+  gboolean update_rc_files = FALSE;
   if (!BST_RC_VERSION || strcmp (BST_RC_VERSION, BST_VERSION))
     {
-      save_rc_file = TRUE;
+      update_rc_files = TRUE;
       bst_app_show_release_notes (app);
       bst_gconfig_set_rc_version (BST_VERSION);
     }
@@ -332,7 +336,7 @@ main (int   argc,
   /* release splash grab, install message dialog handler */
   gtk_widget_hide (beast_splash);
   bst_splash_release_grab (beast_splash);
-  sfi_log_set_handler (bst_user_message_log_handler);
+  sfi_log_set_thread_handler (bst_user_message_log_handler);
   /* away into the main loop */
   while (bst_main_loop_running)
     {
@@ -341,18 +345,12 @@ main (int   argc,
       g_main_iteration (TRUE);
       GDK_THREADS_ENTER ();
     }
-  sfi_log_set_handler (NULL);
+  sfi_log_set_thread_handler (NULL);
   
-  /* stop everything playing
-   */
-  // bse_heart_reset_all_attach ();
-  
-  /* take down GUI leftovers
-   */
+  /* take down GUI leftovers */
   bst_user_messages_kill ();
   
-  /* perform necessary cleanup cycles
-   */
+  /* perform necessary cleanup cycles */
   GDK_THREADS_LEAVE ();
   while (g_main_iteration (FALSE))
     {
@@ -363,9 +361,10 @@ main (int   argc,
   GDK_THREADS_ENTER ();
   
   /* save BSE configuration */
-  bse_server_save_preferences (BSE_SERVER);
-  if (save_rc_file)
+  if (update_rc_files && !bst_preferences_saved())
     {
+      if (may_auto_update_bse_rc_file)
+        bse_server_save_preferences (BSE_SERVER);
       /* save BEAST configuration and accelerator map */
       gchar *file_name = BST_STRDUP_RC_FILE ();
       BseErrorType error = bst_rc_dump (file_name);
@@ -374,10 +373,6 @@ main (int   argc,
       g_free (file_name);
     }
   
-  /* remove pcm devices
-   */
-  // bse_heart_unregister_all_devices ();
-  
   /* perform necessary cleanup cycles
    */
   GDK_THREADS_LEAVE ();
@@ -416,6 +411,7 @@ bst_early_parse_args (int    *argc_p,
   if (envar)
     sfi_debug_deny (envar);
 
+  gboolean initialize_bse_and_exit = FALSE;
   for (i = 1; i < argc; i++)
     {
       if (strncmp (argv[i], "-:", 2) == 0)
@@ -564,6 +560,48 @@ bst_early_parse_args (int    *argc_p,
           g_free (freeme);
 	  exit (0);
 	}
+      else if (strcmp ("--bse-latency", argv[i]) == 0 ||
+               strncmp ("--bse-latency=", argv[i], 14) == 0)
+        {
+          gchar *equal = argv[i] + 13;
+          may_auto_update_bse_rc_file = FALSE;
+          if (*equal != '=')
+            i++;
+          /* leave args for BSE */
+        }
+      else if (strcmp ("--bse-mixing-freq", argv[i]) == 0 ||
+               strncmp ("--bse-mixing-freq=", argv[i], 18) == 0)
+        {
+          gchar *equal = argv[i] + 17;
+          may_auto_update_bse_rc_file = FALSE;
+          if (*equal != '=')
+            i++;
+          /* leave args for BSE */
+        }
+      else if (strcmp ("--bse-control-freq", argv[i]) == 0 ||
+               strncmp ("--bse-control-freq=", argv[i], 19) == 0)
+        {
+          gchar *equal = argv[i] + 18;
+          may_auto_update_bse_rc_file = FALSE;
+          if (*equal != '=')
+            i++;
+          /* leave args for BSE */
+        }
+      else if (strcmp ("--bse-driver-list", argv[i]) == 0)
+        {
+          initialize_bse_and_exit = TRUE;
+          /* leave args for BSE */
+        }
+      else if (strcmp ("-p", argv[i]) == 0)
+        {
+          /* modify args for BSE */
+          argv[i] = "--bse-pcm-driver";
+        }
+      else if (strcmp ("-m", argv[i]) == 0)
+        {
+          /* modify args for BSE */
+          argv[i] = "--bse-midi-driver";
+        }
     }
   gxk_param_set_devel_tips (bst_developer_hints);
 
@@ -576,6 +614,12 @@ bst_early_parse_args (int    *argc_p,
           argv[i] = NULL;
       }
   *argc_p = e;
+
+  if (initialize_bse_and_exit)
+    {
+      bse_init_async (argc_p, argv_p, bseconfig);
+      exit (0);
+    }
 }
 
 static void G_GNUC_NORETURN
@@ -596,7 +640,6 @@ bst_exit_print_version (void)
   if (c)
     g_print (", %s", c);
   g_print (", GTK+ %u.%u.%u", gtk_major_version, gtk_minor_version, gtk_micro_version);
-  g_print (", GdkPixbuf");
 #ifdef BST_WITH_XKB
   g_print (", XKBlib");
 #endif
@@ -622,7 +665,7 @@ bst_exit_print_version (void)
   g_print ("You may redistribute copies of BEAST under the terms of\n");
   g_print ("the GNU General Public License which can be found in the\n");
   g_print ("BEAST source package. Sources, examples and contact\n");
-  g_print ("information are available at http://beast.gtk.org\n");
+  g_print ("information are available at http://beast.gtk.org/.\n");
   g_free (freeme);
   exit (0);
 }
@@ -641,6 +684,22 @@ bst_print_blurb (void)
   g_print ("  -h, --help              show this help message\n");
   g_print ("  -v, --version           print version and file paths\n");
   g_print ("  --display=DISPLAY       X server for the GUI; see X(1)\n");
+  g_print ("  --bse-latency=USECONDS  specify synthesis latency in milliseconds\n");
+  g_print ("  --bse-mixing-freq=FREQ  specify synthesis mixing frequency in Hz \n");
+  g_print ("  --bse-control-freq=FREQ specify control frequency in Hz\n");
+  g_print ("  --bse-pcm-driver DRIVERCONF\n");
+  g_print ("  -p DRIVERCONF           try to use the PCM driver DRIVERCONF, multiple\n");
+  g_print ("                          options may be supplied to try a variety of\n");
+  g_print ("                          drivers, unless -p auto is given, only the\n");
+  g_print ("                          drivers listed by -p options are used; each\n");
+  g_print ("                          DRIVERCONF consists of a driver name and an\n");
+  g_print ("                          optional comma seperated list of arguments,\n");
+  g_print ("                          e.g.: -p oss=/dev/dsp2,rw\n");
+  g_print ("  --bse-midi-driver DRIVERCONF\n");
+  g_print ("  -m DRIVERCONF           try to use the MIDI driver DRIVERCONF, multiple\n");
+  g_print ("                          options may be specified similarly to the\n");
+  g_print ("                          option handling for --bse-pcm-driver\n");
+  g_print ("  --bse-driver-list       list available PCM and MIDI drivers\n");
   g_print ("Development Options:\n");
   g_print ("  --debug=KEYS            enable specific debugging messages\n");
   g_print ("  --no-debug=KEYS         disable specific debugging messages\n");
@@ -677,41 +736,51 @@ beast_show_about_box (void)
    */
   const gchar *contributors[] = {
     /* graphics */
+    //"Artem Popov",
     "Cyria Arweiler",
-    "Tuomas Kuosmanen",
     "Jakub Steiner",
+    "Tuomas Kuosmanen",
     "Ville P\xc3\xa4tsi",
     /* general code and fixes */
-    "Sam Hocevar",
-    "Nedko Arnaudov",
     "Alper Ersoy",
     "Ben Collver",
-    "Tim Janik",
-    "Stefan Westerfeld",
+    "Nedko Arnaudov",
     "Rasmus Kaj",
+    "Sam Hocevar",
+    "Stefan Westerfeld",
+    "Sven Herzberg",
+    "Tim Janik",
     /* plugins */
+    "\xd0\x90\xd1\x80\xd1\x82\xd0\xb5\xd0\xbc \xd0\x9f\xd0\xbe\xd0\xbf\xd0\xbe\xd0\xb2", /* "Artem Popov", */
     "David A. Bartold",
     //"Tim Janik",
     //"Stefan Westerfeld",
     /* translations */
     "Adam Weinberger",
     "Alexandre Prokoudine",
+    //"Artem Popov",
     "Christian Neumair",
     "Christian Rose",
     "Christophe Merlet",
     "Danilo Segan",
+    "David Lodge",
+    "Dirk Janik",
     "Duarte Loreto",
     "Dulmandakh Sukhbaatar",
     "Francisco Javier F. Serrador",
     "Gareth Owen",
     "Ismael Andres Rubio Rojas",
+    "Jorge Gonzalez",
     "Kees van den Broek",
     "Kostas Papadimas",
     "Laurent Dhima",
     "Metin Amiroff",
     "Miloslav Trmac",
+    "Moritz Mekelburger",
+    "Petrecca Michele",
     "Raphael Higino",
     "Robert Sedak",
+    "Satoru SATOH",
     "Tino Meinen",
     "Vincent van Adrighem",
     "Xavier Conde Rueda",
