@@ -75,13 +75,14 @@ param_choice_create (GxkParam    *param,
     widget = g_object_new (GXK_TYPE_MENU_BUTTON,
                            "visible", TRUE,
                            "mode", variant,
+                           "can-focus", variant == GXK_MENU_BUTTON_COMBO_MODE,
                            NULL);
 
   menu = g_object_new (GTK_TYPE_MENU, NULL);
   for (i = 0; i < cvalues.n_values; i++)
     {
       GtkWidget *item = gtk_menu_item_new_with_label (cvalues.values[i].choice_label);
-      gxk_widget_set_latent_tooltip (item, cvalues.values[i].choice_blurb);
+      gxk_widget_set_tooltip (item, cvalues.values[i].choice_blurb);
       gtk_widget_show (item);
       g_object_connect (item, "signal::activate", param_choice_item_activated, widget, NULL);
       g_object_set_qdata (G_OBJECT (item), quark_param_choice_values, (gpointer) &cvalues.values[i]);
@@ -139,25 +140,25 @@ param_choice_update (GxkParam  *param,
 }
 
 static GxkParamEditor param_choice1 = {
-  { "option-button",    N_("Popup Options"), },
-  { G_TYPE_STRING,      "SfiChoice", },
-  { NULL,       +8,     TRUE, },        /* options, rating, editing */
-  param_choice_create,  param_choice_update,    GXK_MENU_BUTTON_OPTION_MODE
-};
-static GxkParamEditor param_choice2 = {
   { "combo-button",     N_("Drop Down Combo"), },
   { G_TYPE_STRING,      "SfiChoice", },
-  { NULL,       +7,     TRUE, },        /* options, rating, editing */
+  { NULL,       +8,     TRUE, },        /* options, rating, editing */
   param_choice_create,  param_choice_update,    GXK_MENU_BUTTON_COMBO_MODE
 };
-static GxkParamEditor param_choice3 = {
+static GxkParamEditor param_choice2 = {
   { "tool-button",      N_("Drop Down Button"), },
   { G_TYPE_STRING,      "SfiChoice", },
-  { NULL,       +6,     TRUE, },        /* options, rating, editing */
+  { NULL,       +7,     TRUE, },        /* options, rating, editing */
   param_choice_create,  param_choice_update,    GXK_MENU_BUTTON_TOOL_MODE
 };
+static GxkParamEditor param_choice3 = {
+  { "choice-button",    N_("Popup Options"), },
+  { G_TYPE_STRING,      "SfiChoice", },
+  { NULL,       +6,     TRUE, },        /* options, rating, editing */
+  param_choice_create,  param_choice_update,    GXK_MENU_BUTTON_OPTION_MODE
+};
 static GxkParamEditor param_choice4 = {
-  { "option-menu",      N_("Standard Option Menu"), },
+  { "choice-menu",      N_("Standard Option Menu"), },
   { G_TYPE_STRING,      "SfiChoice", },
   { NULL,       +5,     TRUE, },        /* options, rating, editing */
   param_choice_create,  param_choice_update,    CHOICE_PARAM_OPTION_MENU
