@@ -64,7 +64,9 @@ param_get_gmask_container (GtkWidget *parent,
   return container;
 }
 
-static const GxkParamEditorSizes param_editor_sizes = {
+static const GxkParamEditorSizes param_editor_homogeneous_sizes = {
+  FALSE,        /* may_resize */
+  FALSE,        /* request_fractions */
   2, 8,         /* char */
   2, 8,         /* uchar */
   2, 8,         /* int */
@@ -83,6 +85,7 @@ bst_param_create_gmask (GxkParam    *param,
                         const gchar *editor_name,
                         GtkWidget   *parent)
 {
+  SfiProxy proxy = bst_param_get_proxy (param);
   const gchar *group;
   GtkWidget *xframe, *action, *prompt = NULL;
   BstGMask *gmask;
@@ -92,7 +95,7 @@ bst_param_create_gmask (GxkParam    *param,
   g_return_val_if_fail (GXK_IS_PARAM (param), NULL);
   g_return_val_if_fail (GTK_IS_CONTAINER (parent), NULL);
   
-  gxk_param_set_sizes (param_size_group, BST_GCONFIG (size_group_input_fields) ? &param_editor_sizes : NULL);
+  gxk_param_set_sizes (param_size_group, BST_GCONFIG (size_group_input_fields) ? &param_editor_homogeneous_sizes : NULL);
   group = sfi_pspec_get_group (param->pspec);
   parent = param_get_gmask_container (parent, group ? g_quark_from_string (group) : 0);
   
@@ -105,9 +108,12 @@ bst_param_create_gmask (GxkParam    *param,
   
   if (GTK_IS_TOGGLE_BUTTON (action))
     {
-      /* there's a prompt widget inside the button already, sneak in xframe */
-      gtk_widget_reparent (GTK_BIN (action)->child, xframe);
-      g_object_set (xframe, "parent", action, "steal_button", TRUE, NULL);
+      /* if there's a prompt widget inside the button already, sneak in xframe */
+      if (GTK_BIN (action)->child)
+        {
+          gtk_widget_reparent (GTK_BIN (action)->child, xframe);
+          g_object_set (xframe, "parent", action, "steal_button", TRUE, NULL);
+        }
     }
   else
     {
@@ -122,9 +128,20 @@ bst_param_create_gmask (GxkParam    *param,
 
   expand_action = !prompt || gxk_widget_check_option (action, "hexpand");
   gmask = bst_gmask_form (parent, action, expand_action ? BST_GMASK_BIG : BST_GMASK_INTERLEAVE);
+  if (BSE_IS_SOURCE (proxy) && sfi_pspec_check_option (param->pspec, "automate"))
+    {
+      GtkWidget *automation = gxk_param_create_editor (param, "automation");
+      if (prompt)
+        {
+          GtkBox *hbox = g_object_new (GTK_TYPE_HBOX, "visible", TRUE, NULL);
+          gtk_box_pack_start (hbox, gtk_widget_get_toplevel (prompt), FALSE, TRUE, 0);
+          gtk_box_pack_end (hbox, automation, FALSE, TRUE, 0);
+        }
+      else
+        prompt = automation;
+    }
   if (prompt)
     bst_gmask_set_prompt (gmask, prompt);
-  
   if (sfi_pspec_check_option (param->pspec, "dial"))
     {
       GtkWidget *dial = gxk_param_create_editor (param, "dial");
@@ -146,6 +163,18 @@ bst_param_create_gmask (GxkParam    *param,
   return gmask;
 }
 
+/* --- value binding --- */
+GxkParam*
+bst_param_new_value (GParamSpec          *pspec,
+                     GxkParamValueNotify  notify,
+                     gpointer             notify_data)
+{
+  GxkParam *param = gxk_param_new_value (pspec, notify, notify_data);
+  if (param)
+    gxk_param_set_size_group (param, param_size_group);
+  return param;
+}
+
 /* --- proxy binding --- */
 static void
 proxy_binding_set_value (GxkParam     *param,
@@ -195,12 +224,42 @@ proxy_binding_destroy (GxkParam *param)
     }
 }
 
+static void
+proxy_binding_start_grouping (GxkParam *param)
+{
+  SfiProxy proxy = param->bdata[0].v_long;
+  gchar *ustr = g_strconcat ("Modify ", g_param_spec_get_nick (param->pspec), NULL);
+  if (proxy)
+    bse_item_group_undo (proxy, ustr);
+  g_free (ustr);
+}
+
+static void
+proxy_binding_stop_grouping (GxkParam *param)
+{
+  SfiProxy proxy = param->bdata[0].v_long;
+  if (proxy)
+    bse_item_ungroup_undo (proxy);
+}
+
+static gboolean
+proxy_binding_check_writable (GxkParam *param)
+{
+  SfiProxy proxy = param->bdata[0].v_long;
+  if (proxy)
+    return bse_item_editable_property (proxy, param->pspec->name);
+  else
+    return FALSE;
+}
+
 static GxkParamBinding proxy_binding = {
-  2, NULL,
-  proxy_binding_set_value,
-  proxy_binding_get_value,
-  proxy_binding_destroy,
-  NULL,	/* check_writable */
+  .n_data_fields        = 2,
+  .set_value            = proxy_binding_set_value,
+  .get_value            = proxy_binding_get_value,
+  .destroy              = proxy_binding_destroy,
+  .check_writable       = proxy_binding_check_writable,
+  .start_grouping       = proxy_binding_start_grouping,
+  .stop_grouping        = proxy_binding_stop_grouping,
 };
 
 GxkParam*
@@ -324,9 +383,11 @@ bst_param_xframe_check_button (GxkParam *param,
 #include "bstparam-note-sequence.c"
 #include "bstparam-note-spinner.c"
 #include "bstparam-proxy.c"
+#include "bstparam-item-seq.c"
 #include "bstparam-scale.c"
 #include "bstparam-searchpath.c"
 #include "bstparam-time.c"
+#include "bstparam-automation.c"
 void
 _bst_init_params (void)
 {
@@ -344,6 +405,8 @@ _bst_init_params (void)
   gxk_param_register_editor (&param_note_sequence, NULL);
   gxk_param_register_editor (&param_note_spinner, NULL);
   gxk_param_register_editor (&param_proxy, NULL);
+  gxk_param_register_editor (&param_item_seq, NULL);
+  gxk_param_register_editor (&param_automation, NULL);
   gxk_param_register_editor (&param_scale1, NULL);
   gxk_param_register_editor (&param_scale2, NULL);
   gxk_param_register_editor (&param_scale3, NULL);
