@@ -64,7 +64,7 @@ static const GxkStockAction piano_edit_actions[] = {
 };
 static const GxkStockAction piano_edit_undo[] = {
   { N_("Undo"),         "<ctrl>Z",   N_("Undo last editing step"),                          ACTION_UNDO,       BST_STOCK_UNDO },
-  { N_("Redo"),         "<ctrl>R",   N_("Redo the last undone editing step"),               ACTION_REDO,       BST_STOCK_REDO },
+  { N_("Redo"),         "<ctrl>Y",   N_("Redo the last undone editing step"),               ACTION_REDO,       BST_STOCK_REDO },
 };
 static const GxkStockAction piano_clear_undo[] = {
   { N_("_Clear Undo"),          NULL,           NULL,
@@ -116,11 +116,10 @@ eparam_changed (gpointer  data,
 static void
 bst_part_dialog_init (BstPartDialog *self)
 {
-  GtkWidget *eb, *child, *box;
+  GtkWidget *eb, *box;
   GtkRange *srange;
   BseCategorySeq *cseq;
   GxkActionList *al1;
-  GtkPaned *paned;
   GtkObject *adjustment;
   GtkAdjustment *adj;
   GParamSpec *pspec;
@@ -129,11 +128,10 @@ bst_part_dialog_init (BstPartDialog *self)
   /* configure self */
   g_object_set (self,
                 "name", "BEAST-PartDialog",
-                "default_width", 600,
-		"default_height", 450,
-		"flags", GXK_DIALOG_STATUS_SHELL,
+                "flags", GXK_DIALOG_STATUS_BAR,
 		NULL);
-
+  gxk_dialog_set_sizes (GXK_DIALOG (self), 640, 400, 1005, 650);
+  
   /* radget-complete GUI */
   radget = gxk_radget_create ("beast", "piano-roll-box", NULL);
   gtk_container_add (GTK_CONTAINER (GXK_DIALOG (self)->vbox), radget);
@@ -155,17 +153,6 @@ bst_part_dialog_init (BstPartDialog *self)
   gxk_action_list_sort (al1);
   gxk_widget_publish_action_list (self, "part-scripts", al1);
 
-  /* FIXME: use paned child-properties after gtk+-2.4 */
-  paned = gxk_radget_find (radget, "piano-paned");
-  child = g_object_ref (paned->child1);
-  gtk_container_remove (GTK_CONTAINER (paned), child);
-  gtk_paned_pack1 (paned, child, TRUE, TRUE);
-  g_object_unref (child);
-  child = g_object_ref (paned->child2);
-  gtk_container_remove (GTK_CONTAINER (paned), child);
-  gtk_paned_pack2 (paned, child, FALSE, TRUE);
-  g_object_unref (child);
-
   BstGrowBar *grow_bar = gxk_radget_find (radget, "piano-roll-hgrow-bar");
 
   /* piano roll */
