@@ -19,10 +19,13 @@
 #include "bstparamview.h"
 #include "bsttrackview.h"
 #include "bstpartview.h"
-#include "bstsongbusview.h"
+#include "bstbusmixer.h"
+#include "bstbusview.h"
 #include "bstwaveview.h"
 #include "bstrackview.h"
 #include "bstsnetrouter.h"
+#include "bstgconfig.h"
+#include <string.h>
 
 enum {
   PROP_0,
@@ -139,26 +142,41 @@ bst_super_shell_finalize (GObject *object)
   G_OBJECT_CLASS (bst_super_shell_parent_class)->finalize (object);
 }
 
-static void
-bst_super_shell_name_set (BstSuperShell *self)
+void
+bst_super_shell_update_label (BstSuperShell *self)
 {
+  g_return_if_fail (BST_IS_SUPER_SHELL (self));
+
   GtkWidget *widget = GTK_WIDGET (self);
-  if (widget->parent && GTK_IS_NOTEBOOK (widget->parent))
+  GtkWidget *tab = gxk_notebook_descendant_get_tab (widget);
+  if (tab && self->super)
     {
-      gchar *name = NULL;
+      /* discriminate super */
+      const gchar *stock, *name = bse_item_get_name (self->super);
+      gchar *tip;
       if (BSE_IS_WAVE_REPO (self->super))
-        name = g_strdup (_("Waves"));
-      else if (self->super)
-        name = g_strconcat (bse_item_get_type (self->super),
-                            ":\n",
-                            bse_item_get_name (self->super),
-                            NULL);
-      widget = gtk_notebook_get_tab_label (GTK_NOTEBOOK (widget->parent), widget);
-      if (widget)
-	g_object_set (widget,
-                      "label", name ? name : "BstSuperShell",
-                      NULL);
-      g_free (name);
+        {
+          name = _("Waves");
+          tip = g_strdup (_("Wave Repository"));
+          stock = BST_STOCK_MINI_WAVE_REPO;
+        }
+      else if (BSE_IS_SONG (self->super))
+        {
+          tip = g_strdup_printf (_("Song: %s"), name);
+          stock = BST_STOCK_MINI_SONG;
+        }
+      else if (BSE_IS_MIDI_SYNTH (self->super))
+        {
+          tip = g_strdup_printf (_("MIDI Synthesizer: %s"), name);
+          stock = BST_STOCK_MINI_MIDI_SYNTH;
+        }
+      else
+        {
+          tip = g_strdup_printf (_("Synthesizer: %s"), name);
+          stock = BST_STOCK_MINI_CSYNTH;
+        }
+      gxk_notebook_change_tabulator (tab, name, stock, tip);
+      g_free (tip);
     }
 }
 
@@ -175,7 +193,7 @@ bst_super_shell_set_super (BstSuperShell *self,
       if (self->super)
 	{
           bse_proxy_disconnect (self->super,
-                                "any_signal::property-notify::uname", bst_super_shell_name_set, self,
+                                "any_signal::property-notify::uname", bst_super_shell_update_label, self,
                                 NULL);
           gtk_container_foreach (GTK_CONTAINER (self), (GtkCallback) gtk_widget_destroy, NULL);
 	  bse_item_unuse (self->super);
@@ -185,20 +203,18 @@ bst_super_shell_set_super (BstSuperShell *self,
 	{
 	  bse_item_use (self->super);
           bse_proxy_connect (self->super,
-                             "swapped_signal::property-notify::uname", bst_super_shell_name_set, self,
+                             "swapped_signal::property-notify::uname", bst_super_shell_update_label, self,
                              NULL);
           super_shell_add_views (self);
-	  bst_super_shell_name_set (self);
+	  bst_super_shell_update_label (self);
 	}
     }
 }
 
-void
-bst_super_shell_update_label (BstSuperShell *self)
+GtkWidget*
+bst_super_shell_create_label (BstSuperShell *super_shell)
 {
-  g_return_if_fail (BST_IS_SUPER_SHELL (self));
-
-  bst_super_shell_name_set (self);
+  return gxk_notebook_create_tabulator ("SuperShell", NULL, NULL);
 }
 
 static void
@@ -209,31 +225,25 @@ super_shell_build_song (BstSuperShell *self,
 
   gtk_notebook_append_page (notebook,
                             bst_track_view_new (song),
-			    g_object_new (GTK_TYPE_LABEL, "visible", TRUE,
-                                          "label", _("Tracks"),
-                                          NULL));
+                            gxk_notebook_create_tabulator (_("Tracks"), BST_STOCK_TRACKS, _("Tracks contain instrument definitions and parts with notes")));
+  gtk_notebook_append_page (notebook,
+                            bst_bus_mixer_new (song),
+                            gxk_notebook_create_tabulator (_("Mixer"), BST_STOCK_MIXER, _("Mix track outputs, adjust volume and add effects")));
   gtk_notebook_append_page (notebook,
                             bst_param_view_new (song),
-                            g_object_new (GTK_TYPE_LABEL, "visible", TRUE,
-                                          "label", _("Parameters"),
-                                          NULL));
-  gtk_notebook_append_page (notebook,
-                            bst_part_view_new (song),
-                            g_object_new (GTK_TYPE_LABEL, "visible", TRUE,
-                                          "label", _("Parts"),
-                                          NULL));
+                            gxk_notebook_create_tabulator (_("Properties"), BST_STOCK_PROPERTIES, _("Adjust overall song behaviour")));
   if (BST_DBG_EXT)
     gtk_notebook_append_page (notebook,
-                              bst_song_bus_view_new (song),
-                              g_object_new (GTK_TYPE_LABEL, "visible", TRUE,
-                                            "label", _("Busses"),
-                                            NULL));
+                              bst_part_view_new (song),
+                              gxk_notebook_create_tabulator (_("Parts"), BST_STOCK_PART, NULL));
+  if (BST_DBG_EXT)
+    gtk_notebook_append_page (notebook,
+                              bst_bus_view_new (song),
+                              gxk_notebook_create_tabulator (_("Busses"), BST_STOCK_BUS, NULL));
   if (BST_DBG_EXT)
     gtk_notebook_append_page (notebook,
                               gtk_widget_get_toplevel (GTK_WIDGET (bst_snet_router_build_page (song))),
-                              g_object_new (GTK_TYPE_LABEL, "visible", TRUE,
-                                            "label", _("Routing"),
-                                            NULL));
+                              gxk_notebook_create_tabulator (_("Routing"), BST_STOCK_MESH, NULL));
 }
 
 static void
@@ -246,19 +256,15 @@ super_shell_build_snet (BstSuperShell *self,
   if (BST_DBG_EXT && bse_snet_supports_user_synths (snet))
     gtk_notebook_append_page (notebook,
                               gtk_widget_get_toplevel (bst_rack_view_new (snet)),
-                              g_object_new (GTK_TYPE_LABEL, "label", _("Rack"), NULL));
+                              gxk_notebook_create_tabulator (_("Rack"), NULL, NULL));
   if (bse_snet_supports_user_synths (snet) || BST_DBG_EXT)
     gtk_notebook_append_page (notebook,
                               gtk_widget_get_toplevel (GTK_WIDGET (bst_snet_router_build_page (snet))),
-                              g_object_new (GTK_TYPE_LABEL, "visible", TRUE,
-                                            "label", _("Routing"),
-                                            NULL));
+                              gxk_notebook_create_tabulator (_("Routing"), BST_STOCK_MESH, _("Add, edit and connect synthsizer mesh components")));
   param_view = bst_param_view_new (snet);
   gtk_notebook_append_page (notebook,
                             bst_param_view_new (snet),
-                            g_object_new (GTK_TYPE_LABEL, "visible", TRUE,
-                                          "label", _("Parameters"),
-                                          NULL));
+                            gxk_notebook_create_tabulator (_("Properties"), BST_STOCK_PROPERTIES, _("Adjust overall synthesizer behaviour")));
 }
 
 static void
@@ -269,14 +275,10 @@ super_shell_build_wave_repo (BstSuperShell *self,
 
   gtk_notebook_append_page (notebook,
                             bst_wave_view_new (wrepo),
-                            g_object_new (GTK_TYPE_LABEL, "visible", TRUE,
-                                          "label", _("Waves"),
-                                          NULL));
+                            gxk_notebook_create_tabulator (_("Waves"), BST_STOCK_MINI_WAVE_REPO, NULL));
   gtk_notebook_append_page (notebook,
                             bst_param_view_new (wrepo),
-                            g_object_new (GTK_TYPE_LABEL, "visible", TRUE,
-                                          "label", _("Parameters"),
-                                          NULL));
+                            gxk_notebook_create_tabulator (_("Properties"), BST_STOCK_PROPERTIES, NULL));
 }
 
 static GtkNotebook*
