@@ -22,11 +22,14 @@
 #include "bsttrackview.h"
 #include "bstwaveview.h"
 #include "bstpartview.h"
-#include "bstsongbusview.h"
+#include "bstbusmixer.h"
+#include "bstbuseditor.h"
+#include "bstbusview.h"
 #include "bstpianoroll.h"
 #include "bstpatternview.h"
 #include "bsteventroll.h"
 #include "bstgrowbar.h"
+#include "bstdbmeter.h"
 #include <fcntl.h>
 #include <errno.h>
 #include <unistd.h>
@@ -114,9 +117,14 @@ _bst_init_radgets (void)
   gxk_radget_define_widget_type (BST_TYPE_VGROW_BAR);
   gxk_radget_define_widget_type (BST_TYPE_WAVE_VIEW);
   gxk_radget_define_widget_type (BST_TYPE_PART_VIEW);
-  gxk_radget_define_widget_type (BST_TYPE_SONG_BUS_VIEW);
+  gxk_radget_define_widget_type (BST_TYPE_BUS_EDITOR);
+  gxk_radget_define_widget_type (BST_TYPE_BUS_MIXER);
+  gxk_radget_define_widget_type (BST_TYPE_BUS_VIEW);
   gxk_radget_define_widget_type (BST_TYPE_PIANO_ROLL);
   gxk_radget_define_widget_type (BST_TYPE_EVENT_ROLL);
+  gxk_radget_define_widget_type (BST_TYPE_DB_BEAM);
+  gxk_radget_define_widget_type (BST_TYPE_DB_LABELING);
+  gxk_radget_define_widget_type (BST_TYPE_DB_METER);
   gxk_radget_define_widget_type (BST_TYPE_PATTERN_VIEW);
   gxk_radget_define_widget_type (BST_TYPE_ZOOMED_WINDOW);
   text = gxk_zfile_uncompress (BST_RADGETS_STANDARD_SIZE, BST_RADGETS_STANDARD_DATA, G_N_ELEMENTS (BST_RADGETS_STANDARD_DATA));
@@ -181,7 +189,30 @@ bst_stock_register_icon (const gchar    *stock_id,
 }
 
 
-/* --- beast/bsw specific extensions --- */
+/* --- BSE utilities --- */
+BseErrorType
+bst_project_restore_from_file (SfiProxy         project,
+                               const gchar     *file_name)
+{
+  BseErrorType error = bse_project_restore_from_file (project, file_name);
+  /* regardless of how good the restoration worked, try to
+   * keep the resulting project in a GUI usable state.
+   */
+  BseItemSeq *iseq = bse_container_list_children (project);
+  guint i;
+  for (i = 0; i < iseq->n_items; i++)
+    if (BSE_IS_SONG (iseq->items[i]))
+      {
+        /* fixup orphaned parts */
+        bse_song_ensure_track_links (iseq->items[i]);
+        /* songs always need a master bus */
+        bse_song_ensure_master_bus (iseq->items[i]);
+      }
+  return error;
+}
+
+
+/* --- beast/bse specific extensions --- */
 void
 bst_status_eprintf (BseErrorType error,
                     const gchar *message_fmt,
@@ -269,6 +300,92 @@ bst_window_sync_title_to_proxy (gpointer     window,
     }
 }
 
+typedef struct {
+  gboolean       (*handler) (gpointer data);
+  gpointer         data;
+  void           (*free_func) (gpointer data);
+} BackgroundHandler;
+
+static SfiRing *background_handlers1 = NULL;
+static SfiRing *background_handlers2 = NULL;
+
+static gboolean
+bst_background_handlers_timeout (gpointer timeout_data)
+{
+  GDK_THREADS_ENTER();
+  if (background_handlers1 || background_handlers2)
+    {
+      gxk_status_set (GXK_STATUS_PROGRESS, _("Updating View"), NULL);
+      BackgroundHandler *bgh = sfi_ring_pop_head (&background_handlers1);
+      gint prio = 1;
+      if (!bgh)
+        {
+          prio = 2;
+          bgh = sfi_ring_pop_head (&background_handlers2);
+        }
+      if (bgh->handler (bgh->data))
+        {
+          if (prio == 1)
+            background_handlers1 = sfi_ring_append (background_handlers1, bgh);
+          else
+            background_handlers2 = sfi_ring_append (background_handlers2, bgh);
+        }
+      else
+        {
+          if (bgh->free_func)
+            bgh->free_func (bgh->data);
+          g_free (bgh);
+        }
+      if (background_handlers1 || background_handlers2)
+        gxk_status_set (GXK_STATUS_PROGRESS, _("Updating View"), NULL);
+    }
+  if (!background_handlers1 && !background_handlers2)
+    gxk_status_set (100, _("Updating View"), NULL); /* done */
+  GDK_THREADS_LEAVE();
+  /* re-queue instead of returning TRUE to start a new delay cycle */
+  if (background_handlers1 || background_handlers2)
+    g_timeout_add_full (G_PRIORITY_LOW - 100,
+                        30, /* milliseconds */
+                        bst_background_handlers_timeout, NULL, NULL);
+  return FALSE;
+}
+
+static void
+bst_background_handler_add (gboolean       (*handler) (gpointer data),
+                            gpointer         data,
+                            void           (*free_func) (gpointer data),
+                            gint             prio)
+{
+  g_return_if_fail (handler != NULL);
+  BackgroundHandler *bgh = g_new0 (BackgroundHandler, 1);
+  bgh->handler = handler;
+  bgh->data = data;
+  bgh->free_func = free_func;
+  if (!background_handlers1 && !background_handlers2)
+    g_timeout_add_full (G_PRIORITY_LOW - 100,
+                        20, /* milliseconds */
+                        bst_background_handlers_timeout, NULL, NULL);
+  if (prio == 1)
+    background_handlers1 = sfi_ring_append (background_handlers1, bgh);
+  else
+    background_handlers2 = sfi_ring_append (background_handlers2, bgh);
+}
+
+void
+bst_background_handler1_add (gboolean       (*handler) (gpointer data),
+                             gpointer         data,
+                             void           (*free_func) (gpointer data))
+{
+  bst_background_handler_add (handler, data, free_func, 1);
+}
+
+void
+bst_background_handler2_add (gboolean       (*handler) (gpointer data),
+                             gpointer         data,
+                             void           (*free_func) (gpointer data))
+{
+  bst_background_handler_add (handler, data, free_func, 2);
+}
 
 /* --- packing utilities --- */
 #define SPACING 3
@@ -1243,14 +1360,13 @@ bst_file_scan_find_key (const gchar *file,
 
 
 /* --- source file key scans --- */
-#include "bstdebugkeys.defs"
-#ifndef BST_DEBUG_KEYS
-#  define BST_DEBUG_KEYS        /* none */
-#endif
-static const gchar *debug_keys[] = { BST_LOG_DEBUG_KEYS NULL };
 const gchar**
 _bst_log_debug_keys (void)
 {
+  static const gchar *debug_keys[] = { 
+#include "bstdebugkeys.defs"
+    NULL
+  };
   return debug_keys;
 }
 
