@@ -26,6 +26,11 @@
 
 G_BEGIN_DECLS
 
+/* --- BSE utilities --- */
+BseErrorType   bst_project_restore_from_file  (SfiProxy         project,
+                                               const gchar     *file_name);
+
+
 /* --- GUI utilities --- */
 void           bst_status_eprintf             (BseErrorType     error,
                                                const gchar     *message_fmt,
@@ -71,6 +76,12 @@ GxkActionList* bst_action_list_from_cats_pred (BseCategorySeq  *cseq,
                                                gpointer         user_data,
                                                BstActionListCategoryP predicate,
                                                gpointer         predicate_data);
+void           bst_background_handler1_add    (gboolean       (*handler) (gpointer data),
+                                               gpointer         data,
+                                               void           (*free_func) (gpointer data));
+void           bst_background_handler2_add    (gboolean       (*handler) (gpointer data),
+                                               gpointer         data,
+                                               void           (*free_func) (gpointer data));
 
 
 /* --- stock utilities --- */
@@ -163,15 +174,16 @@ BstGMask*       bst_gmask_quick         (GtkWidget     *gmask_container,
 #define BST_STOCK_PREFERENCES           GTK_STOCK_PREFERENCES
 #define BST_STOCK_PLAY                  GTK_STOCK_GO_FORWARD // FIXME
 #define BST_STOCK_STOP                  GTK_STOCK_STOP
-#define BST_STOCK_NEW_SONG              GTK_STOCK_NEW // FIXME
-#define BST_STOCK_NEW_CSYNTH            GTK_STOCK_NEW // FIXME
-#define BST_STOCK_NEW_MIDI_SYNTH        GTK_STOCK_NEW // FIXME
-#define BST_STOCK_REMOVE_SYNTH          GTK_STOCK_REMOVE // REMOVE
+#define BST_STOCK_REMOVE_SYNTH          GTK_STOCK_REMOVE
+#define BST_STOCK_DOC_INDEX             GTK_STOCK_HELP // FIXME
 #define BST_STOCK_DOC_NEWS              GTK_STOCK_NEW // FIXME
 #define BST_STOCK_HELP                  GTK_STOCK_HELP
 #define BST_STOCK_DOC_FAQ               GTK_STOCK_NEW // FIXME
 #define BST_STOCK_DOC_DEVEL             GTK_STOCK_NEW // FIXME
 #define BST_STOCK_ABOUT                 GTK_STOCK_YES // FIXME
+#define BST_STOCK_ONLINE_HELP_DESK      GTK_STOCK_HELP // FIXME
+#define BST_STOCK_ONLINE_BEAST_SITE     GTK_STOCK_HOME // FIXME
+#define BST_STOCK_ONLINE_SOUND_ARCHIVE  BST_STOCK_LOAD_LIB // FIXME
 #define BST_STOCK_ADD                   GTK_STOCK_ADD
 #define BST_STOCK_REMOVE                GTK_STOCK_REMOVE
 
