@@ -202,6 +202,48 @@ gxk_menu_item_get_type (void)
   return type;
 }
 
+/* --- scrolled window --- */
+G_DEFINE_TYPE (GxkScrolledWindow, gxk_scrolled_window, GTK_TYPE_SCROLLED_WINDOW);
+
+static void
+gxk_scrolled_window_init (GxkScrolledWindow *self)
+{
+}
+
+enum {
+  SCROLLED_WINDOW_PROP_0,
+  SCROLLED_WINDOW_PROP_SPARE_SPACE,
+};
+static void
+gxk_scrolled_window_set_property (GObject      *object,
+                                  guint         param_id,
+                                  const GValue *value,
+                                  GParamSpec   *pspec)
+{
+  GxkScrolledWindow *self = GXK_SCROLLED_WINDOW (object);
+  switch (param_id)
+    {
+    case SCROLLED_WINDOW_PROP_SPARE_SPACE:
+      if (g_value_get_boolean (value))
+        gxk_scrolled_window_spare_space (self);
+      else
+        gxk_scrolled_window_unspare_space (self);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (self, param_id, pspec);
+      break;
+    }
+}
+
+static void
+gxk_scrolled_window_class_init (GxkScrolledWindowClass *class)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+  gobject_class->set_property = gxk_scrolled_window_set_property;
+  g_object_class_install_property (gobject_class, SCROLLED_WINDOW_PROP_SPARE_SPACE,
+                                   g_param_spec_boolean ("spare-space", NULL, NULL, FALSE, G_PARAM_WRITABLE));
+  GTK_SCROLLED_WINDOW_CLASS (class)->scrollbar_spacing = 1;
+}
 
 /* --- GxkFreeRadioButton --- */
 static void
@@ -321,6 +363,10 @@ enum {
   PATCHER_PROP_TOOLTIP,
   PATCHER_PROP_TOOLTIP_VISIBLE,
   PATCHER_PROP_MUTE_EVENTS,
+  PATCHER_PROP_NORMAL_BG_AS_BASE,
+  PATCHER_PROP_BG_AS_BASE,
+  PATCHER_PROP_BG_AS_ACTIVE,
+  PATCHER_PROP_BASE_AS_BG,
   PATCHER_PROP_LOWER_WINDOWS,
   PATCHER_PROP_HIDE_INSENSITIVE,
   PATCHER_PROP_WIDTH_FROM_HEIGHT,
@@ -348,6 +394,18 @@ gxk_widget_patcher_set_property (GObject      *object,
     case PATCHER_PROP_MUTE_EVENTS:
       self->mute_events = g_value_get_boolean (value);
       break;
+    case PATCHER_PROP_NORMAL_BG_AS_BASE:
+      self->modify_normal_bg_as_base = g_value_get_boolean (value);
+      break;
+    case PATCHER_PROP_BG_AS_ACTIVE:
+      self->modify_bg_as_active = g_value_get_boolean (value);
+      break;
+    case PATCHER_PROP_BG_AS_BASE:
+      self->modify_bg_as_base = g_value_get_boolean (value);
+      break;
+    case PATCHER_PROP_BASE_AS_BG:
+      self->modify_base_as_bg = g_value_get_boolean (value);
+      break;
     case PATCHER_PROP_LOWER_WINDOWS:
       self->lower_windows = g_value_get_boolean (value);
       break;
@@ -398,6 +456,14 @@ gxk_widget_patcher_class_init (GxkWidgetPatcherClass *class)
                                                          TRUE, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
   g_object_class_install_property (gobject_class, PATCHER_PROP_MUTE_EVENTS,
                                    g_param_spec_boolean ("mute-events", NULL, NULL, FALSE, G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class, PATCHER_PROP_NORMAL_BG_AS_BASE,
+                                   g_param_spec_boolean ("normal-bg-as-base", NULL, NULL, FALSE, G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class, PATCHER_PROP_BG_AS_ACTIVE,
+                                   g_param_spec_boolean ("bg-as-active", NULL, NULL, FALSE, G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class, PATCHER_PROP_BG_AS_BASE,
+                                   g_param_spec_boolean ("bg-as-base", NULL, NULL, FALSE, G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class, PATCHER_PROP_BASE_AS_BG,
+                                   g_param_spec_boolean ("base-as-bg", NULL, NULL, FALSE, G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class, PATCHER_PROP_LOWER_WINDOWS,
                                    g_param_spec_boolean ("lower-windows", NULL, NULL, FALSE, G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class, PATCHER_PROP_HIDE_INSENSITIVE,
@@ -506,20 +572,28 @@ widget_patcher_adopt (GxkRadget          *radget,
         gtk_tooltips_set_tip (GXK_TOOLTIPS, parent, self->tooltip, NULL);
     }
   if (self->mute_events &&
-      !gxk_signal_handler_pending (parent, "event", G_CALLBACK (widget_mute_events), NULL))
+      !gxk_signal_handler_exists (parent, "event", G_CALLBACK (widget_mute_events), NULL))
     g_object_connect (parent, "signal::event", widget_mute_events, NULL, NULL);
+  if (self->modify_normal_bg_as_base)
+    gxk_widget_modify_normal_bg_as_base (parent);
+  if (self->modify_bg_as_active)
+    gxk_widget_modify_bg_as_active (parent);
+  if (self->modify_bg_as_base)
+    gxk_widget_modify_bg_as_base (parent);
+  if (self->modify_base_as_bg)
+    gxk_widget_modify_base_as_bg (parent);
   if (self->lower_windows &&
-      !gxk_signal_handler_pending (parent, "map", G_CALLBACK (widget_lower_windows), NULL))
+      !gxk_signal_handler_exists (parent, "map", G_CALLBACK (widget_lower_windows), NULL))
     g_object_connect (parent, "signal_after::map", widget_lower_windows, NULL, NULL);
   if (self->hide_insensitive &&
-      !gxk_signal_handler_pending (parent, "state-changed", G_CALLBACK (widget_hide_insensitive), NULL))
+      !gxk_signal_handler_exists (parent, "state-changed", G_CALLBACK (widget_hide_insensitive), NULL))
     g_object_connect (parent, "signal_after::state-changed", widget_hide_insensitive, NULL, NULL);
   if (self->width_from_height &&
-      !gxk_signal_handler_pending (parent, "size-request", G_CALLBACK (widget_patcher_width_from_height), NULL))
+      !gxk_signal_handler_exists (parent, "size-request", G_CALLBACK (widget_patcher_width_from_height), NULL))
     g_object_connect (parent, "signal_after::size-request", widget_patcher_width_from_height, NULL, NULL);
   g_object_set_double (parent, "width-from-height", self->width_from_height);
   if (self->height_from_width &&
-      !gxk_signal_handler_pending (parent, "size-request", G_CALLBACK (widget_patcher_height_from_width), NULL))
+      !gxk_signal_handler_exists (parent, "size-request", G_CALLBACK (widget_patcher_height_from_width), NULL))
     g_object_connect (parent, "signal_after::size-request", widget_patcher_height_from_width, NULL, NULL);
   g_object_set_double (parent, "height-from-width", self->height_from_width);
   if ((self->resize_hsteps || self->resize_vsteps ||
@@ -555,24 +629,26 @@ gxk_widget_patcher_get_type (void)
   return type;
 }
 
+static GParamSpec*
+widget_patcher_find_prop (GTypeClass  *klass,
+                          const gchar *prop_name)
+{
+  return g_object_class_find_property (G_OBJECT_CLASS (klass), prop_name);
+}
+
 static GxkRadget*
 widget_patcher_create (GType               type,
                        const gchar        *name,
+                       guint               n_construct_params,
+                       GParameter         *construct_params,
                        GxkRadgetData      *gdgdata)
 {
-  return g_object_new (type, NULL);
-}
-
-static GParamSpec*
-widget_patcher_find_prop (GxkRadget    *radget,
-                          const gchar  *prop_name)
-{
-  return g_object_class_find_property (G_OBJECT_GET_CLASS (radget), prop_name);
+  return g_object_newv (type, n_construct_params, construct_params);
 }
 
 static const GxkRadgetType widget_patcher_def = {
-  widget_patcher_create,
   widget_patcher_find_prop,
+  widget_patcher_create,
   (void(*)(GxkRadget*,const gchar*,const GValue*)) g_object_set_property,
   widget_patcher_adopt,
   NULL,         /* find_pack */
@@ -581,42 +657,6 @@ static const GxkRadgetType widget_patcher_def = {
 const GxkRadgetType *_gxk_widget_patcher_def = &widget_patcher_def;
 
 
-/* --- focus frame --- */
-G_DEFINE_TYPE (GxkFocusFrame, gxk_focus_frame, GTK_TYPE_FRAME);
-
-static gboolean
-focus_frame_expose_event (GtkWidget      *widget,
-                          GdkEventExpose *event)
-{
-  GtkFrame *frame = GTK_FRAME (widget);
-  gint x = frame->child_allocation.x - widget->style->xthickness;
-  gint y = frame->child_allocation.y - widget->style->ythickness;
-  gint width = frame->child_allocation.width + 2 * widget->style->xthickness;
-  gint height =  frame->child_allocation.height + 2 * widget->style->ythickness;
-  if (frame->shadow_type != GTK_SHADOW_NONE)
-    gtk_paint_focus (widget->style, widget->window, GTK_WIDGET_STATE (widget),
-                     &event->area, widget, "button", x, y, width, height);
-  if (0)
-    gtk_paint_shadow (widget->style, widget->window, GTK_STATE_NORMAL, frame->shadow_type,
-                      &event->area, widget, "frame", x, y, width, height);
-  /* skip normal frame drawing code */
-  return GTK_WIDGET_CLASS (g_type_class_peek (g_type_parent (GTK_TYPE_FRAME)))->expose_event (widget, event);
-}
-
-static void
-gxk_focus_frame_init (GxkFocusFrame *self)
-{
-  GTK_WIDGET_SET_FLAGS (self, GTK_VISIBLE);
-  gtk_container_set_border_width (GTK_CONTAINER (self), 1);
-}
-
-static void
-gxk_focus_frame_class_init (GxkFocusFrameClass *class)
-{
-  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
-  widget_class->expose_event = focus_frame_expose_event;
-}
-
 /* --- back shade --- */
 G_DEFINE_TYPE (GxkBackShade, gxk_back_shade, GTK_TYPE_ALIGNMENT);
 
@@ -625,9 +665,9 @@ static void
 rgb_to_hls (gdouble  red,
             gdouble  green,
             gdouble  blue,
-            gdouble *h_p,
-            gdouble *l_p,
-            gdouble *s_p)
+            gdouble *h_p,	/* 0..360 */
+            gdouble *l_p,	/* 0..1 */
+            gdouble *s_p)	/* 0..1 */
 {
   gdouble max = MAX (MAX (red, green), blue);
   gdouble min = MIN (MIN (red, green), blue);
@@ -655,9 +695,9 @@ rgb_to_hls (gdouble  red,
 }
 
 static void
-hls_to_rgb (gdouble  hue,
-            gdouble  lightness,
-            gdouble  saturation,
+hls_to_rgb (gdouble  hue,		/* 0..360 */
+            gdouble  lightness,		/* 0..1 */
+            gdouble  saturation,	/* 0..1 */
             gdouble *r_p,
             gdouble *g_p,
             gdouble *b_p)
@@ -725,9 +765,9 @@ static void
 color_shade (GdkColor *color,
              gdouble   k)
 {
-  gdouble r = color->red / 65536.;
-  gdouble g = color->green / 65536.;
-  gdouble b = color->blue / 65536.;
+  gdouble r = color->red / 65535.;
+  gdouble g = color->green / 65535.;
+  gdouble b = color->blue / 65535.;
   gdouble h, l, s;
   rgb_to_hls (r, g, b, &h, &l, &s);
   l = CLAMP (l * k, 0, 1);
@@ -735,7 +775,7 @@ color_shade (GdkColor *color,
   hls_to_rgb (h, l, s, &r, &g, &b);
   color->red = r * 65535.;
   color->green = g * 65535.;
-  color->blue = b * 65535.0;
+  color->blue = b * 65535.;
 }
 
 static GdkGC*
@@ -779,4 +819,3 @@ gxk_back_shade_class_init (GxkBackShadeClass *class)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
   widget_class->expose_event = back_shade_expose_event;
 }
-
