@@ -169,18 +169,18 @@ gxk_dialog_init (GxkDialog *self)
   self->hbox = g_object_new (GTK_TYPE_HBOX,
                              "visible", FALSE,
                              "homogeneous", TRUE,
-                             "spacing", GXK_INNER_PADDING,
-                             "border_width", GXK_INNER_PADDING,
+                             "spacing", GXK_BUTTON_PADDING,
+                             "border_width", GXK_OUTER_BORDER,
                              NULL);
   gxk_nullify_in_object (self, &self->hbox);
-  gtk_box_pack_end (GTK_BOX (self->mbox), self->hbox, FALSE, TRUE, GXK_INNER_PADDING);
+  gtk_box_pack_end (GTK_BOX (self->mbox), self->hbox, FALSE, TRUE, 0);
   
   /* separator */
   self->sep = g_object_new (GTK_TYPE_HSEPARATOR,
                             "visible", FALSE,
                             NULL);
   gxk_nullify_in_object (self, &self->sep);
-  gtk_box_pack_end (GTK_BOX (self->mbox), self->sep, FALSE, FALSE, GXK_INNER_PADDING);
+  gtk_box_pack_end (GTK_BOX (self->mbox), self->sep, FALSE, FALSE, 0);
 }
 
 static void
@@ -194,9 +194,9 @@ gxk_dialog_set_property (GObject      *object,
   switch (prop_id)
     {
       GtkWindow *window;
-      const gchar *cstring;
+      const gchar *cstring, *astring;
       gchar *string;
-      guint old_flags;
+      guint old_flags, added_flags, removed_flags;
     case PROP_ALIVE_OBJECT:
       if (self->alive_object)
 	g_signal_handlers_disconnect_by_func (self->alive_object, gtk_widget_destroy, self);
@@ -213,43 +213,50 @@ gxk_dialog_set_property (GObject      *object,
       cstring = g_value_get_string (value);
       if (!cstring)
 	cstring = "";
-      string = g_strconcat (cstring, " - ", g_get_prgname (), NULL);
+      astring = g_get_application_name();
+      string = g_strconcat (cstring, astring ? " - " : NULL, astring, NULL);
       g_object_set (self, "GtkWindow::title", string, NULL);
       g_free (string);
       break;
     case PROP_FLAGS:
       old_flags = self->flags;
       self->flags = g_value_get_flags (value);
+      added_flags = self->flags & ~old_flags;
+      removed_flags = old_flags & ~self->flags;
       window = GTK_WINDOW (self);
-      if (!(old_flags & GXK_DIALOG_STATUS_SHELL) && (self->flags & GXK_DIALOG_STATUS_SHELL))
+      if (added_flags & GXK_DIALOG_WINDOW_GROUP)
 	{
 	  GtkWindowGroup *wgroup = gtk_window_group_new ();
 	  if (window->group)
 	    gtk_window_group_remove_window (window->group, window);
-	  /* gtk_window_group_add_window (wgroup, window); GTKFIX: Gtk+-2.2.1 window groups seem to be broken wrt menus */
+          gtk_window_group_add_window (wgroup, window); // GTKFIX: Gtk+-2.2.1 window groups seem to be broken wrt menus, but we're using Gtk+-2.4 now */
 	  g_object_unref (wgroup);
-	  if (self->status_bar)
-	    gtk_widget_show (self->status_bar);
 	}
-      else if ((old_flags & GXK_DIALOG_STATUS_SHELL) && !(self->flags & GXK_DIALOG_STATUS_SHELL))
+      else if (removed_flags & GXK_DIALOG_WINDOW_GROUP)
 	{
 	  if (window->group)
 	    gtk_window_group_remove_window (window->group, window);
+	}
+      if (added_flags & GXK_DIALOG_STATUS_BAR)
+	{
+	  if (self->status_bar)
+	    gtk_widget_show (self->status_bar);
+	}
+      else if (removed_flags & GXK_DIALOG_STATUS_BAR)
+	{
 	  if (self->status_bar)
 	    gtk_widget_hide (self->status_bar);
 	}
       gtk_window_set_modal (GTK_WINDOW (self), self->flags & GXK_DIALOG_MODAL);
-      /* some flags can't be unset */
-      if (!(old_flags & GXK_DIALOG_DELETE_BUTTON) &&
-	  (self->flags & GXK_DIALOG_DELETE_BUTTON))
+      if (added_flags & GXK_DIALOG_DELETE_BUTTON)
 	{
 	  /* we synthesize a delete event instead of hiding/destroying
 	   * directly, because derived classes may override delete_event
 	   */
 	  gxk_dialog_default_action (self, GTK_STOCK_CLOSE, gxk_toplevel_delete, NULL);
 	}
-      else if (old_flags & GXK_DIALOG_DELETE_BUTTON)
-	self->flags |= GXK_DIALOG_DELETE_BUTTON;
+      else if (removed_flags & GXK_DIALOG_DELETE_BUTTON)
+	self->flags |= GXK_DIALOG_DELETE_BUTTON;        /* some flags can't be unset */
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -325,15 +332,16 @@ gxk_dialog_finalize (GObject *object)
  *
  * Create a new configurable dialog. Possible values for the
  * flags are:
- * %GXK_DIALOG_HIDE_ON_DELETE: only hide and not desroy the
+ * @* %GXK_DIALOG_HIDE_ON_DELETE: only hide and not destroy the
  * dialog upon window manager delete events;
- * %GXK_DIALOG_STATUS_SHELL: the dialog has a status bar and
- * acts as a shell window for primary application data;
- * %GXK_DIALOG_MODAL: the dialog is modal while visible;
- * %GXK_DIALOG_POPUP_POS: popup the dialog below mouse pointer;
- * %GXK_DIALOG_DELETE_BUTTON: add a "Close" button to the dialog
- * (not recommended for GXK_DIALOG_STATUS_SHELL dialogs, which
- * usually have menus).
+ * @* %GXK_DIALOG_IGNORE_ESCAPE: prevents delete event generation on Escape key presses;
+ * @* %GXK_DIALOG_DELETE_BUTTON: add a "Close" button to the dialog;
+ * @* %GXK_DIALOG_STATUS_BAR: add a status bar widget to the dialog;
+ * @* %GXK_DIALOG_WINDOW_GROUP: open up an extra window group for the dialog;
+ * @* %GXK_DIALOG_MODAL: the dialog is modal while visible;
+ * @* %GXK_DIALOG_POPUP_POS: popup the dialog below mouse pointer;
+ * @* %GXK_DIALOG_PRESERVE_STATE: prevents unrealization of the dialog upon hiding,
+ * which preserves properties like the window size.
  */
 gpointer
 gxk_dialog_new (gpointer       pointer_loc,
@@ -371,6 +379,41 @@ gxk_dialog_new_radget (gpointer        pointer_loc,
     return NULL;
 }
 
+/**
+ * gxk_dialog_set_sizes
+ * @dialog:         valid GxkDialog
+ * @min_width:      minimum dialog width or -1
+ * @min_height:     minimum dialog height or -1
+ * @default_width:  default dialog width or -1
+ * @default_height: default dialog height or -1
+ *
+ * Set the dialog's minimum and default sizes, constrained to not exceed the
+ * screen dimensions.
+ */
+void
+gxk_dialog_set_sizes (GxkDialog      *dialog,
+                      gint            min_width,
+                      gint            min_height,
+                      gint            default_width,
+                      gint            default_height)
+{
+  GdkGeometry geometry = { 0, };
+  gint swidth = gdk_screen_width();
+  gint sheight = gdk_screen_height();
+  geometry.min_width = CLAMP (min_width, -1, swidth);
+  geometry.min_height = CLAMP (min_height, -1, sheight);
+  if (default_width > 0)
+    default_width = MIN (default_width, swidth * 0.95);
+  if (default_height > 0)
+    default_height = MIN (default_height, sheight * 0.95);
+  if (geometry.min_width > 0 && default_width > 0)
+    geometry.min_width = MIN (geometry.min_width, default_width);
+  if (geometry.min_height > 0 && default_height > 0)
+    geometry.min_height = MIN (geometry.min_height, default_height);
+  gtk_window_set_geometry_hints (GTK_WINDOW (dialog), NULL, &geometry, GDK_HINT_MIN_SIZE);
+  gtk_window_set_default_size (GTK_WINDOW (dialog), default_width, default_height);
+}
+
 /**
  * gxk_dialog_set_title
  * @dialog: valid GxkDialog
@@ -553,36 +596,6 @@ gxk_dialog_hide (GtkWidget *widget)
     gxk_idle_unrealize_widget (widget);
 }
 
-#include <gmodule.h>    // FIXME: remove when depending on Gtk+-2.4
-static gboolean
-call_gtk_window_activate_key (GtkWindow   *window,      // FIXME: remove when depending on Gtk+-2.4
-                              GdkEventKey *event)
-{
-#if     GTK_CHECK_VERSION (2, 3, 4)
-  return gtk_window_activate_key (window, event);
-#elif   defined (__linux__)     /* on linux, Gtk+-2.2 exports _gtk_window_activate_key() */
-  /* Gtk+-2.2 case. we need to go through some hoops, to allow beast to
-   * continue to work, even if built against Gtk+-2.2 and later used
-   * with Gtk+-2.4 which doesn't have _gtk_window_activate_key() anymore.
-   */
-  if (gtk_check_version (2, 3, 4) == NULL)
-    {
-      static gboolean (*window_activate_key) (GtkWindow*, GdkEventKey*) = NULL;
-      if (!window_activate_key)
-        {
-          GModule *main_module = g_module_open (NULL, 0);
-          if (!g_module_symbol (main_module, "gtk_window_activate_key", (gpointer) &window_activate_key))
-            window_activate_key = NULL;
-          g_module_close (main_module);
-        }
-      return window_activate_key (window, event);
-    }
-  return _gtk_window_activate_key (window, event);
-#else
-  return FALSE;
-#endif
-}
-
 static gboolean
 gxk_dialog_key_press_event (GtkWidget   *widget,
 			    GdkEventKey *event)
@@ -597,7 +610,7 @@ gxk_dialog_key_press_event (GtkWidget   *widget,
    */
   if (event->keyval == GDK_Escape &&
       ((dialog->flags & GXK_DIALOG_DELETE_BUTTON) ||
-       !(dialog->flags & GXK_DIALOG_STATUS_SHELL)))
+       !(dialog->flags & GXK_DIALOG_IGNORE_ESCAPE)))
     {
       /* trigger delete event */
       gxk_toplevel_delete (widget);
@@ -608,52 +621,19 @@ gxk_dialog_key_press_event (GtkWidget   *widget,
    * the focus widget precedence over unmodified accelerators
    * before the accelerator activation scheme.
    */
-#if     GTK_CHECK_VERSION (2, 3, 4)
+
   /* invoke control/alt accelerators */
   if (!handled && event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK))
-    handled = call_gtk_window_activate_key (window, event);
+    handled = gtk_window_activate_key (window, event);
   /* invoke focus widget handlers */
   if (!handled)
     handled = gtk_window_propagate_key_event (window, event);
   /* invoke non-(control/alt) accelerators */
   if (!handled && !(event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK)))
-    handled = call_gtk_window_activate_key (window, event);
+    handled = gtk_window_activate_key (window, event);
   /* chain up, bypassing gtk_window_key_press(), to invoke binding set */
   if (!handled)
     handled = GTK_WIDGET_CLASS (g_type_class_peek (g_type_parent (GTK_TYPE_WINDOW)))->key_press_event (widget, event);
-#elif   defined (__linux__)     /* on linux we can circumvent gtk+-2.2 also */
-  /* invoke control/alt accelerators */
-  if (!handled && event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK))
-    handled = call_gtk_window_activate_key (window, event); /* available due to broken libtool */
-  /* invoke focus widget handlers */
-  if (!handled)
-    {
-      GtkWidget *focus = window->focus_widget;
-      if (focus)
-        g_object_ref (focus);
-      while (!handled && focus && focus != widget && gtk_widget_get_toplevel (focus) == widget)
-        {
-          GtkWidget *parent;
-          if (GTK_WIDGET_IS_SENSITIVE (focus))
-            handled = gtk_widget_event (focus, (GdkEvent*) event);
-          parent = focus->parent;
-          if (parent)
-            g_object_ref (parent);
-          g_object_unref (focus);
-          focus = parent;
-        }
-      if (focus)
-        g_object_unref (focus);
-    }
-  /* invoke non-(control/alt) accelerators */
-  if (!handled && !(event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK)))
-    handled = call_gtk_window_activate_key (window, event);
-  /* chain up, bypassing gtk_window_key_press(), to invoke binding set */
-  if (!handled)
-    handled = GTK_WIDGET_CLASS (g_type_class_peek (g_type_parent (GTK_TYPE_WINDOW)))->key_press_event (widget, event);
-#else   /* gtk+ <= 2.2, non-linux */
-  handled = GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
-#endif
 
   return handled;
 }
@@ -665,10 +645,10 @@ gxk_dialog_delete_event (GtkWidget   *widget,
   GxkDialog *dialog = GXK_DIALOG (widget);
 
   if (dialog->flags & GXK_DIALOG_HIDE_ON_DELETE)
-    gtk_widget_hide (GTK_WIDGET (dialog));
-
-  if (dialog->flags & GXK_DIALOG_HIDE_ON_DELETE)
-    return TRUE;
+    {
+      gtk_widget_hide (GTK_WIDGET (dialog));
+      return TRUE;
+    }
   else
     return FALSE;
 }
