@@ -29,7 +29,12 @@ param_update_flags (GxkParam *param)
   gboolean was_sensitive = param->sensitive;
   GSList *slist;
   param->editable = !param->constant;
-  param->sensitive = param->constant || (!param->treadonly && param->ueditable);
+  if (!param->constant && param->ueditable &&
+      param->binding && param->binding->check_writable)
+    param->breadonly = !param->binding->check_writable (param);
+  else
+    param->breadonly = FALSE;
+  param->sensitive = param->constant || (!param->breadonly && !param->greadonly && param->ueditable);
   if (was_sensitive != param->sensitive)
     for (slist = param->objects; slist; slist = slist->next)
       if (GTK_IS_WIDGET (slist->data))
@@ -52,14 +57,15 @@ param_new (GParamSpec      *pspec,
                     !(pspec->flags & G_PARAM_WRITABLE) ||
                     g_param_spec_check_option (pspec, "ro");
   param->ueditable = TRUE;
+  param->greadonly = FALSE;
+  param_update_flags (param);
   param->binding = binding;
   if (param->binding->setup)
     {
-      param_update_flags (param);
       param->binding->setup (param, user_data);
       /* setup may change param->constant */
+      param_update_flags (param);
     }
-  param_update_flags (param);
   return param;
 }
 
@@ -92,7 +98,10 @@ param_call_update (GxkParam *param,
     {
       gboolean updating = param->updating;
       param->updating = TRUE;   /* protect value from change-notifications during setup */
+      gboolean was_greadonly = param->greadonly;
       ufunc (param, object);
+      if (was_greadonly != param->greadonly)
+        param_update_flags (param);     /* update() may change greadonly */
       param->updating = updating;
     }
 }
@@ -101,6 +110,7 @@ void
 gxk_object_set_param_callback (GtkObject          *object,
                                GxkParamUpdateFunc  ufunc)
 {
+  /* sets the callback to apply param->value to object (update GUI) */
   g_return_if_fail (GTK_IS_OBJECT (object));
   g_object_set_data (object, "GxkParamUpdateFunc", ufunc);
 }
@@ -128,19 +138,47 @@ gxk_param_update (GxkParam *param)
   param->updating = updating;
 }
 
-void
-gxk_param_add_object (GxkParam          *param,
-                      GtkObject         *object)
+static gboolean
+gxk_param_grouping_event (GtkWidget *widget,
+                          GdkEvent  *event,
+                          GxkParam  *param)
 {
-  g_return_if_fail (GXK_IS_PARAM (param));
-  if (GTK_IS_WIDGET (object))
-    gtk_widget_set_sensitive (GTK_WIDGET (object), param->sensitive);
-  param->objects = g_slist_prepend (param->objects, g_object_ref (object));
-  g_object_connect (object, "swapped_signal::destroy", gxk_param_remove_object, param, NULL);
-  param_call_update (param, object);
+  switch (event->type)
+    {
+      // case GDK_KEY_PRESS:
+    case GDK_BUTTON_PRESS:
+      if (GTK_WIDGET_DRAWABLE (widget))
+        gxk_param_start_grouping (param);
+      break;
+      // case GDK_KEY_RELEASE:
+    case GDK_BUTTON_RELEASE:
+      gxk_param_stop_grouping (param);
+      break;
+    default: ;
+    }
+  return FALSE;
+}
+
+static void
+gxk_param_grouping_unrealized (GtkWidget *widget,
+                               GxkParam  *param)
+{
+  while (param->grouping)
+    gxk_param_stop_grouping (param);
 }
 
 void
+gxk_param_add_grab_widget (GxkParam           *param,
+                           GtkWidget          *widget)
+{
+  gxk_param_add_object (param, GTK_OBJECT (widget));
+  g_object_connect (widget,
+                    "signal::event", gxk_param_grouping_event, param,
+                    "signal::unrealize", gxk_param_grouping_unrealized, param,
+                    NULL);
+}
+
+static void
 gxk_param_remove_object (GxkParam          *param,
                          GtkObject         *object)
 {
@@ -150,9 +188,30 @@ gxk_param_remove_object (GxkParam          *param,
   g_return_if_fail (slist != NULL);
   param->objects = g_slist_delete_link (param->objects, slist);
   g_object_disconnect (object, "any_signal::destroy", gxk_param_remove_object, param, NULL);
+  if (GTK_IS_WIDGET (object) && /* check for grab_widget: */
+      gxk_signal_handler_pending (object, "unrealize", G_CALLBACK (gxk_param_grouping_unrealized), param))
+    g_object_disconnect (object,
+                         "any_signal::event", gxk_param_grouping_event, param,
+                         "any_signal::unrealize", gxk_param_grouping_unrealized, param,
+                         NULL);
   g_object_unref (object);
 }
 
+void
+gxk_param_add_object (GxkParam          *param,
+                      GtkObject         *object)
+{
+  g_return_if_fail (GXK_IS_PARAM (param));
+  if (gxk_signal_handler_pending (object, "destroy", G_CALLBACK (gxk_param_remove_object), param) == FALSE)
+    {
+      if (GTK_IS_WIDGET (object))
+        gtk_widget_set_sensitive (GTK_WIDGET (object), param->sensitive);
+      param->objects = g_slist_prepend (param->objects, g_object_ref (object));
+      g_object_connect (object, "swapped_signal::destroy", gxk_param_remove_object, param, NULL);
+      param_call_update (param, object);
+    }
+}
+
 void
 gxk_param_apply_value (GxkParam *param)
 {
@@ -233,11 +292,38 @@ gxk_param_dup_tooltip (GxkParam *param)
   return tooltip;
 }
 
+void
+gxk_param_start_grouping (GxkParam *param)
+{
+  g_return_if_fail (GXK_IS_PARAM (param));
+  g_return_if_fail (param->binding != NULL);
+  g_return_if_fail (param->grouping < 0xff);
+  if (!param->grouping++ &&
+      param->binding->start_grouping)
+    param->binding->start_grouping (param);
+}
+
+void
+gxk_param_stop_grouping (GxkParam *param)
+{
+  g_return_if_fail (GXK_IS_PARAM (param));
+  g_return_if_fail (param->binding != NULL);
+  if (param->grouping)
+    {
+      if (!--param->grouping &&
+          param->binding->stop_grouping)
+        param->binding->stop_grouping (param);
+    }
+}
+
 void
 gxk_param_destroy (GxkParam *param)
 {
   g_return_if_fail (GXK_IS_PARAM (param));
   g_return_if_fail (param->binding != NULL);
+
+  while (param->grouping)
+    gxk_param_stop_grouping (param);
   
   while (param->objects)
     {
@@ -431,13 +517,13 @@ param_editor_name_match (const gchar          *editor_name,
 
 static guint
 param_score_editor (const GxkParamEditor *editor,
-                    GParamSpec           *pspec)
+                    GParamSpec           *pspec,
+                    gboolean              annotate)
 {
-  gboolean fundamental_specific, type_mismatch = FALSE, type_specific = FALSE;
+  gboolean fundamental_specific, type_mismatch = FALSE, option_mismatch = FALSE, type_specific = FALSE;
   guint8 bonus = 0, type_distance = 0;
   GType vtype = G_PARAM_SPEC_VALUE_TYPE (pspec);
   GType etype = editor->type_match.type;
-  guint rating = 0;
   gboolean is_editable = editor->features.editing;
   if (editor->type_match.type_name)
     {
@@ -458,13 +544,30 @@ param_score_editor (const GxkParamEditor *editor,
     }
   if (editor->features.options)
     {
-      type_mismatch |= !g_param_spec_provides_options (pspec, editor->features.options);
+      option_mismatch |= !g_param_spec_provides_options (pspec, editor->features.options);
       bonus++;
     }
+
+  if (annotate)
+    g_printerr ("  %s(%s): fundamental=%s derived=%s%s%s editing=%s bonus=%+d%s%s: ",
+                editor->ident.name, editor->ident.nick,
+                g_type_name (editor->type_match.type),
+                editor->type_match.type_name ? editor->type_match.type_name : "<any>",
+                editor->type_match.all_int_nums ? "(all-ints)" : "",
+                editor->type_match.all_float_nums ? "(all-floats)" : "",
+                editor->features.editing ? "yes" : "no",
+                editor->features.rating,
+                editor->features.options ? " options=" : "",
+                editor->features.options ? editor->features.options : "");
+
+  if (type_mismatch || option_mismatch) /* bail out on mismatches */
+    {
+      if (annotate)
+        g_printerr ("%s mismatch\n", type_mismatch ? "type" : "option");
+      return 0;
+    }
   
-  if (type_mismatch)    /* bail out on mismatches */
-    return 0;
-  
+  guint rating = 0;
   rating |= 256 - type_distance;
   rating <<= 1;
   rating |= is_editable;
@@ -477,6 +580,8 @@ param_score_editor (const GxkParamEditor *editor,
   rating <<= 8;
   rating += bonus;      /* bonus is provided for overcomming additional mismatch possibilities */
   
+  if (annotate)
+    g_printerr ("%d\n", rating);
   return rating;
 }
 
@@ -492,7 +597,7 @@ param_lookup_editor (const gchar *editor_name,
       GxkParamEditor *editor = slist->data;
       if (param_editor_name_match (editor_name, editor))
         {
-          guint r = param_score_editor (editor, pspec);
+          guint r = param_score_editor (editor, pspec, FALSE);
           if (r > rating)       /* only notice improvements */
             {
               best = editor;
@@ -500,8 +605,10 @@ param_lookup_editor (const gchar *editor_name,
             }
         }
     }
+#if 0
   if (!best)                    /* if editor_name!=NULL, best might be NULL */
     best = param_lookup_editor (NULL, pspec);
+#endif
   return best;
 }
 
@@ -518,13 +625,34 @@ gxk_param_editor_score (const gchar *editor_name,
       GxkParamEditor *editor = slist->data;
       if (param_editor_name_match (editor_name, editor))
         {
-          guint r = param_score_editor (editor, pspec);
+          guint r = param_score_editor (editor, pspec, FALSE);
           rating = MAX (r, rating);
         }
     }
   return rating;
 }
 
+void
+gxk_param_editor_debug_score (GParamSpec *pspec)
+{
+  guint rating = 0;             /* threshold for mismatch */
+  GSList *slist;
+  g_return_if_fail (G_IS_PARAM_SPEC (pspec));
+  const gchar *options = g_param_spec_get_options (pspec);
+  g_printerr ("GxkParamEditor: rating for pspec: name=%s fundamental=%s type=%s options=%s nick=\"%s\" blurb=\"%s\"\n",
+              pspec->name,
+              g_type_name (G_TYPE_FUNDAMENTAL (G_PARAM_SPEC_VALUE_TYPE (pspec))),
+              g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)),
+              options ? options : "",
+              g_param_spec_get_nick (pspec), g_param_spec_get_blurb (pspec));
+  for (slist = _param_editor_list; slist; slist = slist->next)
+    {
+      GxkParamEditor *editor = slist->data;
+      guint r = param_score_editor (editor, pspec, TRUE);
+      rating = MAX (r, rating);
+    }
+}
+
 const gchar*
 gxk_param_lookup_editor (const gchar *editor_name,
                          GParamSpec  *pspec)
@@ -539,6 +667,8 @@ gxk_param_create_editor (GxkParam               *param,
                          const gchar            *editor_name)
 {
   GxkParamEditor *editor = param_lookup_editor (editor_name, param->pspec);
+  if (!editor)
+    return NULL;
   gchar *tooltip = gxk_param_dup_tooltip (param);
   GtkWidget *widget, *toplevel;
   gboolean updating = param->updating;
@@ -556,6 +686,8 @@ gxk_param_create_editor (GxkParam               *param,
 
 /* --- param editor sizes --- */
 static const GxkParamEditorSizes param_editor_default_sizes = {
+  TRUE,         /* may_resize */
+  FALSE,        /* request_fractions */
   1, 3,         /* char */
   0, 3,         /* uchar */
   1, 10,        /* int */
@@ -572,7 +704,6 @@ static const GxkParamEditorSizes param_editor_default_sizes = {
   2, 7,         /* float */
   2, 17,        /* double */
   8, 2,         /* string */
-  TRUE,         /* may_shrink */
 };
 static GxkParamEditorSizes *size_groups = NULL;
 static guint                n_size_groups = 0;
@@ -581,11 +712,12 @@ guint
 gxk_param_create_size_group (void)
 {
   guint i;
-  g_assert (n_size_groups < 0xffff);
+  g_assert (n_size_groups < 0xff);
   i = n_size_groups++;
   size_groups = g_renew (GxkParamEditorSizes, size_groups, n_size_groups);
   size_groups[i] = param_editor_default_sizes;
-  size_groups[i].may_shrink = FALSE;
+  size_groups[i].may_resize = FALSE;
+  size_groups[i].request_fractions = FALSE;
   return n_size_groups;
 }
 
@@ -601,6 +733,7 @@ void
 gxk_param_set_size_group (GxkParam                  *param,
                           guint                      size_group)
 {
+  g_return_if_fail (GXK_IS_PARAM (param));
   g_return_if_fail (size_group <= n_size_groups);
   param->size_group = size_group;
 }
@@ -646,6 +779,7 @@ _gxk_init_params (void)
   gxk_param_register_editor (&param_spinner1, GXK_I18N_DOMAIN);
   gxk_param_register_editor (&param_spinner2, GXK_I18N_DOMAIN);
   gxk_param_register_editor (&param_toggle, GXK_I18N_DOMAIN);
+  gxk_param_register_editor (&param_toggle_empty, GXK_I18N_DOMAIN);
 }
 
 /* --- param implementation utils --- */
@@ -765,7 +899,8 @@ gxk_param_get_adjustment_with_stepping (GxkParam  *param,
   gboolean isint = TRUE;
   GSList *slist;
   for (slist = param->objects; slist; slist = slist->next)
-    if (GTK_IS_ADJUSTMENT (slist->data))
+    if (GTK_IS_ADJUSTMENT (slist->data) &&
+        g_object_get_data (slist->data, "gxk-param-func") == gxk_param_get_adjustment_with_stepping)
       return slist->data;
   
 #define EXTRACT_FIELDS(p,T,mi,ma,df) { T *t = (T*) p; mi = t->minimum; ma = t->maximum; df = t->default_value; }
@@ -844,6 +979,8 @@ gxk_param_get_adjustment_with_stepping (GxkParam  *param,
   g_object_connect (adjustment,
                     "signal_after::value-changed", param_adjustment_value_changed, param,
                     NULL);
+  /* recognize adjustments created by this function */
+  g_object_set_data (adjustment, "gxk-param-func", gxk_param_get_adjustment_with_stepping);
   return GTK_ADJUSTMENT (adjustment);
 }
 
@@ -859,7 +996,8 @@ gxk_param_get_log_adjustment (GxkParam *param)
   GtkAdjustment *adjustment;
   GSList *slist;
   for (slist = param->objects; slist; slist = slist->next)
-    if (GXK_IS_LOG_ADJUSTMENT (slist->data))
+    if (GXK_IS_LOG_ADJUSTMENT (slist->data) &&
+        g_object_get_data (slist->data, "gxk-param-func") == gxk_param_get_log_adjustment)
       return slist->data;
   adjustment = gxk_param_get_adjustment (param);
   if (adjustment)
@@ -872,11 +1010,70 @@ gxk_param_get_log_adjustment (GxkParam *param)
           gxk_log_adjustment_setup (GXK_LOG_ADJUSTMENT (log_adjustment), center, base, n_steps);
           gxk_param_add_object (param, object);
           gtk_object_sink (object);
-          /* keep the linear adjustment before the log_adjustment */
-          param->objects = g_slist_remove (param->objects, adjustment);
-          param->objects = g_slist_prepend (param->objects, adjustment);
+          /* recognize adjustments created by this function */
+          g_object_set_data (log_adjustment, "gxk-param-func", gxk_param_get_log_adjustment);
           return log_adjustment;
         }
     }
   return NULL;
 }
+
+typedef struct {
+  gdouble mindb, maxdb;
+} GxkParamDBData;
+
+static gdouble
+gxk_param_db_adjustment_convert (GxkAdapterAdjustment           *self,
+                                 GxkAdapterAdjustmentConvertType convert_type,
+                                 gdouble                         value,
+                                 gpointer                        data)
+{
+  GxkParamDBData *dbdata = data;
+  switch (convert_type)
+    {
+    case GXK_ADAPTER_ADJUSTMENT_CONVERT_TO_CLIENT:
+      return pow (10, value / 20);
+    default:
+    case GXK_ADAPTER_ADJUSTMENT_CONVERT_FROM_CLIENT:
+      value = 20 * log10 (value);
+      value = CLAMP (value, dbdata->mindb, dbdata->maxdb);
+      return value;
+    case GXK_ADAPTER_ADJUSTMENT_CONVERT_STEP_INCREMENT:
+      return MIN (1, (dbdata->maxdb - dbdata->mindb) / 10.0);
+    case GXK_ADAPTER_ADJUSTMENT_CONVERT_PAGE_INCREMENT:
+      return MIN (6, (dbdata->maxdb - dbdata->mindb) / 2.0);
+    case GXK_ADAPTER_ADJUSTMENT_CONVERT_PAGE_SIZE:
+      return 0;
+    }
+}
+
+GtkAdjustment*
+gxk_param_get_decibel_adjustment (GxkParam *param)
+{
+  GtkAdjustment *adjustment;
+  GSList *slist;
+  for (slist = param->objects; slist; slist = slist->next)
+    if (GXK_IS_LOG_ADJUSTMENT (slist->data) &&
+        g_object_get_data (slist->data, "gxk-param-func") == gxk_param_get_decibel_adjustment)
+      return slist->data;
+  adjustment = gxk_param_get_adjustment (param);
+  if (adjustment && adjustment->lower >= 0 &&
+      !g_param_spec_check_option (param->pspec, "db-value"))
+    {
+      GxkParamDBData dbdata = { 0, };
+      dbdata.maxdb = 20 * log10 (adjustment->upper);
+      dbdata.mindb = 20 * log10 (adjustment->lower);
+      const double minimum_db = -144.494397918711; /* 24bit: -144.494397918711 32bit: -192.659197224948 */
+      if (adjustment->lower <= 0 || dbdata.mindb < minimum_db)
+        dbdata.mindb = minimum_db;
+      GtkAdjustment *db_adjustment = gxk_adapter_adjustment_from_adj (adjustment, gxk_param_db_adjustment_convert,
+                                                                      g_memdup (&dbdata, sizeof (dbdata)), g_free);
+      GtkObject *object = GTK_OBJECT (db_adjustment);
+      gxk_param_add_object (param, object);
+      gtk_object_sink (object);
+      /* recognize adjustments created by this function */
+      g_object_set_data (db_adjustment, "gxk-param-func", gxk_param_get_decibel_adjustment);
+      return db_adjustment;
+    }
+  return NULL;
+}
