@@ -1,5 +1,5 @@
 /* GXK - Gtk+ Extension Kit
- * Copyright (C) 2002-2003 Tim Janik
+ * Copyright (C) 2002-2004 Tim Janik
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -50,9 +50,10 @@ typedef struct {
 } EnvSpecials;
 
 typedef struct {
-  GSList       *args_list; /* GxkRadgetArgs* */
+  GSList       *args_list;              /* GxkRadgetArgs* */
   const gchar  *name;
-  EnvSpecials  *specials;
+  EnvSpecials  *specials;               /* per expression level */
+  guint         skip_property : 1;      /* per call */
   GData        *hgroups, *vgroups, *hvgroups;
   GxkRadget    *xdef_radget;
 } Env;
@@ -75,9 +76,9 @@ struct Node {
   GxkRadgetArgs *prop_args;
   GxkRadgetArgs *pack_args;
   GxkRadgetArgs *dfpk_args;
-  const gchar  *size_hgroup;
-  const gchar  *size_vgroup;
-  const gchar  *size_hvgroup;
+  GxkRadgetArgs *hook_args;
+  const gchar  *size_hgroup, *size_vgroup, *size_hvgroup;
+  const gchar  *size_window_hgroup, *size_window_vgroup, *size_window_hvgroup;
   const gchar *default_area;
   GSList       *children;       /* Node* */
   GSList       *call_stack;     /* expanded call_args */
@@ -112,10 +113,13 @@ static Node*           node_children_find_area  (Node                *node,
 static const gchar*    radget_args_lookup_quark (const GxkRadgetArgs *args,
                                                  GQuark               quark,
                                                  guint               *nth);
+static GParamSpec*     find_hook                (const gchar         *name,
+                                                 GxkRadgetHook       *hook_func_p);
 
 
 /* --- variables --- */
 static Domain *standard_domain = NULL;
+static GQuark  quark_id = 0;
 static GQuark  quark_name = 0;
 static GQuark  quark_radget_type = 0;
 static GQuark  quark_radget_node = 0;
@@ -220,9 +224,13 @@ clone_node_intern (Node        *source,
   node->prop_args = clone_args (source->prop_args);
   node->pack_args = clone_args (source->pack_args);
   node->dfpk_args = clone_args (source->dfpk_args);
+  node->hook_args = clone_args (source->hook_args);
   node->size_hgroup = source->size_hgroup;
   node->size_vgroup = source->size_vgroup;
   node->size_hvgroup = source->size_hvgroup;
+  node->size_window_hgroup = source->size_window_hgroup;
+  node->size_window_vgroup = source->size_window_vgroup;
+  node->size_window_hvgroup = source->size_window_hvgroup;
   node->default_area = source->default_area;
   for (slist = source->children; slist; slist = slist->next)
     {
@@ -393,9 +401,9 @@ env_lookup (Env         *env,
 {
   const GxkRadgetArgs *args;
   GQuark quark = g_quark_try_string (var);
-  guint nth;
-  if (strcmp (var, "name") == 0)
+  if (quark == quark_name || quark == quark_id)
     return env->name;
+  guint nth;
   args = quark ? env_find_quark (env, quark, &nth) : NULL;
   return args ? ARGS_NTH_VALUE (args, nth) : NULL;
 }
@@ -512,7 +520,7 @@ parse_dollar (const gchar *c,
       var = g_strndup (mark, c - mark);
       quark = g_quark_try_string (var);
       g_free (var);
-      if (quark == quark_name)
+      if (quark == quark_name || quark == quark_id)
         g_string_append (result, env->name);
       else
         {
@@ -551,7 +559,9 @@ expand_expr (const gchar *expr,
     }
   g_string_append (result, c);
   env->specials = saved_specials;
-  if (specials.null_collapse && !result->str[0])
+  gboolean null_collapse = specials.null_collapse;
+  specials.null_collapse = FALSE;
+  if (null_collapse && !result->str[0])
     return g_string_free (result, TRUE);
   else
     return g_string_free (result, FALSE);
@@ -716,7 +726,10 @@ node_define (Domain       *domain,
       node->pack_args = radget_args_intern_set (node->pack_args, attribute_names[i] + 5, attribute_values[i]);
     else if (strncmp (attribute_names[i], "default-pack:", 13) == 0)
       node->dfpk_args = radget_args_intern_set (node->dfpk_args, attribute_names[i] + 13, attribute_values[i]);
-    else if (strcmp (attribute_names[i], "name") == 0 || strcmp (attribute_names[i], "_name") == 0)
+    else if (strncmp (attribute_names[i], "hook:", 5) == 0)
+      node->hook_args = radget_args_intern_set (node->hook_args, attribute_names[i] + 5, attribute_values[i]);
+    else if (strcmp (attribute_names[i], "name") == 0 || strcmp (attribute_names[i], "_name") == 0 ||
+             strcmp (attribute_names[i], "id") == 0 || strcmp (attribute_names[i], "_id") == 0)
       {
         if (name_p && !*name_p)
           *name_p = g_intern_string (attribute_values[i]);
@@ -729,6 +742,12 @@ node_define (Domain       *domain,
       node->size_vgroup = g_intern_string (attribute_values[i]);
     else if (strcmp (attribute_names[i], "size:hvgroup") == 0 && g_type_is_a (node->type, GTK_TYPE_WIDGET))
       node->size_hvgroup = g_intern_string (attribute_values[i]);
+    else if (strcmp (attribute_names[i], "size:window-hgroup") == 0 && g_type_is_a (node->type, GTK_TYPE_WIDGET))
+      node->size_window_hgroup = g_intern_string (attribute_values[i]);
+    else if (strcmp (attribute_names[i], "size:window-vgroup") == 0 && g_type_is_a (node->type, GTK_TYPE_WIDGET))
+      node->size_window_vgroup = g_intern_string (attribute_values[i]);
+    else if (strcmp (attribute_names[i], "size:window-hvgroup") == 0 && g_type_is_a (node->type, GTK_TYPE_WIDGET))
+      node->size_window_hvgroup = g_intern_string (attribute_values[i]);
     else if (strcmp (attribute_names[i], "inherit") == 0 ||
              strcmp (attribute_names[i], "default-area") == 0 ||
              strcmp (attribute_names[i], "area") == 0)
@@ -957,6 +976,57 @@ gxk_radget_parse_text (const gchar    *domain_name,
     }
 }
 
+static GtkSizeGroup*
+toplevel_get_size_group (GtkWidget   *toplevel,
+                         const gchar *name,
+                         gchar        type)
+{
+  gchar *key = g_strdup_printf ("gxk-toplevel-%cgroup#%s", type, name);
+  GtkSizeGroup *sg = g_object_get_data (toplevel, key);
+  if (!sg)
+    {
+      sg = gtk_size_group_new (type == 'h' ? GTK_SIZE_GROUP_HORIZONTAL :
+                               type == 'v' ? GTK_SIZE_GROUP_VERTICAL :
+                               GTK_SIZE_GROUP_BOTH);
+      g_object_set_data_full (toplevel, key, sg, g_object_unref);
+    }
+  g_free (key);
+  return sg;
+}
+
+static void
+radget_widget_hierarchy_changed (GtkWidget *widget,
+                                 GtkWidget *previous_toplevel)
+{
+  if (previous_toplevel)
+    return;
+  const gchar *hgroup = g_object_get_data (widget, "gxk-window-hgroup");
+  const gchar *vgroup = g_object_get_data (widget, "gxk-window-vgroup");
+  const gchar *bgroup = g_object_get_data (widget, "gxk-window-hvgroup");
+  if (!hgroup && !vgroup && !bgroup)
+    return;
+  GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
+  g_assert (GTK_WIDGET_TOPLEVEL (toplevel));
+  if (hgroup)
+    {
+      GtkSizeGroup *sg = toplevel_get_size_group (toplevel, hgroup, 'h');
+      gtk_size_group_add_widget (sg, widget);
+      g_object_set_data (widget, "gxk-window-hgroup", NULL);    /* gtk_size_group_add_widget() <= 2.4.4 may not be called twice */
+    }
+  if (vgroup)
+    {
+      GtkSizeGroup *sg = toplevel_get_size_group (toplevel, vgroup, 'v');
+      gtk_size_group_add_widget (sg, widget);
+      g_object_set_data (widget, "gxk-window-vgroup", NULL);    /* gtk_size_group_add_widget() <= 2.4.4 may not be called twice */
+    }
+  if (bgroup)
+    {
+      GtkSizeGroup *sg = toplevel_get_size_group (toplevel, bgroup, 'b');
+      gtk_size_group_add_widget (sg, widget);
+      g_object_set_data (widget, "gxk-window-hvgroup", NULL);   /* gtk_size_group_add_widget() <= 2.4.4 may not be called twice */
+    }
+}
+
 static void
 property_value_from_string (GtkType      widget_type,
                             GParamSpec  *pspec,
@@ -978,7 +1048,14 @@ property_value_from_string (GtkType      widget_type,
       vtype = PANGO_TYPE_WEIGHT;
       edefault = G_PARAM_SPEC_INT (pspec)->default_value;
     }
+  env->skip_property = FALSE;
   exvalue = expand_expr (pvalue, env);
+  if (env->skip_property)
+    {
+      env->skip_property = FALSE;
+      g_free (exvalue);
+      return;
+    }
   switch (G_TYPE_FUNDAMENTAL (vtype))
     {
       GEnumClass *eclass;
@@ -1123,20 +1200,50 @@ radget_create_from_node (Node         *node,
 {
   GxkRadgetType tinfo;
   guint i, n_pops = 0;
-  /* prepare for $name lookups */
+  /* prepare for $id/$name lookups */
   env->name = node->name;
   /* retrive type info */
   if (!gxk_radget_type_lookup (node->type, &tinfo))
     g_error ("invalid radget type: %s", g_type_name (node->type));
+  /* precedence for property value lookups:
+   * - all node ancestry args
+   * - expanded call_args
+   */
+  if (node->call_stack)
+    n_pops++, env->args_list = g_slist_prepend (env->args_list, node->call_stack->data);
+  if (node->parent_arg_list)
+    n_pops += g_slist_length (node->parent_arg_list), env->args_list = g_slist_concat (g_slist_copy (node->parent_arg_list),
+                                                                                       env->args_list);
   /* create radget */
+  GTypeClass *klass = g_type_class_ref (node->type);
   if (!radget)
     {
+      GParameter *cparams = g_alloca (sizeof (*cparams) * ARGS_N_ENTRIES (node->prop_args));
+      memset (cparams, 0, sizeof (*cparams) * ARGS_N_ENTRIES (node->prop_args));
+      guint n_cparams = 0;
+      /* construct properties */
+      for (i = 0; i < ARGS_N_ENTRIES (node->prop_args); i++)
+        {
+          const gchar *pname = ARGS_NTH_NAME (node->prop_args, i);
+          const gchar *pvalue = ARGS_NTH_VALUE (node->prop_args, i);
+          GParamSpec *pspec = tinfo.find_prop (klass, pname);
+          if (pspec && (pspec->flags & (G_PARAM_CONSTRUCT | G_PARAM_CONSTRUCT_ONLY)))
+            {
+              guint j = n_cparams++;
+              cparams[j].name = pspec->name;
+              property_value_from_string (node->type, pspec, &cparams[j].value, pname, pvalue, env, error);
+              if (!G_VALUE_TYPE (&cparams[j].value))
+                n_cparams--;
+            }
+        }
       GxkRadgetData gdgdata;
       gdgdata.node = node;
       gdgdata.call_stack_top = node->call_stack->data;
       gdgdata.xdef_radget = env->xdef_radget;
       gdgdata.env = env;
-      radget = tinfo.create (node->type, node->name, &gdgdata);
+      radget = tinfo.create (node->type, node->name, n_cparams, cparams, &gdgdata);
+      for (i = 0; i < n_cparams; i++)
+        g_value_unset (&cparams[i].value);
     }
   g_object_set_qdata (radget, quark_radget_node, node);
   /* keep global xdef_radget for gdg_data */
@@ -1149,22 +1256,16 @@ radget_create_from_node (Node         *node,
     gtk_size_group_add_widget (env_get_size_group (env, node->size_vgroup, 'v'), radget);
   if (node->size_hvgroup)
     gtk_size_group_add_widget (env_get_size_group (env, node->size_hvgroup, 'b'), radget);
-  /* precedence for property value lookups:
-   * - all node ancestry args
-   * - expanded call_args
-   */
-  if (node->call_stack)
-    n_pops++, env->args_list = g_slist_prepend (env->args_list, node->call_stack->data);
-  if (node->parent_arg_list)
-    n_pops += g_slist_length (node->parent_arg_list), env->args_list = g_slist_concat (g_slist_copy (node->parent_arg_list),
-                                                                                       env->args_list);
+  g_object_set_data (radget, "gxk-window-hgroup", (gchar*) node->size_window_hgroup);
+  g_object_set_data (radget, "gxk-window-vgroup", (gchar*) node->size_window_vgroup);
+  g_object_set_data (radget, "gxk-window-hvgroup", (gchar*) node->size_window_hvgroup);
   /* set properties */
   for (i = 0; i < ARGS_N_ENTRIES (node->prop_args); i++)
     {
       const gchar *pname = ARGS_NTH_NAME (node->prop_args, i);
       const gchar *pvalue = ARGS_NTH_VALUE (node->prop_args, i);
-      GParamSpec *pspec = tinfo.find_prop (radget, pname);
-      if (pspec)
+      GParamSpec *pspec = tinfo.find_prop (klass, pname);
+      if (pspec && !(pspec->flags & (G_PARAM_CONSTRUCT | G_PARAM_CONSTRUCT_ONLY)))
         {
           GValue value = { 0 };
           property_value_from_string (node->type, pspec, &value, pname, pvalue, env, error);
@@ -1174,10 +1275,20 @@ radget_create_from_node (Node         *node,
               g_value_unset (&value);
             }
         }
-      else
+      else if (!pspec)
         set_error (error, "radget \"%s\" has no property: %s", node->name, pname);
     }
+  /* hierarchy-changed setup */
+  if (GTK_IS_WIDGET (radget) &&
+      !gxk_signal_handler_exists (radget, "hierarchy-changed", G_CALLBACK (radget_widget_hierarchy_changed), NULL))
+    {
+      g_signal_connect_after (radget, "hierarchy-changed", G_CALLBACK (radget_widget_hierarchy_changed), NULL);
+      /* check anchored */
+      if (GTK_WIDGET_TOPLEVEL (gtk_widget_get_toplevel (radget)))
+        radget_widget_hierarchy_changed (radget, NULL);
+    }
   /* cleanup */
+  g_type_class_unref (klass);
   while (n_pops--)
     g_slist_pop_head (&env->args_list);
   return radget;
@@ -1247,6 +1358,55 @@ radget_add_to_parent (GxkRadget    *parent,
     g_slist_pop_head (&env->args_list);
 }
 
+static void
+radget_apply_hooks (GxkRadget    *radget,
+                    Env          *env,
+                    GError      **error)
+{
+  Node *cnode = g_object_get_qdata (radget, quark_radget_node);
+  GxkRadgetType tinfo;
+  guint i, n_pops = 0;
+  /* prepare for $name lookups */
+  env->name = cnode->name;
+  /* retrive type info */
+  gxk_radget_type_lookup (cnode->type, &tinfo);
+  /* precedence for property value lookups:
+   * - all node ancestry args
+   * - expanded call_args
+   */
+  if (cnode->call_stack)
+    n_pops++, env->args_list = g_slist_prepend (env->args_list, cnode->call_stack->data);
+  if (cnode->parent_arg_list)
+    n_pops += g_slist_length (cnode->parent_arg_list), env->args_list = g_slist_concat (g_slist_copy (cnode->parent_arg_list),
+                                                                                        env->args_list);
+  /* set hook args */
+  const GxkRadgetArgs *args = cnode->hook_args;
+  for (i = 0; i < ARGS_N_ENTRIES (args); i++)
+    {
+      const gchar *hname = ARGS_NTH_NAME (args, i);
+      const gchar *hvalue = ARGS_NTH_VALUE (args, i);
+      GxkRadgetHook hook_func;
+      GParamSpec *pspec = find_hook (hname, &hook_func);
+      if (pspec)
+        {
+          GValue value = { 0 };
+          property_value_from_string (0, pspec, &value, hname, hvalue, env, error);
+          if (G_VALUE_TYPE (&value))
+            {
+              /* we always assume G_PARAM_LAX_VALIDATION */
+              g_param_value_validate (pspec, &value);
+              hook_func (radget, pspec->param_id, &value, pspec);
+              g_value_unset (&value);
+            }
+        }
+      else
+        g_printerr ("GXK: no such hook property: %s (radget=%s)\n", hname, G_OBJECT_TYPE_NAME (radget));
+    }
+  /* cleanup */
+  while (n_pops--)
+    g_slist_pop_head (&env->args_list);
+}
+
 static void
 radget_create_children (GxkRadget    *parent,
                         Env          *env,
@@ -1267,6 +1427,7 @@ radget_create_children (GxkRadget    *parent,
       if (cnode->children)
         radget_create_children (radget, env, error);
       radget_add_to_parent (parent, radget, env, error);
+      radget_apply_hooks (radget, env, error);
       g_slist_pop_head (&cnode->call_stack);
       gxk_radget_free_args (call_args);
     }
@@ -1307,6 +1468,8 @@ radget_creator (GxkRadget          *radget,
             }
           if (parent && radget)
             radget_add_to_parent (parent, radget, &env, &error);
+          if (radget)
+            radget_apply_hooks (radget, &env, &error);
           /* cleanup */
           while (n_pops--)
             g_slist_pop_head (&env.args_list);
@@ -1324,6 +1487,7 @@ radget_creator (GxkRadget          *radget,
   return radget;
 }
 
+/* --- radget args --- */
 GxkRadgetArgs*
 gxk_radget_data_copy_call_args (GxkRadgetData *gdgdata)
 {
@@ -1336,68 +1500,6 @@ gxk_radget_data_copy_call_args (GxkRadgetData *gdgdata)
   return args;
 }
 
-GxkRadget*
-gxk_radget_data_get_scope_radget (GxkRadgetData *gdgdata)
-{
-  return gdgdata->xdef_radget;
-}
-
-GxkRadget*
-gxk_radget_creator (GxkRadget          *radget,
-                    const gchar        *domain_name,
-                    const gchar        *name,
-                    GxkRadget          *parent,
-                    GSList             *call_args,
-                    GSList             *env_args)
-{
-  g_return_val_if_fail (domain_name != NULL, NULL);
-  g_return_val_if_fail (name != NULL, NULL);
-  if (radget)
-    {
-      Node *radget_node = g_object_get_qdata (radget, quark_radget_node);
-      g_return_val_if_fail (radget_node == NULL, NULL);
-    }
-  return radget_creator (radget, domain_name, name, parent, call_args, env_args);
-}
-
-GxkRadget*
-gxk_radget_create (const gchar        *domain_name,
-                   const gchar        *name,
-                   const gchar        *var1,
-                   ...)
-{
-  GxkRadgetArgs *gargs;
-  GxkRadget *radget;
-  GSList olist = { 0, };
-  va_list vargs;
-  va_start (vargs, var1);
-  gargs = gxk_radget_args_valist (var1, vargs);
-  olist.data = gargs;
-  radget = gxk_radget_creator (NULL, domain_name, name, NULL, &olist, NULL);
-  gxk_radget_free_args (gargs);
-  va_end (vargs);
-  return radget;
-}
-
-GxkRadget*
-gxk_radget_complete (GxkRadget          *radget,
-                     const gchar        *domain_name,
-                     const gchar        *name,
-                     const gchar        *var1,
-                     ...)
-{
-  GxkRadgetArgs *gargs;
-  GSList olist = { 0, };
-  va_list vargs;
-  va_start (vargs, var1);
-  gargs = gxk_radget_args_valist (var1, vargs);
-  olist.data = gargs;
-  radget = gxk_radget_creator (radget, domain_name, name, NULL, &olist, NULL);
-  gxk_radget_free_args (gargs);
-  va_end (vargs);
-  return radget;
-}
-
 GxkRadgetArgs*
 gxk_radget_const_args (void)
 {
@@ -1518,6 +1620,69 @@ gxk_radget_free_args (GxkRadgetArgs *args)
     }
 }
 
+/* --- radget functions --- */
+GxkRadget*
+gxk_radget_data_get_scope_radget (GxkRadgetData *gdgdata)
+{
+  return gdgdata->xdef_radget;
+}
+
+GxkRadget*
+gxk_radget_creator (GxkRadget          *radget,
+                    const gchar        *domain_name,
+                    const gchar        *name,
+                    GxkRadget          *parent,
+                    GSList             *call_args,
+                    GSList             *env_args)
+{
+  g_return_val_if_fail (domain_name != NULL, NULL);
+  g_return_val_if_fail (name != NULL, NULL);
+  if (radget)
+    {
+      Node *radget_node = g_object_get_qdata (radget, quark_radget_node);
+      g_return_val_if_fail (radget_node == NULL, NULL);
+    }
+  return radget_creator (radget, domain_name, name, parent, call_args, env_args);
+}
+
+GxkRadget*
+gxk_radget_create (const gchar        *domain_name,
+                   const gchar        *name,
+                   const gchar        *var1,
+                   ...)
+{
+  GxkRadgetArgs *gargs;
+  GxkRadget *radget;
+  GSList olist = { 0, };
+  va_list vargs;
+  va_start (vargs, var1);
+  gargs = gxk_radget_args_valist (var1, vargs);
+  olist.data = gargs;
+  radget = gxk_radget_creator (NULL, domain_name, name, NULL, &olist, NULL);
+  gxk_radget_free_args (gargs);
+  va_end (vargs);
+  return radget;
+}
+
+GxkRadget*
+gxk_radget_complete (GxkRadget          *radget,
+                     const gchar        *domain_name,
+                     const gchar        *name,
+                     const gchar        *var1,
+                     ...)
+{
+  GxkRadgetArgs *gargs;
+  GSList olist = { 0, };
+  va_list vargs;
+  va_start (vargs, var1);
+  gargs = gxk_radget_args_valist (var1, vargs);
+  olist.data = gargs;
+  radget = gxk_radget_creator (radget, domain_name, name, NULL, &olist, NULL);
+  gxk_radget_free_args (gargs);
+  va_end (vargs);
+  return radget;
+}
+
 const gchar*
 gxk_radget_get_domain (GxkRadget *radget)
 {
@@ -1624,6 +1789,7 @@ _gxk_init_radget_types (void)
 {
   GType types[1024], *t = types;
   g_assert (quark_radget_type == 0);
+  quark_id = g_quark_from_static_string ("id");
   quark_name = g_quark_from_static_string ("name");
   quark_radget_type = g_quark_from_static_string ("GxkRadget-type");
   quark_radget_node = g_quark_from_static_string ("GxkRadget-node");
@@ -1631,6 +1797,7 @@ _gxk_init_radget_types (void)
   standard_domain->domain = g_intern_string ("standard");
   g_datalist_set_data (&domains, standard_domain->domain, standard_domain);
   *t++ = GTK_TYPE_WINDOW;       *t++ = GTK_TYPE_ARROW;          *t++ = GTK_TYPE_SCROLLED_WINDOW;
+  *t++ = GTK_TYPE_VIEWPORT;
   *t++ = GTK_TYPE_TABLE;        *t++ = GTK_TYPE_FRAME;          *t++ = GTK_TYPE_ALIGNMENT;
   *t++ = GTK_TYPE_NOTEBOOK;     *t++ = GTK_TYPE_BUTTON;         *t++ = GTK_TYPE_MENU_BAR;
   *t++ = GTK_TYPE_TREE_VIEW;    *t++ = GTK_TYPE_LABEL;          *t++ = GTK_TYPE_PROGRESS_BAR;
@@ -1639,10 +1806,11 @@ _gxk_init_radget_types (void)
   *t++ = GTK_TYPE_HBOX;         *t++ = GTK_TYPE_VBOX;           *t++ = GXK_TYPE_MENU_BUTTON;
   *t++ = GTK_TYPE_CHECK_BUTTON; *t++ = GTK_TYPE_ENTRY;          *t++ = GXK_TYPE_MENU_ITEM;
   *t++ = GTK_TYPE_HSCROLLBAR;   *t++ = GTK_TYPE_HSCALE;         *t++ = GTK_TYPE_TEAROFF_MENU_ITEM;
-  *t++ = GTK_TYPE_VSCROLLBAR;   *t++ = GTK_TYPE_VSCALE;         *t++ = GXK_TYPE_IMAGE;
+  *t++ = GTK_TYPE_VSCROLLBAR;   *t++ = GTK_TYPE_VSCALE;         *t++ = GXK_TYPE_FREE_RADIO_BUTTON;
   *t++ = GTK_TYPE_VSEPARATOR;   *t++ = GXK_TYPE_SIMPLE_LABEL;   *t++ = GTK_TYPE_HSEPARATOR;
-  *t++ = GTK_TYPE_HWRAP_BOX;    *t++ = GTK_TYPE_VWRAP_BOX;      *t++ = GXK_TYPE_FREE_RADIO_BUTTON;
-  *t++ = GXK_TYPE_RACK_TABLE;   *t++ = GXK_TYPE_RACK_ITEM;	*t++ = GXK_TYPE_BACK_SHADE;
+  *t++ = GTK_TYPE_HWRAP_BOX;    *t++ = GTK_TYPE_VWRAP_BOX;      *t++ = GXK_TYPE_SCROLLED_WINDOW;
+  *t++ = GXK_TYPE_IMAGE;        *t++ = GXK_TYPE_BACK_SHADE;
+  *t++ = GXK_TYPE_RACK_TABLE;   *t++ = GXK_TYPE_RACK_ITEM;
   while (t-- > types)
     gxk_radget_define_widget_type (*t);
   radget_define_gtk_menu ();
@@ -1681,19 +1849,23 @@ gxk_radget_define_type (GType                type,
 
 
 /* --- widget types --- */
+static GParamSpec*
+widget_find_prop (GTypeClass   *klass,
+                  const gchar  *construct_param_name)
+{
+  return g_object_class_find_property (G_OBJECT_CLASS (klass), construct_param_name);
+}
+
 static GxkRadget*
 widget_create (GType               type,
                const gchar        *name,
+               guint               n_construct_params,
+               GParameter         *construct_params,
                GxkRadgetData      *gdgdata)
 {
-  return g_object_new (type, "name", name, NULL);
-}
-
-static GParamSpec*
-widget_find_prop (GxkRadget    *radget,
-                  const gchar  *prop_name)
-{
-  return g_object_class_find_property (G_OBJECT_GET_CLASS (radget), prop_name);
+  GtkWidget *widget = g_object_newv (type, n_construct_params, construct_params);
+  g_object_set (widget, "name", name, NULL);
+  return widget;
 }
 
 static gboolean
@@ -1726,8 +1898,8 @@ void
 gxk_radget_define_widget_type (GType type)
 {
   static const GxkRadgetType widget_info = {
-    widget_create,
     widget_find_prop,
+    widget_create,
     (void(*)(GxkRadget*,const gchar*,const GValue*)) g_object_set_property,
     widget_adopt,
     widget_find_pack,
@@ -1739,7 +1911,7 @@ gxk_radget_define_widget_type (GType type)
     {   "prop:width-request",   "$(first-occupied,$width,0)" },
     {   "prop:height-request",  "$(first-occupied,$height,0)" },
     {   "prop:events",          "$events" },
-    {   "prop:can-focus",       "$(first-occupied,$can-focus,$focus,1)" },
+    {   "prop:can-focus",       "$(first-occupied,$can-focus,$focus,$(skip-property))" },
     {   "prop:has-focus",       "$(first-occupied,$has-focus,$focus,0)" },
     {   "prop:can-default",     "$(first-occupied,$can-default,$default,0)" },
     {   "prop:has-default",     "$(first-occupied,$has-default,$default,0)" },
@@ -1799,8 +1971,8 @@ static void
 radget_define_gtk_menu (void)
 {
   static const GxkRadgetType widget_info = {
-    widget_create,
     widget_find_prop,
+    widget_create,
     (void(*)(GxkRadget*,const gchar*,const GValue*)) g_object_set_property,
     menu_adopt,
     (void*) return_NULL,/* find_pack */
@@ -1815,6 +1987,53 @@ radget_define_gtk_menu (void)
   radget_define_type (type, g_type_name (type), attribute_names, attribute_values, NULL);
 }
 
+/* --- radget hooks --- */
+typedef struct RadgetHook RadgetHook;
+struct RadgetHook {
+  GParamSpec          *pspec;
+  GxkRadgetHook        hook_func;
+  RadgetHook          *next;
+};
+static RadgetHook *radget_hooks = NULL;
+
+void
+gxk_radget_register_hook (GParamSpec   *pspec,
+                          guint         property_id,
+                          GxkRadgetHook hook_func)
+{
+  g_return_if_fail (G_IS_PARAM_SPEC (pspec));
+  g_return_if_fail (pspec->flags & G_PARAM_WRITABLE);
+  g_return_if_fail (property_id > 0);
+  g_return_if_fail (pspec->param_id == 0);
+  g_return_if_fail (pspec->owner_type == 0);
+  g_return_if_fail (hook_func != NULL);
+  if (find_hook (pspec->name, NULL))
+    {
+      g_printerr ("GXK: not re-registering hook property: %s\n", pspec->name);
+      return;
+    }
+  RadgetHook *hook = g_new0 (RadgetHook, 1);
+  hook->pspec = g_param_spec_ref (pspec);
+  g_param_spec_sink (pspec);
+  g_quark_from_static_string (pspec->name);
+  hook->pspec->param_id = property_id;
+  hook->pspec->owner_type = G_TYPE_OBJECT; /* common base for all radgets */
+  hook->hook_func = hook_func;
+  hook->next = radget_hooks;
+  radget_hooks = hook;
+}
+
+static GParamSpec*
+find_hook (const gchar   *name,
+           GxkRadgetHook *hook_func_p)
+{
+  RadgetHook *hook = radget_hooks;
+  while (hook && strcmp (hook->pspec->name, name) != 0)
+    hook = hook->next;
+  if (hook && hook_func_p)
+    *hook_func_p = hook->hook_func;
+  return hook ? hook->pspec : NULL;
+}
 
 /* --- macro functions --- */
 static inline const gchar*
@@ -1923,11 +2142,45 @@ mf_floatcmp (GSList *args,
           result = match;
           break;
         }
+      last = v;
       arg = argiter_exp (&argiter, env);
     }
   return g_strdup (result ? "1" : "0");
 }
 
+static gchar*
+mf_floatcollect (GSList *args,
+                 Env    *env)
+{
+  GSList      *argiter = args;
+  const gchar *name = argiter_pop (&argiter);
+  gchar *arg = argiter_exp (&argiter, env);
+  double last = arg ? float_from_string (arg) : 0;
+  guint n = arg ? 1 : 0;
+  double accu = last;
+  g_free (arg);
+  arg = argiter_exp (&argiter, env);
+  while (arg)
+    {
+      double v = float_from_string (arg);
+      g_free (arg);
+      if (strcmp (name, "max") == 0)
+        accu = MAX (accu, v);
+      else if (strcmp (name, "min") == 0)
+        accu = MIN (accu, v);
+      else if (strcmp (name, "avg") == 0)
+        accu += v;
+      else if (strcmp (name, "sum") == 0)
+        accu += v;
+      last = v;
+      n++;
+      arg = argiter_exp (&argiter, env);
+    }
+  if (strcmp (name, "avg") == 0 && n)
+    accu /= n;
+  return g_strdup_printf ("%.17g", accu);
+}
+
 static gchar*
 mf_first_occupied (GSList *args,
                    Env    *env)
@@ -1981,6 +2234,14 @@ mf_null_collapse (GSList *args,
   return value;
 }
 
+static gchar*
+mf_skip_property (GSList *args,
+                  Env    *env)
+{
+  env->skip_property = TRUE;
+  return g_strdup ("");
+}
+
 static gchar*
 mf_empty (GSList *args,
           Env    *env)
@@ -2027,10 +2288,15 @@ macro_func_lookup (const gchar *name)
     { "ge",             mf_floatcmp, },
     { "ne",             mf_floatcmp, },
     { "eq",             mf_floatcmp, },
+    { "min",            mf_floatcollect, },
+    { "max",            mf_floatcollect, },
+    { "sum",            mf_floatcollect, },
+    { "avg",            mf_floatcollect, },
     { "nth",            mf_nth, },
     { "ifdef",          mf_ifdef, },
     { "first-occupied", mf_first_occupied, },
     { "null-collapse",  mf_null_collapse, },
+    { "skip-property",  mf_skip_property, },
     { "empty",          mf_empty, },
     { "println",        mf_println, },
   };
