@@ -31,6 +31,7 @@ G_BEGIN_DECLS
 
 
 /* --- Gtk+ workarounds and amendments --- */
+gboolean        gxk_widget_ancestry_viewable       (GtkWidget       *widget);
 void            gxk_cell_editable_is_focus_handler (GtkCellEditable *ecell);
 gboolean        gxk_cell_editable_canceled         (GtkCellEditable *ecell);
 GtkWidget*      gxk_item_factory_sensitize         (GtkItemFactory  *ifactory,
@@ -40,7 +41,9 @@ GtkWidget*      gxk_item_factory_get_item          (GtkItemFactory  *ifactory,
                                                     const gchar     *path);
 GtkWidget*      gxk_item_factory_get_widget        (GtkItemFactory  *ifactory,
                                                     const gchar     *path);
-void            gxk_widget_proxy_requisition       (GtkWidget       *widget);
+void            gxk_widget_proxy_requisition       (GtkWidget       *widget,
+                                                    gdouble          xscale,
+                                                    gdouble          yscale);
 void            gxk_widget_request_hclient_height  (GtkWidget       *widget,
                                                     GtkWidget       *client);
 void            gxk_widget_request_vclient_width   (GtkWidget       *widget,
@@ -90,6 +93,8 @@ void            gxk_window_set_geometry_width_inc  (GtkWindow       *window,
                                                     guint            width_increment);
 void            gxk_window_set_geometry_height_inc (GtkWindow       *window,
                                                     guint            height_increment);
+void            gxk_expander_connect_to_widget     (GtkWidget       *expander,
+                                                    GtkWidget       *widget);
 
 guint           gxk_container_get_insertion_slot   (GtkContainer    *container);
 void            gxk_container_slot_reorder_child   (GtkContainer    *container,
@@ -157,7 +162,16 @@ void	 gxk_color_alloc		(GdkColormap		*colormap,
 GdkColor gdk_color_from_rgb             (guint                   rgb_value);
 GdkColor gdk_color_from_argb            (guint                   rgb_value);
 GdkColor gdk_color_from_rgba            (guint                   rgb_value);
-
+void     gdk_draw_hline                 (GdkDrawable            *drawable,
+                                         GdkGC                  *gc,
+                                         gint                    x,
+                                         gint                    y,
+                                         gint                    width);
+void     gdk_draw_vline                 (GdkDrawable            *drawable,
+                                         GdkGC                  *gc,
+                                         gint                    x,
+                                         gint                    y,
+                                         gint                    height);
 
 /* --- Gtk convenience --- */
 #define GTK_STYLE_THICKNESS(s,xy)	((s)-> xy##thickness)
@@ -167,14 +181,31 @@ void         gxk_widget_showraise                 (GtkWidget        *widget);
 void         gxk_idle_showraise                   (GtkWidget        *widget);
 void         gxk_idle_show_widget                 (GtkWidget        *widget);
 void         gxk_idle_unrealize_widget            (GtkWidget        *widget);
-void         gxk_notebook_add_page                (GtkNotebook      *notebook,
-                                                   GtkWidget        *child,
-                                                   const gchar      *tab_text,
-                                                   gboolean          expand_fill);
+GtkWidget*   gxk_notebook_create_tabulator        (const gchar      *label_text,
+                                                   const gchar      *stock_image,
+                                                   const gchar      *tooltip);
+void         gxk_notebook_change_tabulator        (GtkWidget        *tabulator,
+                                                   const gchar      *label_text,
+                                                   const gchar      *stock_image,
+                                                   const gchar      *tooltip);
 void         gxk_notebook_set_current_page_widget (GtkNotebook      *notebook,
                                                    GtkWidget        *page);
 GtkWidget*   gxk_vseparator_space_new             (gboolean          draw_seperator);
 GtkWidget*   gtk_notebook_current_widget          (GtkNotebook      *notebook);
+GtkWidget*   gxk_notebook_descendant_get_page     (GtkWidget        *widget);
+GtkWidget*   gxk_notebook_descendant_get_tab      (GtkWidget        *widget);
+void	     gxk_notebook_append		  (GtkNotebook	    *notebook,
+                                                   GtkWidget	    *child,
+                                                   const gchar	    *label,
+                                                   gboolean          fillexpand);
+GtkWidget*   gtk_box_get_nth_child                (GtkBox           *box,
+                                                   gint              pos);
+void         gxk_scrolled_window_spare_space      (GtkScrolledWindow*scwin);
+void         gxk_scrolled_window_unspare_space    (GtkScrolledWindow*scwin);
+GtkWidget*   gxk_scrolled_window_create           (GtkWidget        *child,
+                                                   GtkShadowType     shadow_type,
+                                                   gdouble           xrequest,
+                                                   gdouble           yrequest);
 
 /* functions to affect a widget tree's toplevel */
 void         gxk_toplevel_delete                  (GtkWidget        *widget);
@@ -182,11 +213,15 @@ void         gxk_toplevel_hide                    (GtkWidget        *widget);
 void         gxk_toplevel_activate_default        (GtkWidget        *widget);
 
 /* widget utilities */
+GtkWidget*   gxk_parent_find_descendant           (GtkWidget        *parent,
+                                                   GType             descendant_type);
 void         gxk_widget_modify_as_title           (GtkWidget        *widget);
+void         gxk_widget_modify_normal_bg_as_base  (GtkWidget        *widget);
 void         gxk_widget_modify_bg_as_base         (GtkWidget        *widget);
 void         gxk_widget_modify_base_as_bg         (GtkWidget        *widget);
+void         gxk_widget_modify_bg_as_active       (GtkWidget        *widget);
 void         gxk_widget_force_bg_clear            (GtkWidget        *widget);
-void         gxk_widget_set_tooltip               (GtkWidget        *widget,
+void         gxk_widget_set_tooltip               (gpointer          widget,
                                                    const gchar      *tooltip);
 void         gxk_widget_set_latent_tooltip        (GtkWidget        *widget,
                                                    const gchar      *tooltip);
@@ -198,54 +233,53 @@ void         gxk_size_group                       (GtkSizeGroupMode  sgmode,
                                                    ...);
 
 /* tree view convenience */
-gint	 gxk_tree_spath_index0			(const gchar		*strpath);
-gboolean gxk_tree_model_get_iter                (GtkTreeModel           *tree_model,
-                                                 GtkTreeIter            *iter,
-                                                 GtkTreePath            *path);
-gboolean gxk_tree_path_prev			(GtkTreePath		*path);
-guint	 gxk_tree_view_add_column		(GtkTreeView		*tree_view,
-						 gint			 position,
-						 GtkTreeViewColumn	*column,
-						 GtkCellRenderer	*cell,
-						 const gchar		*attrib_name,
-						 ...);
-void	 gxk_tree_view_append_text_columns	(GtkTreeView		*tree_view,
-						 guint			 n_cols,
-						 ...);
-void	 gxk_tree_view_add_text_column		(GtkTreeView  *tree_view,
-						 guint	       model_column,
-						 const gchar  *column_flags,
-						 gdouble       xalign,
-						 const gchar  *title,
-						 const gchar  *tooltip,
-						 gpointer      edited_callback,
-						 gpointer      data,
-						 GConnectFlags cflags);
-void	 gxk_tree_view_add_popup_column		(GtkTreeView  *tree_view,
-						 guint	       model_column,
-						 const gchar  *column_flags,
-						 gdouble       xalign,
-						 const gchar  *title,
-						 const gchar  *tooltip,
-						 gpointer      edited_callback,
-						 gpointer      popup_callback,
-						 gpointer      data,
-						 GConnectFlags cflags);
-void	gxk_tree_view_add_toggle_column		(GtkTreeView  *tree_view,
-						 guint	       model_column,
-						 const gchar  *column_flags,
-						 gdouble       xalign,
-						 const gchar  *title,
-						 const gchar  *tooltip,
-						 gpointer      toggled_callback,
-						 gpointer      data,
-						 GConnectFlags cflags);
-void	gxk_tree_view_column_set_tip_title	(GtkTreeViewColumn   *tree_column,
-						 const gchar         *title,
-						 const gchar	     *tooltip);
-void	gxk_tree_view_set_editable      	(GtkTreeView         *tview,
-                                                 gboolean             maybe_editable);
-
+gint	           gxk_tree_spath_index0		(const gchar		*strpath);
+gboolean           gxk_tree_model_get_iter              (GtkTreeModel           *tree_model,
+                                                         GtkTreeIter            *iter,
+                                                         GtkTreePath            *path);
+gboolean           gxk_tree_path_prev			(GtkTreePath		*path);
+guint	           gxk_tree_view_add_column		(GtkTreeView		*tree_view,
+                                                         gint			 position,
+                                                         GtkTreeViewColumn	*column,
+                                                         GtkCellRenderer	*cell,
+                                                         const gchar		*attrib_name,
+                                                         ...);
+void               gxk_tree_view_append_text_columns    (GtkTreeView		*tree_view,
+                                                         guint			 n_cols,
+                                                         ...);
+GtkTreeViewColumn* gxk_tree_view_add_text_column	(GtkTreeView            *tree_view,
+                                                         guint	                 model_column,
+                                                         const gchar            *column_flags,
+                                                         gdouble                 xalign,
+                                                         const gchar            *title,
+                                                         const gchar            *tooltip,
+                                                         gpointer                edited_callback,
+                                                         gpointer                data,
+                                                         GConnectFlags           cflags);
+GtkTreeViewColumn* gxk_tree_view_add_popup_column	(GtkTreeView            *tree_view,
+                                                         guint	                 model_column,
+                                                         const gchar            *column_flags,
+                                                         gdouble                 xalign,
+                                                         const gchar            *title,
+                                                         const gchar            *tooltip,
+                                                         gpointer                edited_callback,
+                                                         gpointer                popup_callback,
+                                                         gpointer                data,
+                                                         GConnectFlags           cflags);
+GtkTreeViewColumn* gxk_tree_view_add_toggle_column	(GtkTreeView            *tree_view,
+                                                         guint	                 model_column,
+                                                         const gchar            *column_flags,
+                                                         gdouble                 xalign,
+                                                         const gchar            *title,
+                                                         const gchar            *tooltip,
+                                                         gpointer                toggled_callback,
+                                                         gpointer                data,
+                                                         GConnectFlags           cflags);
+void	           gxk_tree_view_column_set_tip_title	(GtkTreeViewColumn      *tree_column,
+                                                         const gchar            *title,
+                                                         const gchar	        *tooltip);
+void	           gxk_tree_view_set_editable      	(GtkTreeView            *tview,
+                                                         gboolean                maybe_editable);
 
 /* tree selection convenience */
 void   gxk_tree_selection_select_spath   (GtkTreeSelection      *selection,
@@ -278,38 +312,19 @@ void     gxk_tree_view_focus_row	 (GtkTreeView		*tree,
 gboolean gxk_tree_view_is_row_selected	 (GtkTreeView		*tree,
 					  gint			 row);
 gint     gxk_tree_view_get_selected_row	 (GtkTreeView		*tree);
-
-/* misc widgets */
-void	gxk_notebook_append		(GtkNotebook	*notebook,
-					 GtkWidget	*child,
-					 const gchar	*label);
-
+gboolean gxk_show_url                    (const gchar           *url);
 
 /* --- signal convenience --- */
+gboolean	gxk_signal_handler_exists	(gpointer	 instance,
+						 const gchar	*detailed_signal,
+						 GCallback	 callback,
+						 gpointer	 data);
 gboolean	gxk_signal_handler_pending	(gpointer	 instance,
 						 const gchar	*detailed_signal,
 						 GCallback	 callback,
 						 gpointer	 data);
 
 
-/* --- derivation convenience --- */
-typedef enum /*< skip >*/
-{
-  GXK_METHOD_NONE,
-  GXK_METHOD_INIT,
-  GXK_METHOD_FINALIZE,
-  GXK_METHOD_DISPOSE,
-  GXK_METHOD_DESTROY,
-} GxkMethodType;
-GType	gxk_object_derive	(GType		parent_type,
-				 const gchar   *name,
-				 gpointer      *parent_class_p,
-				 guint          instance_size,
-				 guint          class_size,
-				 GxkMethodType  mtype,
-				 ...);
-
-
 /* --- zlib support --- */
 gchar*  gxk_zfile_uncompress    (guint                uncompressed_size,
                                  const unsigned char *cdata,
