@@ -76,18 +76,6 @@ static GFlagsValue bse_magic_flags_values[] = {
   { 0, NULL, NULL }
 };
 
-/* BseUserMsgType
- */
-static GEnumValue bse_user_msg_type_values[] = {
-  { BSE_USER_MSG_UNSPECIFIED, "BSE_USER_MSG_UNSPECIFIED", "unspecified" },
-  { BSE_USER_MSG_DEBUG, "BSE_USER_MSG_DEBUG", "debug" },
-  { BSE_USER_MSG_DIAG, "BSE_USER_MSG_DIAG", "diag" },
-  { BSE_USER_MSG_INFO, "BSE_USER_MSG_INFO", "info" },
-  { BSE_USER_MSG_WARNING, "BSE_USER_MSG_WARNING", "warning" },
-  { BSE_USER_MSG_ERROR, "BSE_USER_MSG_ERROR", "error" },
-  { 0, NULL, NULL }
-};
-
 /* BseErrorType
  */
 static GEnumValue bse_error_type_values[] = {
@@ -122,15 +110,20 @@ static GEnumValue bse_error_type_values[] = {
   { BSE_ERROR_DEVICE_NOT_AVAILABLE, "BSE_ERROR_DEVICE_NOT_AVAILABLE", "device-not-available" },
   { BSE_ERROR_DEVICE_ASYNC, "BSE_ERROR_DEVICE_ASYNC", "device-async" },
   { BSE_ERROR_DEVICE_BUSY, "BSE_ERROR_DEVICE_BUSY", "device-busy" },
-  { BSE_ERROR_DEVICE_GET_CAPS, "BSE_ERROR_DEVICE_GET_CAPS", "device-get-caps" },
-  { BSE_ERROR_DEVICE_CAPS_MISMATCH, "BSE_ERROR_DEVICE_CAPS_MISMATCH", "device-caps-mismatch" },
-  { BSE_ERROR_DEVICE_SET_CAPS, "BSE_ERROR_DEVICE_SET_CAPS", "device-set-caps" },
+  { BSE_ERROR_DEVICE_FORMAT, "BSE_ERROR_DEVICE_FORMAT", "device-format" },
+  { BSE_ERROR_DEVICE_BUFFER, "BSE_ERROR_DEVICE_BUFFER", "device-buffer" },
+  { BSE_ERROR_DEVICE_LATENCY, "BSE_ERROR_DEVICE_LATENCY", "device-latency" },
+  { BSE_ERROR_DEVICE_CHANNELS, "BSE_ERROR_DEVICE_CHANNELS", "device-channels" },
+  { BSE_ERROR_DEVICE_FREQUENCY, "BSE_ERROR_DEVICE_FREQUENCY", "device-frequency" },
+  { BSE_ERROR_DEVICES_MISMATCH, "BSE_ERROR_DEVICES_MISMATCH", "devices-mismatch" },
   { BSE_ERROR_SOURCE_NO_SUCH_MODULE, "BSE_ERROR_SOURCE_NO_SUCH_MODULE", "source-no-such-module" },
   { BSE_ERROR_SOURCE_NO_SUCH_ICHANNEL, "BSE_ERROR_SOURCE_NO_SUCH_ICHANNEL", "source-no-such-ichannel" },
   { BSE_ERROR_SOURCE_NO_SUCH_OCHANNEL, "BSE_ERROR_SOURCE_NO_SUCH_OCHANNEL", "source-no-such-ochannel" },
   { BSE_ERROR_SOURCE_NO_SUCH_CONNECTION, "BSE_ERROR_SOURCE_NO_SUCH_CONNECTION", "source-no-such-connection" },
+  { BSE_ERROR_SOURCE_PRIVATE_ICHANNEL, "BSE_ERROR_SOURCE_PRIVATE_ICHANNEL", "source-private-ichannel" },
   { BSE_ERROR_SOURCE_ICHANNEL_IN_USE, "BSE_ERROR_SOURCE_ICHANNEL_IN_USE", "source-ichannel-in-use" },
   { BSE_ERROR_SOURCE_CHANNELS_CONNECTED, "BSE_ERROR_SOURCE_CHANNELS_CONNECTED", "source-channels-connected" },
+  { BSE_ERROR_SOURCE_CONNECTION_INVALID, "BSE_ERROR_SOURCE_CONNECTION_INVALID", "source-connection-invalid" },
   { BSE_ERROR_SOURCE_PARENT_MISMATCH, "BSE_ERROR_SOURCE_PARENT_MISMATCH", "source-parent-mismatch" },
   { BSE_ERROR_SOURCE_BAD_LOOPBACK, "BSE_ERROR_SOURCE_BAD_LOOPBACK", "source-bad-loopback" },
   { BSE_ERROR_SOURCE_BUSY, "BSE_ERROR_SOURCE_BUSY", "source-busy" },
@@ -140,6 +133,8 @@ static GEnumValue bse_error_type_values[] = {
   { BSE_ERROR_PROC_PARAM_INVAL, "BSE_ERROR_PROC_PARAM_INVAL", "proc-param-inval" },
   { BSE_ERROR_PROC_EXECUTION, "BSE_ERROR_PROC_EXECUTION", "proc-execution" },
   { BSE_ERROR_PROC_ABORT, "BSE_ERROR_PROC_ABORT", "proc-abort" },
+  { BSE_ERROR_INVALID_PROPERTY, "BSE_ERROR_INVALID_PROPERTY", "invalid-property" },
+  { BSE_ERROR_INVALID_MIDI_CONTROL, "BSE_ERROR_INVALID_MIDI_CONTROL", "invalid-midi-control" },
   { BSE_ERROR_PARSE_ERROR, "BSE_ERROR_PARSE_ERROR", "parse-error" },
   { BSE_ERROR_SPAWN, "BSE_ERROR_SPAWN", "spawn" },
   { BSE_ERROR_NO_ENTRY, "BSE_ERROR_NO_ENTRY", "no-entry" },
@@ -230,18 +225,6 @@ static GEnumValue bse_project_state_values[] = {
 };
 
 
-/* --- bsessequencer.h --- */
-#include	"bsessequencer.h"
-/* BseSSequencerJobType
- */
-static GEnumValue bse_ssequencer_job_type_values[] = {
-  { BSE_SSEQUENCER_JOB_NOP, "BSE_SSEQUENCER_JOB_NOP", "nop" },
-  { BSE_SSEQUENCER_JOB_ADD, "BSE_SSEQUENCER_JOB_ADD", "add" },
-  { BSE_SSEQUENCER_JOB_REMOVE, "BSE_SSEQUENCER_JOB_REMOVE", "remove" },
-  { 0, NULL, NULL }
-};
-
-
 /* --- bsestandardosc.h --- */
 #include	"bsestandardosc.h"
 /* BseStandardOscWaveType
