@@ -14,7 +14,6 @@
   { "BseInterpolType", G_TYPE_ENUM, &BSE_TYPE_ID (BseInterpolType), bse_interpol_type_values },
   { "BseLoopType", G_TYPE_ENUM, &BSE_TYPE_ID (BseLoopType), bse_loop_type_values },
   { "BseMagicFlags", G_TYPE_FLAGS, &BSE_TYPE_ID (BseMagicFlags), bse_magic_flags_values },
-  { "BseUserMsgType", G_TYPE_ENUM, &BSE_TYPE_ID (BseUserMsgType), bse_user_msg_type_values },
   { "BseErrorType", G_TYPE_ENUM, &BSE_TYPE_ID (BseErrorType), bse_error_type_values },
 
 /* --- bseglobals.h --- */
@@ -26,11 +25,11 @@
 /* --- bseproject.h --- */
   { "BseProjectState", G_TYPE_ENUM, &BSE_TYPE_ID (BseProjectState), bse_project_state_values },
 
-/* --- bsessequencer.h --- */
-  { "BseSSequencerJobType", G_TYPE_ENUM, &BSE_TYPE_ID (BseSSequencerJobType), bse_ssequencer_job_type_values },
-
 /* --- bsestandardosc.h --- */
   { "BseStandardOscWaveType", G_TYPE_ENUM, &BSE_TYPE_ID (BseStandardOscWaveType), bse_standard_osc_wave_type_values },
 
+/* --- bsemididecoder.h --- */
+  { "BseMidiDecoderState", G_TYPE_ENUM, &BSE_TYPE_ID (BseMidiDecoderState), bse_midi_decoder_state_values },
+
 /* Generated data ends here */
 
