@@ -54,15 +54,6 @@ typedef enum
   BSE_MAGIC_BSE_SONG            = 1 << 2
 } BseMagicFlags;
 typedef enum
-{
-  BSE_USER_MSG_UNSPECIFIED,
-  BSE_USER_MSG_DEBUG,
-  BSE_USER_MSG_DIAG,
-  BSE_USER_MSG_INFO,
-  BSE_USER_MSG_WARNING,
-  BSE_USER_MSG_ERROR
-} BseUserMsgType;
-typedef enum
 {
   /* GSL errors are mirrored into BSE */
   BSE_ERROR_NONE		= GSL_ERROR_NONE,	/* 0 */
@@ -102,16 +93,21 @@ typedef enum
   BSE_ERROR_DEVICE_NOT_AVAILABLE,
   BSE_ERROR_DEVICE_ASYNC,
   BSE_ERROR_DEVICE_BUSY,
-  BSE_ERROR_DEVICE_GET_CAPS,
-  BSE_ERROR_DEVICE_CAPS_MISMATCH,
-  BSE_ERROR_DEVICE_SET_CAPS,
+  BSE_ERROR_DEVICE_FORMAT,
+  BSE_ERROR_DEVICE_BUFFER,
+  BSE_ERROR_DEVICE_LATENCY,
+  BSE_ERROR_DEVICE_CHANNELS,
+  BSE_ERROR_DEVICE_FREQUENCY,
+  BSE_ERROR_DEVICES_MISMATCH,
   /* BseSource errors */
   BSE_ERROR_SOURCE_NO_SUCH_MODULE,
   BSE_ERROR_SOURCE_NO_SUCH_ICHANNEL,
   BSE_ERROR_SOURCE_NO_SUCH_OCHANNEL,
   BSE_ERROR_SOURCE_NO_SUCH_CONNECTION,
+  BSE_ERROR_SOURCE_PRIVATE_ICHANNEL,
   BSE_ERROR_SOURCE_ICHANNEL_IN_USE,
   BSE_ERROR_SOURCE_CHANNELS_CONNECTED,
+  BSE_ERROR_SOURCE_CONNECTION_INVALID,
   BSE_ERROR_SOURCE_PARENT_MISMATCH,
   BSE_ERROR_SOURCE_BAD_LOOPBACK,
   BSE_ERROR_SOURCE_BUSY,
@@ -123,6 +119,8 @@ typedef enum
   BSE_ERROR_PROC_EXECUTION,
   BSE_ERROR_PROC_ABORT,
   /* miscellaneous errors */
+  BSE_ERROR_INVALID_PROPERTY,
+  BSE_ERROR_INVALID_MIDI_CONTROL,
   BSE_ERROR_PARSE_ERROR,
   BSE_ERROR_SPAWN,
   /* various procedure errors */
@@ -137,12 +135,21 @@ typedef enum
 
 
 /* --- convenience functions --- */
-gchar*		bse_error_name			(BseErrorType	 error_value);
-gchar*		bse_error_nick			(BseErrorType	 error_value);
-gchar*		bse_error_blurb			(BseErrorType	 error_value);
+const gchar*	bse_error_name			(BseErrorType	 error_value);
+const gchar*	bse_error_nick			(BseErrorType	 error_value);
+const gchar*	bse_error_blurb			(BseErrorType	 error_value);
 BseErrorType	bse_error_from_errno		(gint		 v_errno,
 						 BseErrorType    fallback);
 
+#define bse_assert_ok(error)    G_STMT_START{                           \
+     if G_UNLIKELY (error)                                              \
+       {                                                                \
+         g_log (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR,                        \
+                "%s:%d: unexpected error: %s",                          \
+                __FILE__, __LINE__, bse_error_blurb (error));           \
+       }                                                                \
+}G_STMT_END
+
 G_END_DECLS
 
 
