@@ -1,7 +1,7 @@
 
-/**
- ** Generated data (by mkcproc.pl)
- **/
+/*
+ * Generated data (by mkcproc.pl)
+ */
 #line 1 "bsejanitor.proc"
 
 #include <bse/bseplugin.h>
@@ -576,164 +576,248 @@ exit_exec (BseProcedureClass *proc,
   
 #line 418 "bsejanitor.proc"
   
-  bse_server_user_message (bse_server_get (), mtype, message);
+  bse_server_user_message (bse_server_get (), "BSE-SCRIPT", mtype, NULL, message, self->port ? self->port->remote_pid : 0, self->script_name);
   bse_janitor_close (self);
   
   return BSE_ERROR_NONE;
 }
 
 /* --- Export to BSE --- */
+static void
+__enode_trigger_action__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Trigger an installed user action of this janitor.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_trigger_action = {
   { NULL, BSE_EXPORT_NODE_PROC,
     "BseJanitor+trigger-action", 
     NULL,
     "/Methods/BseJanitor/General/Trigger Action",
-    "Trigger an installed user action of this janitor.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_trigger_action__fill_strings,
   },
   0, trigger_action_setup, trigger_action_exec, 
 };
+static void
+__enode_get_script_name__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve the script name of this janitor.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_script_name = {
   { (BseExportNode*) &__enode_trigger_action, BSE_EXPORT_NODE_PROC,
     "BseJanitor+get-script-name", 
     NULL,
     "/Methods/BseJanitor/General/Get Script Name",
-    "Retrieve the script name of this janitor.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_script_name__fill_strings,
   },
   0, get_script_name_setup, get_script_name_exec, 
 };
+static void
+__enode_get_proc_name__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve the procedure name of this janitor.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_proc_name = {
   { (BseExportNode*) &__enode_get_script_name, BSE_EXPORT_NODE_PROC,
     "BseJanitor+get-proc-name", 
     NULL,
     "/Methods/BseJanitor/General/Get Proc Name",
-    "Retrieve the procedure name of this janitor.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_proc_name__fill_strings,
   },
   0, get_proc_name_setup, get_proc_name_exec, 
 };
+static void
+__enode_n_actions__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve number of user actions of this janitor.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_n_actions = {
   { (BseExportNode*) &__enode_get_proc_name, BSE_EXPORT_NODE_PROC,
     "BseJanitor+n-actions", 
     NULL,
     "/Methods/BseJanitor/General/N Actions",
-    "Retrieve number of user actions of this janitor.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_n_actions__fill_strings,
   },
   0, n_actions_setup, n_actions_exec, 
 };
+static void
+__enode_get_action__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve an action of this janitor.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_action = {
   { (BseExportNode*) &__enode_n_actions, BSE_EXPORT_NODE_PROC,
     "BseJanitor+get-action", 
     NULL,
     "/Methods/BseJanitor/General/Get Action",
-    "Retrieve an action of this janitor.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_action__fill_strings,
   },
   0, get_action_setup, get_action_exec, 
 };
+static void
+__enode_get_action_name__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve the name of an action of this janitor.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_action_name = {
   { (BseExportNode*) &__enode_get_action, BSE_EXPORT_NODE_PROC,
     "BseJanitor+get-action-name", 
     NULL,
     "/Methods/BseJanitor/General/Get Action Name",
-    "Retrieve the name of an action of this janitor.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_action_name__fill_strings,
   },
   0, get_action_name_setup, get_action_name_exec, 
 };
+static void
+__enode_get_action_blurb__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve the help string of an action of this janitor.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_action_blurb = {
   { (BseExportNode*) &__enode_get_action_name, BSE_EXPORT_NODE_PROC,
     "BseJanitor+get-action-blurb", 
     NULL,
     "/Methods/BseJanitor/General/Get Action Blurb",
-    "Retrieve the help string of an action of this janitor.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_action_blurb__fill_strings,
   },
   0, get_action_blurb_setup, get_action_blurb_exec, 
 };
+static void
+__enode_kill__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Kill a currently running janitor.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_kill = {
   { (BseExportNode*) &__enode_get_action_blurb, BSE_EXPORT_NODE_PROC,
     "BseJanitor+kill", 
     NULL,
     "/Methods/BseJanitor/General/Kill",
-    "Kill a currently running janitor.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_kill__fill_strings,
   },
   0, kill_setup, kill_exec, 
 };
+static void
+__enode_add_action__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Install a user action for script control.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_add_action = {
   { (BseExportNode*) &__enode_kill, BSE_EXPORT_NODE_PROC,
     "BseJanitor+add-action", 
     NULL,
     "/Methods/BseJanitor/General/Add Action",
-    "Install a user action for script control.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_add_action__fill_strings,
   },
   0, add_action_setup, add_action_exec, 
 };
+static void
+__enode_remove_action__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Remove a previously installed user action from script control.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_remove_action = {
   { (BseExportNode*) &__enode_add_action, BSE_EXPORT_NODE_PROC,
     "BseJanitor+remove-action", 
     NULL,
     "/Methods/BseJanitor/General/Remove Action",
-    "Remove a previously installed user action from script control.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_remove_action__fill_strings,
   },
   0, remove_action_setup, remove_action_exec, 
 };
+static void
+__enode_bse_janitor_get_specific__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve the connection dependant janitor object.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_bse_janitor_get_specific = {
   { (BseExportNode*) &__enode_remove_action, BSE_EXPORT_NODE_PROC,
     "bse-janitor-get-specific", 
     NULL,
     "/Proc/Janitor/Get Specific",
-    "Retrieve the connection dependant janitor object.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_bse_janitor_get_specific__fill_strings,
   },
   0, bse_janitor_get_specific_setup, bse_janitor_get_specific_exec, 
 };
+static void
+__enode_set_msg__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Alter script status message.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_set_msg = {
   { (BseExportNode*) &__enode_bse_janitor_get_specific, BSE_EXPORT_NODE_PROC,
     "BseJanitor+set-msg", 
     NULL,
     "/Methods/BseJanitor/General/Set Msg",
-    "Alter script status message.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_set_msg__fill_strings,
   },
   0, set_msg_setup, set_msg_exec, 
 };
+static void
+__enode_progress__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Indicate progress (from 0.0 to 1.0 or -1 for unknown amount).";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_progress = {
   { (BseExportNode*) &__enode_set_msg, BSE_EXPORT_NODE_PROC,
     "BseJanitor+progress", 
     NULL,
     "/Methods/BseJanitor/General/Progress",
-    "Indicate progress (from 0.0 to 1.0 or -1 for unknown amount).",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_progress__fill_strings,
   },
   0, progress_setup, progress_exec, 
 };
+static void
+__enode_exit__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Exit a script, leaving a message.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_exit = {
   { (BseExportNode*) &__enode_progress, BSE_EXPORT_NODE_PROC,
     "BseJanitor+exit", 
     NULL,
     "/Methods/BseJanitor/General/Exit",
-    "Exit a script, leaving a message.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_exit__fill_strings,
   },
   0, exit_setup, exit_exec, 
 };
@@ -743,6 +827,6 @@ BseExportNode* bse__builtin_init_bsejanitor_gen_proc_c (void)
   return (BseExportNode*) &__enode_exit;
 }
 
-/**
- ** Generated data ends here
- **/
+/*
+ * Generated data ends here
+ */
