@@ -18,13 +18,16 @@
 #include "bsemain.h"
 #include "topconfig.h"
 #include "bseserver.h"
-#include "bsessequencer.h"
+#include "bsesequencer.h"
 #include "bsejanitor.h"
 #include "bseplugin.h"
 #include "bsecategories.h"
 #include "bsemidireceiver.h"
+#include "bsemathsignal.h"
 #include "gslcommon.h"
-#include "gslengine.h"
+#include "bsepcmdevice.h"
+#include "bsemididevice.h"
+#include "bseengine.h"
 #include <string.h>
 #include <stdlib.h>
 #include <sys/time.h>
@@ -36,7 +39,8 @@
 static void	bse_main_loop		(gpointer	data);
 static void	bse_async_parse_args	(gint	       *argc_p,
 					 gchar	     ***argv_p,
-					 SfiRec        *config);
+                                         BseMainArgs   *margs,
+                                         SfiRec        *config);
 
 
 /* --- variables --- */
@@ -53,6 +57,10 @@ gboolean	         bse_main_debug_extensions = FALSE;
 SfiThread               *bse_main_thread = NULL;
 static volatile gboolean bse_initialization_stage = 0;
 static gboolean          textdomain_setup = FALSE;
+static BseMainArgs       default_main_args = {
+  .path_binaries = BSE_PATH_BINARIES,
+};
+BseMainArgs             *bse_main_args = &default_main_args;
 
 
 /* --- functions --- */
@@ -80,18 +88,22 @@ bse_init_async (gint    *argc,
   bind_textdomain_codeset (BSE_GETTEXT_DOMAIN, "UTF-8");
   textdomain_setup = TRUE;
 
-  /* this function is running in the user thread and needs to start the main BSE thread */
+  /* this function is running in the user program and needs to start the main BSE thread */
   
   /* initialize submodules */
   sfi_init ();
   /* paranoid assertions */
   g_assert (G_BYTE_ORDER == G_LITTLE_ENDIAN || G_BYTE_ORDER == G_BIG_ENDIAN);
-  /* early argument handling */
+
+  /* handle argument early*/
+  SfiRec *unref_me = NULL;
+  if (!config)
+    config = unref_me = sfi_rec_new();
   if (argc && argv)
     {
       if (*argc && !g_get_prgname ())
 	g_set_prgname (**argv);
-      bse_async_parse_args (argc, argv, config);
+      bse_async_parse_args (argc, argv, bse_main_args, config);
     }
   
   /* start main BSE thread */
@@ -102,6 +114,9 @@ bse_init_async (gint    *argc,
   /* wait for initialization completion of the core thread */
   while (bse_initialization_stage < 2)
     sfi_thread_sleep (-1);
+  /* cleanup */
+  if (unref_me)
+    sfi_rec_unref (unref_me);
 }
 
 gchar*
@@ -189,6 +204,7 @@ bse_init_core (void)
   bse_main_context = g_main_context_new ();
   sfi_thread_set_wakeup ((SfiThreadWakeup) g_main_context_wakeup,
 			 bse_main_context, NULL);
+  sfi_log_set_thread_handler (bse_log_handler);
 
   /* initialize random numbers */
   {
@@ -199,6 +215,7 @@ bse_init_core (void)
   
   /* initialize basic components */
   bse_globals_init ();
+  _bse_init_signal();
   _bse_init_categories ();
   bse_type_init ();
   bse_cxx_init ();
@@ -233,6 +250,29 @@ bse_init_core (void)
 
   /* make sure the server is alive */
   bse_server_get ();
+
+  /* load drivers early */
+  if (bse_main_args->load_drivers_early)
+    {
+      SfiRing *ring = bse_plugin_path_list_files (TRUE, FALSE);
+      while (ring)
+        {
+          gchar *name = sfi_ring_pop_head (&ring);
+          const char *error = bse_plugin_check_load (name);
+          if (error)
+            sfi_diag ("while loading \"%s\": %s", name, error);
+          g_free (name);
+        }
+    }
+
+  /* dump device list */
+  if (bse_main_args->dump_driver_list)
+    {
+      g_printerr (_("\nAvailable PCM drivers:\n"));
+      bse_device_dump_list (BSE_TYPE_PCM_DEVICE, "  ", TRUE, NULL, NULL);
+      g_printerr (_("\nAvailable MIDI drivers:\n"));
+      bse_device_dump_list (BSE_TYPE_MIDI_DEVICE, "  ", TRUE, NULL, NULL);
+    }
 }
 
 void
@@ -262,7 +302,7 @@ bse_init_intern (gint    *argc,
     {
       if (*argc && !g_get_prgname ())
 	g_set_prgname (**argv);
-      bse_async_parse_args (argc, argv, config);
+      bse_async_parse_args (argc, argv, bse_main_args, config);
     }
   
   bse_init_core ();
@@ -270,7 +310,7 @@ bse_init_intern (gint    *argc,
   /* initialize core plugins */
   if (sfi_rec_get_bool (config, "load-core-plugins"))
     {
-      SfiRing *ring = bse_plugin_path_list_files ();
+      SfiRing *ring = bse_plugin_path_list_files (!bse_main_args->load_drivers_early, TRUE);
       while (ring)
         {
           gchar *name = sfi_ring_pop_head (&ring);
@@ -293,13 +333,13 @@ bse_main_loop (gpointer data)
 
   bse_init_core ();
 
+  /* start other threads */
+  bse_sequencer_init_thread ();
+
   /* notify client about completion */
   bse_initialization_stage++;   /* =2 */
   sfi_thread_wakeup (client);
 
-  /* start other threads */
-  bse_ssequencer_init_thread ();
-
   /* and away into the main loop */
   do
     {
@@ -318,10 +358,56 @@ bse_main_getpid (void)
     return 0;
 }
 
+static gboolean
+core_thread_send_message (gpointer data)
+{
+  BseUserMsg *umsg = data;
+  bse_server_user_message (bse_server_get(),
+                           umsg->log_domain,
+                           umsg->msg_type,
+                           umsg->config_blurb,
+                           umsg->message,
+                           umsg->pid,
+                           umsg->process);
+  bse_user_msg_free (umsg);
+  return FALSE;
+}
+
+/**
+ * bse_log_handler
+ * BSE log handler, suitable for sfi_log_set_thread_handler().
+ * This function is MT-safe and may be called from any thread.
+ */
+void
+bse_log_handler (SfiLogMessage *message)
+{
+  if (!message->message)
+    return;
+  BseUserMsg *umsg = bse_user_msg_new();
+  g_free (umsg->log_domain);
+  umsg->log_domain = g_strdup (message->log_domain);
+  switch (message->level)
+    {
+    case SFI_LOG_ERROR:   umsg->msg_type = BSE_USER_MSG_ERROR;   break;
+    case SFI_LOG_WARNING: umsg->msg_type = BSE_USER_MSG_WARNING; break;
+    case SFI_LOG_INFO:    umsg->msg_type = BSE_USER_MSG_INFO;    break;
+    default:              umsg->msg_type = BSE_USER_MSG_MISC;    break;
+    }
+  g_free (umsg->config_blurb);
+  umsg->config_blurb = g_strdup (message->config_blurb);
+  g_free (umsg->message);
+  umsg->message = g_strdup (message->message);
+  umsg->pid = sfi_thread_get_pid (NULL);
+  g_free (umsg->process);
+  umsg->process = g_strdup (sfi_thread_get_name (NULL));
+  bse_idle_now (core_thread_send_message, umsg);
+}
+
 static void
-bse_async_parse_args (gint    *argc_p,
-		      gchar ***argv_p,
-		      SfiRec  *config)
+bse_async_parse_args (gint        *argc_p,
+		      gchar     ***argv_p,
+                      BseMainArgs *margs,
+		      SfiRec      *config)
 {
   guint argc = *argc_p;
   gchar **argv = *argv_p;
@@ -343,9 +429,7 @@ bse_async_parse_args (gint    *argc_p,
     {
       if (strcmp (argv[i], "--g-fatal-warnings") == 0)
 	{
-	  GLogLevelFlags fatal_mask;
-	  
-	  fatal_mask = g_log_set_always_fatal (G_LOG_FATAL_MASK);
+	  GLogLevelFlags fatal_mask = g_log_set_always_fatal (G_LOG_FATAL_MASK);
 	  fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
 	  g_log_set_always_fatal (fatal_mask);
 	  
@@ -355,7 +439,6 @@ bse_async_parse_args (gint    *argc_p,
 	       strncmp ("--bse-debug=", argv[i], 12) == 0)
 	{
 	  gchar *equal = argv[i] + 11;
-	  
 	  if (*equal == '=')
             sfi_debug_allow (equal + 1);
 	  else if (i + 1 < argc)
@@ -366,10 +449,9 @@ bse_async_parse_args (gint    *argc_p,
 	  argv[i] = NULL;
 	}
       else if (strcmp ("--bse-no-debug", argv[i]) == 0 ||
-	       strncmp ("--bse-no-debug=", argv[i], 12) == 0)
+	       strncmp ("--bse-no-debug=", argv[i], 15) == 0)
 	{
-	  gchar *equal = argv[i] + 11;
-	  
+	  gchar *equal = argv[i] + 14;
 	  if (*equal == '=')
             sfi_debug_deny (equal + 1);
 	  else if (i + 1 < argc)
@@ -379,6 +461,89 @@ bse_async_parse_args (gint    *argc_p,
 	    }
 	  argv[i] = NULL;
 	}
+      else if (strcmp ("--bse-latency", argv[i]) == 0 ||
+	       strncmp ("--bse-latency=", argv[i], 14) == 0)
+	{
+	  gchar *equal = argv[i] + 13;
+	  if (*equal == '=')
+            margs->latency = g_ascii_strtoull (equal + 1, NULL, 10);
+	  else if (i + 1 < argc)
+	    {
+	      argv[i++] = NULL;
+              margs->latency = g_ascii_strtoull (argv[i], NULL, 10);
+	    }
+	  argv[i] = NULL;
+	}
+      else if (strcmp ("--bse-mixing-freq", argv[i]) == 0 ||
+	       strncmp ("--bse-mixing-freq=", argv[i], 18) == 0)
+	{
+	  gchar *equal = argv[i] + 17;
+	  if (*equal == '=')
+            margs->mixing_freq = g_ascii_strtoull (equal + 1, NULL, 10);
+	  else if (i + 1 < argc)
+	    {
+	      argv[i++] = NULL;
+              margs->mixing_freq = g_ascii_strtoull (argv[i], NULL, 10);
+	    }
+	  argv[i] = NULL;
+	}
+      else if (strcmp ("--bse-control-freq", argv[i]) == 0 ||
+	       strncmp ("--bse-control-freq=", argv[i], 19) == 0)
+	{
+	  gchar *equal = argv[i] + 18;
+	  if (*equal == '=')
+            margs->control_freq = g_ascii_strtoull (equal + 1, NULL, 10);
+	  else if (i + 1 < argc)
+	    {
+	      argv[i++] = NULL;
+              margs->control_freq = g_ascii_strtoull (argv[i], NULL, 10);
+	    }
+	  argv[i] = NULL;
+	}
+      else if (strcmp ("--bse-driver-list", argv[i]) == 0)
+	{
+          margs->load_drivers_early = TRUE;
+          margs->dump_driver_list = TRUE;
+	  argv[i] = NULL;
+	}
+      else if (strcmp ("--bse-pcm-driver", argv[i]) == 0)
+	{
+          if (i + 1 < argc)
+	    {
+	      argv[i++] = NULL;
+              margs->pcm_drivers = sfi_ring_append (margs->pcm_drivers, argv[i]);
+	    }
+	  argv[i] = NULL;
+	}
+      else if (strcmp ("--bse-midi-driver", argv[i]) == 0)
+	{
+          if (i + 1 < argc)
+	    {
+	      argv[i++] = NULL;
+              margs->midi_drivers = sfi_ring_append (margs->midi_drivers, argv[i]);
+	    }
+	  argv[i] = NULL;
+	}
+      else if (strcmp ("--bse-override-plugin-globs", argv[i]) == 0 && i + 1 < argc)
+	{
+          argv[i++] = NULL;
+          margs->override_plugin_globs = argv[i];
+	  argv[i] = NULL;
+	}
+      else if (strcmp ("--bse-override-script-path", argv[i]) == 0 && i + 1 < argc)
+	{
+          argv[i++] = NULL;
+          margs->override_script_path = argv[i];
+	  argv[i] = NULL;
+	}
+#if 0
+      else if (strcmp ("--bse-override-binaries-path", argv[i]) == 0 && i + 1 < argc)
+	{
+          argv[i++] = NULL;
+          margs->path_binaries = argv[i];
+	  argv[i] = NULL;
+	}
+#endif
     }
 
   e = 1;
@@ -391,6 +556,6 @@ bse_async_parse_args (gint    *argc_p,
       }
   *argc_p = e;
 
-  if (config && sfi_rec_get_bool (config, "debug-extensions"))
+  if (sfi_rec_get_bool (config, "debug-extensions"))
     bse_main_debug_extensions = TRUE;
 }
