@@ -42,6 +42,8 @@ void		bse_init_intern		(gint		*argc,
 /* BSE thread pid (or 0) */
 guint           bse_main_getpid         (void);
 
+/* MT-safe log handler */
+void            bse_log_handler         (SfiLogMessage  *message);
 
 /* --- global macros --- */
 #define	BSE_THREADS_ENTER()			// bse_main_global_lock ()
@@ -50,9 +52,24 @@ guint           bse_main_getpid         (void);
 #define	BSE_SEQUENCER_UNLOCK()			SFI_SYNC_UNLOCK (&bse_main_sequencer_mutex)
 #define	BSE_DBG_EXT     			(bse_main_debug_extensions != FALSE)
 
+/* --- argc/argv overide settings --- */
+typedef struct {
+  SfiInt       latency;
+  SfiInt       mixing_freq;
+  SfiInt       control_freq;
+  SfiRing     *pcm_drivers;
+  SfiRing     *midi_drivers;
+  gboolean     load_drivers_early;
+  gboolean     dump_driver_list;
+  const gchar *override_plugin_globs;
+  const gchar *override_script_path;
+  const gchar *path_binaries;
+} BseMainArgs;
+
 
 /* --- internal --- */
 void    _bse_init_c_wrappers    ();
+extern BseMainArgs  *bse_main_args;
 extern GMainContext *bse_main_context;
 extern SfiMutex	     bse_main_sequencer_mutex;
 extern gboolean      bse_main_debug_extensions;
