@@ -28,8 +28,7 @@
 /* --- macros --- */
 #define parse_or_return         bse_storage_scanner_parse_or_return
 #define peek_or_return          bse_storage_scanner_peek_or_return
-#define EXEC_DEBUG              sfi_debug_keyfunc ("proc-exec")
-#define HACK_DEBUG /* very slow and leaks memory */ while (0) g_message
+#define HACK_DEBUG /* very slow and leaks memory */ while (0) g_printerr
 
 
 /* --- prototypes --- */
@@ -239,11 +238,11 @@ signal_exec_status (BseErrorType       error,
 }
 
 static BseErrorType
-call_proc (BseProcedureClass  *proc,
-           GValue             *ivalues,
-           GValue             *ovalues,
-           BseProcedureMarshal marshal,
-           gpointer            marshal_data)
+bse_procedure_call (BseProcedureClass  *proc,
+                    GValue             *ivalues,
+                    GValue             *ovalues,
+                    BseProcedureMarshal marshal,
+                    gpointer            marshal_data)
 {
   guint i, bail_out = FALSE;
   BseErrorType error;
@@ -265,13 +264,13 @@ call_proc (BseProcedureClass  *proc,
     error = BSE_ERROR_PROC_PARAM_INVAL;
   else
     {
-      if (sfi_debug_test_key ("procs"))
+      if (sfi_debug_check ("procs"))
         {
           if (proc->n_in_pspecs && G_TYPE_IS_OBJECT (G_PARAM_SPEC_VALUE_TYPE (proc->in_pspecs[0])))
-            EXEC_DEBUG ("executing procedure \"%s\" on object %s",
-                   BSE_PROCEDURE_NAME (proc), bse_object_debug_name (g_value_get_object (ivalues + 0)));
+            sfi_debug ("procs", "executing procedure \"%s\" on object %s",
+                       BSE_PROCEDURE_NAME (proc), bse_object_debug_name (g_value_get_object (ivalues + 0)));
           else
-            EXEC_DEBUG ("executing procedure \"%s\"", BSE_PROCEDURE_NAME (proc));
+            sfi_debug ("procs", "executing procedure \"%s\"", BSE_PROCEDURE_NAME (proc));
         }
       if (marshal)
         error = marshal (marshal_data, proc, ivalues, ovalues);
@@ -333,7 +332,7 @@ bse_procedure_marshal (GType               proc_type,
   if (bail_out)
     error = BSE_ERROR_PROC_PARAM_INVAL;
   else
-    error = call_proc (proc, tmp_ivalues, tmp_ovalues, marshal, marshal_data);
+    error = bse_procedure_call (proc, tmp_ivalues, tmp_ovalues, marshal, marshal_data);
   signal_exec_status (error, proc, tmp_ovalues);
   
   for (i = 0; i < proc->n_in_pspecs; i++)
@@ -372,7 +371,7 @@ bse_procedure_call_collect (BseProcedureClass  *proc,
   HACK_DEBUG ("call %s: ", BSE_PROCEDURE_NAME (proc));
 
   /* collect first arg */
-  if (first_value && first_value != ivalues) /* may skip this since call_proc() does extra validation */
+  if (first_value && first_value != ivalues) /* may skip this since bse_procedure_call() does extra validation */
     {
       if (proc->n_in_pspecs < 1)
         g_warning ("%s: input arg supplied for procedure taking `void'",
@@ -431,7 +430,7 @@ bse_procedure_call_collect (BseProcedureClass  *proc,
       if (bail_out)
         error = BSE_ERROR_PROC_PARAM_INVAL;
       else
-        error = call_proc (proc, ivalues, ovalues, marshal, marshal_data);
+        error = bse_procedure_call (proc, ivalues, ovalues, marshal, marshal_data);
       HACK_DEBUG ("  call result: %s", bse_error_blurb (error));
       signal_exec_status (error, proc, ovalues);
 
