@@ -1,7 +1,7 @@
 
-/**
- ** Generated data (by mkcproc.pl)
- **/
+/*
+ * Generated data (by mkcproc.pl)
+ */
 #line 1 "bseprocedure.proc"
 
 #include <bse/bseplugin.h>
@@ -11,7 +11,7 @@
 #line 22 "bseprocedure.proc"
 #include <bse/gslcommon.h>      
 #line 23 "bseprocedure.proc"
-#include <bse/gslmath.h>        
+#include <bse/bsemath.h>        
 #line 24 "bseprocedure.proc"
 #include <string.h>     
 #line 25 "bseprocedure.proc"
@@ -334,7 +334,7 @@ bse_string_extract_number_exec (BseProcedureClass *proc,
             {
 #line 257 "bseprocedure.proc"
               if (*ep == 'm')       
-                number = gsl_temp_freq (gsl_get_config ()->kammer_freq,
+                number = bse_temp_freq (gsl_get_config ()->kammer_freq,
                                         number - gsl_get_config ()->midi_kammer_note);
 #line 260 "bseprocedure.proc"
               else 
@@ -474,6 +474,6 @@ BseExportNode* bse__builtin_init_bseprocedure_gen_proc_c (void)
   return (BseExportNode*) &__enode_bse_string_extract_number;
 }
 
-/**
- ** Generated data ends here
- **/
+/*
+ * Generated data ends here
+ */
