@@ -21,15 +21,16 @@
 #include "bsestorage.h"
 #include "bsesong.h"
 #include "bsesnet.h"
+#include "bsecsynth.h"
 #include "bsewaverepo.h"
-#include "bsessequencer.h"
+#include "bsesequencer.h"
 #include "bseserver.h"
 #include "bseundostack.h"
 #include "bsemain.h"
 #include "bsestandardsynths.h"
 #include "bsemidireceiver.h"
 #include "gslcommon.h"
-#include "gslengine.h"
+#include "bseengine.h"
 #include <string.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -91,8 +92,6 @@ static BseUndoStack* bse_project_get_undo       (BseItem                *item);
 /* --- variables --- */
 static GTypeClass *parent_class = NULL;
 static guint       signal_state_changed = 0;
-static GSList     *plist_auto_stop_SL = NULL;
-static guint       handler_id_auto_stop_check_SL = 0;
 static GQuark      quark_storage_trap = 0;
 
 
@@ -275,10 +274,6 @@ bse_project_finalize (GObject *object)
 {
   BseProject *self = BSE_PROJECT (object);
   
-  BSE_SEQUENCER_LOCK ();
-  plist_auto_stop_SL = g_slist_remove (plist_auto_stop_SL, self);
-  BSE_SEQUENCER_UNLOCK ();
-
   bse_midi_receiver_unref (self->midi_receiver);
   self->midi_receiver = NULL;
 
@@ -296,6 +291,18 @@ bse_project_get_undo (BseItem *item)
   return self->in_undo ? self->redo_stack : self->undo_stack;
 }
 
+void
+bse_project_clear_undo (BseProject *self)
+{
+  g_return_if_fail (BSE_IS_PROJECT (self));
+  if (!self->in_undo && !self->in_redo)
+    {
+      bse_undo_stack_clear (self->undo_stack);
+      bse_undo_stack_clear (self->redo_stack);
+      g_object_notify (self, "dirty");
+    }
+}
+
 static void
 bse_project_add_item (BseContainer *container,
 		      BseItem      *item)
@@ -522,6 +529,15 @@ bse_project_restore (BseProject *self,
 
   bse_storage_resolve_item_links (storage);
 
+  GSList *slist = self->supers;
+  while (slist)
+    {
+      BseSuper *super = slist->data;
+      slist = slist->next;
+      BseSuperClass *super_class = BSE_SUPER_GET_CLASS (super);
+      super_class->compat_finish (super, storage->major_version, storage->minor_version, storage->micro_version);
+    }
+
   g_object_unref (self);
 
   return (scanner->parse_errors >= scanner->max_parse_errors ?
@@ -640,6 +656,18 @@ bse_project_create_intern_synth (BseProject  *self,
   return synth;
 }
 
+BseCSynth*
+bse_project_create_intern_csynth (BseProject *self,
+                                  const char *base_name)
+{
+  BseCSynth *csynth = bse_container_new_child_bname (BSE_CONTAINER (self),
+                                                     BSE_TYPE_CSYNTH,
+                                                     base_name,
+                                                     NULL);
+  bse_item_set_internal (BSE_ITEM (csynth), TRUE);
+  return csynth;
+}
+
 static void
 bse_project_prepare (BseSource *source)
 {
@@ -682,7 +710,7 @@ bse_project_state_changed (BseProject     *self,
       SfiTime stamp = gsl_tick_stamp ();
       SfiTime delay_usecs = 0;
       if (self->deactivate_min_tick > stamp)
-	delay_usecs = (self->deactivate_min_tick - stamp) * 1000000 / gsl_engine_sample_freq ();
+	delay_usecs = (self->deactivate_min_tick - stamp) * 1000000 / bse_engine_sample_freq ();
       self->deactivate_timer = bse_idle_timed (self->deactivate_usecs + delay_usecs, auto_deactivate, self);
     }
   g_signal_emit (self, signal_state_changed, 0, state);
@@ -706,7 +734,7 @@ BseErrorType
 bse_project_activate (BseProject *self)
 {
   BseErrorType error;
-  GslTrans *trans;
+  BseTrans *trans;
   GSList *slist;
 
   g_return_val_if_fail (BSE_IS_PROJECT (self), BSE_ERROR_INTERNAL);
@@ -723,7 +751,7 @@ bse_project_activate (BseProject *self)
   bse_source_prepare (BSE_SOURCE (self));
   self->deactivate_min_tick = 0;
   
-  trans = gsl_trans_open ();
+  trans = bse_trans_open ();
   for (slist = self->supers; slist; slist = slist->next)
     {
       BseSuper *super = BSE_SUPER (slist->data);
@@ -739,7 +767,7 @@ bse_project_activate (BseProject *self)
       else
 	super->context_handle = ~0;
     }
-  gsl_trans_commit (trans);
+  bse_trans_commit (trans);
   bse_project_state_changed (self, BSE_PROJECT_ACTIVE);
   return BSE_ERROR_NONE;
 }
@@ -747,8 +775,7 @@ bse_project_activate (BseProject *self)
 void
 bse_project_start_playback (BseProject *self)
 {
-  SfiRing *seq_list = NULL;
-  GslTrans *trans;
+  BseTrans *trans;
   GSList *slist;
   guint seen_synth = 0;
 
@@ -758,12 +785,12 @@ bse_project_start_playback (BseProject *self)
     return;
   g_return_if_fail (BSE_SOURCE_PREPARED (self) == TRUE);
 
-  trans = gsl_trans_open ();
+  SfiRing *songs = NULL;
+  trans = bse_trans_open ();
   for (slist = self->supers; slist; slist = slist->next)
     {
       BseSuper *super = BSE_SUPER (slist->data);
-      if ((BSE_SUPER_NEEDS_CONTEXT (super) ||
-	   BSE_SUPER_NEEDS_SEQUENCER_CONTEXT (super)) &&
+      if (BSE_SUPER_NEEDS_CONTEXT (super) &&
 	  super->context_handle == ~0)
 	{
           BseMidiContext mcontext = { 0, 0, 0 };
@@ -772,21 +799,29 @@ bse_project_start_playback (BseProject *self)
           mcontext.midi_channel = 1; /* midi channel default */
           super->context_handle = bse_snet_create_context (snet, mcontext, trans);
 	  bse_source_connect_context (BSE_SOURCE (snet), super->context_handle, trans);
-	  seen_synth++;
 	}
-      if (BSE_SUPER_NEEDS_SEQUENCER (super))
-	seq_list = sfi_ring_append (seq_list, super);
+      if (BSE_SUPER_NEEDS_CONTEXT (super))
+        seen_synth++;
+      if (BSE_IS_SONG (super))
+	songs = sfi_ring_append (songs, super);
     }
-  bse_ssequencer_start_supers (seq_list, trans);
-  if (seen_synth || seq_list)
+  /* enfore MasterThread roundtrip */
+  bse_trans_add (trans, bse_job_nop());
+  bse_trans_commit (trans);
+  /* first, enforce integrated (and possibly scheduled) modules; */
+  bse_engine_wait_on_trans();
+  /* update state */
+  if (seen_synth || songs)
     bse_project_state_changed (self, BSE_PROJECT_PLAYING);
+  /* then, start the sequencer */
+  while (songs)
+    bse_sequencer_start_song (sfi_ring_pop_head (&songs), 0);
 }
 
 void
 bse_project_stop_playback (BseProject *self)
 {
-  SfiRing *seq_jobs = NULL;
-  GslTrans *trans;
+  BseTrans *trans;
   GSList *slist;
 
   g_return_if_fail (BSE_IS_PROJECT (self));
@@ -795,12 +830,12 @@ bse_project_stop_playback (BseProject *self)
     return;
   g_return_if_fail (BSE_SOURCE_PREPARED (self) == TRUE);
 
-  trans = gsl_trans_open ();
+  trans = bse_trans_open ();
   for (slist = self->supers; slist; slist = slist->next)
     {
       BseSuper *super = BSE_SUPER (slist->data);
-
-      seq_jobs = sfi_ring_prepend (seq_jobs, bse_ssequencer_job_stop_super (super));
+      if (BSE_IS_SONG (super))
+        bse_sequencer_remove_song (BSE_SONG (super));
       if (super->context_handle != ~0 && !BSE_SUPER_NEEDS_CONTEXT (super))
 	{
 	  BseSource *source = BSE_SOURCE (super);
@@ -808,11 +843,12 @@ bse_project_stop_playback (BseProject *self)
 	  super->context_handle = ~0;
 	}
     }
-  if (seq_jobs)
-    bse_ssequencer_handle_jobs (seq_jobs);
-  gsl_trans_commit (trans);
+  /* enfore MasterThread roundtrip */
+  bse_trans_add (trans, bse_job_nop());
+  bse_trans_commit (trans);
   /* wait until after all modules have actually been dismissed */
-  gsl_engine_wait_on_trans ();
+  bse_engine_wait_on_trans ();
+  /* update state */
   bse_project_state_changed (self, BSE_PROJECT_ACTIVE);
 }
 
@@ -829,9 +865,7 @@ bse_project_check_auto_stop (BseProject *self)
 	  BseSuper *super = BSE_SUPER (slist->data);
 	  if (super->context_handle != ~0)
 	    {
-	      if (BSE_SUPER_NEEDS_SEQUENCER_CONTEXT (super) ||
-		  !BSE_IS_SONG (super) ||
-		  !BSE_SONG (super)->song_done_SL)
+	      if (!BSE_IS_SONG (super) || !BSE_SONG (super)->sequencer_done_SL)
 		return;
 	    }
 	}
@@ -839,35 +873,10 @@ bse_project_check_auto_stop (BseProject *self)
     }
 }
 
-static gboolean
-auto_stop_check_handler (gpointer data)
-{
-  BSE_SEQUENCER_LOCK ();
-  while (plist_auto_stop_SL)
-    {
-      BseProject *self = g_slist_pop_head (&plist_auto_stop_SL);
-      BSE_SEQUENCER_UNLOCK ();
-      bse_project_check_auto_stop (self);
-      BSE_SEQUENCER_LOCK ();
-    }
-  handler_id_auto_stop_check_SL = 0;
-  BSE_SEQUENCER_UNLOCK ();
-  return FALSE;
-}
-
-void
-bse_project_queue_auto_stop_SL (BseProject *self)
-{
-  if (!g_slist_find (plist_auto_stop_SL, self))
-    plist_auto_stop_SL = g_slist_prepend (plist_auto_stop_SL, self);
-  if (!handler_id_auto_stop_check_SL)
-    handler_id_auto_stop_check_SL = bse_idle_now (auto_stop_check_handler, NULL);
-}
-
 void
 bse_project_deactivate (BseProject *self)
 {
-  GslTrans *trans;
+  BseTrans *trans;
   GSList *slist;
 
   g_return_if_fail (BSE_IS_PROJECT (self));
@@ -878,7 +887,7 @@ bse_project_deactivate (BseProject *self)
 
   bse_project_stop_playback (self);
 
-  trans = gsl_trans_open ();
+  trans = bse_trans_open ();
   for (slist = self->supers; slist; slist = slist->next)
     {
       BseSuper *super = BSE_SUPER (slist->data);
@@ -889,9 +898,9 @@ bse_project_deactivate (BseProject *self)
 	  super->context_handle = ~0;
 	}
     }
-  gsl_trans_commit (trans);
+  bse_trans_commit (trans);
   /* wait until after all modules have actually been dismissed */
-  gsl_engine_wait_on_trans ();
+  bse_engine_wait_on_trans ();
   bse_source_reset (BSE_SOURCE (self));
   bse_project_state_changed (self, BSE_PROJECT_INACTIVE);
 
