@@ -17,7 +17,7 @@
  */
 #include "bseserver.h"
 #include "bseproject.h"
-#include "gslengine.h"
+#include "bseengine.h"
 #include "gslcommon.h"
 #include "bseglue.h"
 #include "bsegconfig.h"
@@ -33,7 +33,7 @@
 #include <string.h>
 
 
-/* --- PCM GslModule implementations ---*/
+/* --- PCM BseModule implementations ---*/
 #include "bsepcmmodule.c"
 
 
@@ -140,8 +140,9 @@ bse_server_class_init (BseServerClass *class)
 			      bse_gconfig_pspec ());	/* "bse-preferences" */
   bse_object_class_add_param (object_class, "PCM Recording",
 			      PROP_WAVE_FILE,
-			      sfi_pspec_string ("wave_file", "WAVE File", NULL,
-						NULL, SFI_PARAM_GUI));
+			      sfi_pspec_string ("wave_file", _("WAVE File"),
+                                                _("Name of the WAVE file used for recording BSE sound output"),
+						NULL, SFI_PARAM_GUI ":filename"));
   
   signal_registration = bse_object_class_add_signal (object_class, "registration",
 						     G_TYPE_NONE, 3,
@@ -149,9 +150,7 @@ bse_server_class_init (BseServerClass *class)
 						     G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
 						     G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
   signal_user_message = bse_object_class_add_signal (object_class, "user-message",
-						     G_TYPE_NONE, 2,
-						     BSE_TYPE_USER_MSG_TYPE,
-						     G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
+						     G_TYPE_NONE, 1, BSE_TYPE_USER_MSG | G_SIGNAL_TYPE_STATIC_SCOPE);
   signal_script_start = bse_object_class_add_signal (object_class, "script-start",
 						     G_TYPE_NONE, 1,
 						     BSE_TYPE_JANITOR);
@@ -223,6 +222,9 @@ bse_server_init (BseServer *self)
       close (fd);
     }
   g_free (file_name);
+
+  /* integrate argv overides */
+  bse_gconfig_merge_args (bse_main_args);
 }
 
 static void
@@ -427,86 +429,85 @@ bse_server_find_project (BseServer   *server,
   return NULL;
 }
 
-static BseErrorType
-bse_server_open_pcm_device (BseServer *server)
+void
+bse_server_require_pcm_input (BseServer *server)
 {
-  GType *children, choice = 0;
-  guint n, i, rating = 0;
-  BseErrorType error;
-  
-  g_return_val_if_fail (server->pcm_device == NULL, BSE_ERROR_INTERNAL);
-  
-  /* pick PCM driver implementation and try to open device */
-  children = g_type_children (BSE_TYPE_PCM_DEVICE, &n);
-  for (i = 0; i < n; i++)
+  if (server->pcm_device && !server->pcm_input_checked)
     {
-      BsePcmDeviceClass *class = g_type_class_ref (children[i]);
-      if (class->driver_rating > rating)
-	{
-	  rating = class->driver_rating;
-	  choice = children[i];
-	}
-      g_type_class_unref (class);
+      server->pcm_input_checked = TRUE;
+      if (!BSE_DEVICE_READABLE (server->pcm_device))
+        sfi_warning_msg (_("Show messages about audio recording problems"),
+                         _("An audio input module is in use, but the audio device "
+                           "has not been opened in recording mode. "
+                           "A quiet audio signal is used as input instead of a "
+                           "recorded signal, so playback operation may produce "
+                           "results not actually intended (such as reducing the "
+                           "output signal to pure silence)."));
     }
-  g_free (children);
-  
-  if (!choice)
-    return BSE_ERROR_DEVICE_NOT_AVAILABLE;
-  
-  server->pcm_device = g_object_new (choice, NULL);
-  bse_pcm_device_request (server->pcm_device, 2, bse_pcm_freq_mode_from_freq (BSE_GCONFIG (synth_mixing_freq)));
-  error = bse_pcm_device_open (server->pcm_device);
-  if (error)
-    {
-      g_object_unref (server->pcm_device);
-      server->pcm_device = NULL;
-    }
-  return error;
+}
+
+typedef struct {
+  guint      n_channels;
+  guint      mix_freq;
+  guint      latency;
+  guint      block_size;
+} PcmRequest;
+
+static void
+pcm_request_callback (BseDevice *device,
+                      gpointer   data)
+{
+  PcmRequest *pr = data;
+  bse_pcm_device_request (BSE_PCM_DEVICE (device), pr->n_channels, pr->mix_freq, pr->latency, pr->block_size);
 }
 
 static BseErrorType
-bse_server_open_midi_device (BseServer *server)
+server_open_pcm_device (BseServer *server,
+                        guint      mix_freq,
+                        guint      latency,
+                        guint      block_size)
 {
-  GType *children, choice = 0;
-  guint n, i, rating = 0;
-  BseErrorType error;
+  g_return_val_if_fail (server->pcm_device == NULL, BSE_ERROR_INTERNAL);
   
+  BseErrorType error = 0;
+  PcmRequest pr;
+  pr.n_channels = 2;
+  pr.mix_freq = mix_freq;
+  pr.latency = latency;
+  pr.block_size = block_size;
+  server->pcm_device = (BsePcmDevice*) bse_device_open_best (BSE_TYPE_PCM_DEVICE, TRUE, TRUE,
+                                                             bse_main_args->pcm_drivers,
+                                                             pcm_request_callback, &pr, error ? NULL : &error);
+  if (!server->pcm_device)
+    server->pcm_device = (BsePcmDevice*) bse_device_open_best (BSE_TYPE_PCM_DEVICE, FALSE, TRUE,
+                                                               bse_main_args->pcm_drivers,
+                                                               pcm_request_callback, &pr, error ? NULL : &error);
+  if (!server->pcm_device)
+    sfi_error_msg (_("Show messages about PCM device selections problems"),
+                   _("Failed to open PCM devices, giving up: %s"),
+                   bse_error_blurb (error));
+  server->pcm_input_checked = FALSE;
+  return server->pcm_device ? BSE_ERROR_NONE : error;
+}
+
+static BseErrorType
+server_open_midi_device (BseServer *server)
+{
   g_return_val_if_fail (server->midi_device == NULL, BSE_ERROR_INTERNAL);
-  
-  /* pick MIDI driver implementation and try to open device */
-  children = g_type_children (BSE_TYPE_MIDI_DEVICE, &n);
-  for (i = 0; i < n; i++)
-    {
-      BseMidiDeviceClass *class = g_type_class_ref (children[i]);
-      if (class->driver_rating > rating)
-	{
-	  rating = class->driver_rating;
-	  choice = children[i];
-	}
-      g_type_class_unref (class);
-    }
-  g_free (children);
 
-  if (!choice)
+  BseErrorType error;
+  server->midi_device = (BseMidiDevice*) bse_device_open_best (BSE_TYPE_MIDI_DEVICE, TRUE, FALSE, bse_main_args->midi_drivers, NULL, NULL, &error);
+  if (!server->midi_device)
     {
-      /* return BSE_ERROR_DEVICE_NOT_AVAILABLE; */
-    retry_with_null_device:
-      choice = BSE_TYPE_MIDI_DEVICE_NULL;
+      SfiRing *ring = sfi_ring_prepend (NULL, "null");
+      server->midi_device = (BseMidiDevice*) bse_device_open_best (BSE_TYPE_MIDI_DEVICE_NULL, TRUE, FALSE, ring, NULL, NULL, NULL);
+      sfi_ring_free (ring);
+      if (server->midi_device)
+	sfi_warning_msg (_("Show messages about MIDI device selections problems"),
+                         _("Failed to open MIDI devices (reverting to null device): %s"),
+                         bse_error_blurb (error));
     }
-
-  server->midi_device = g_object_new (choice, NULL);
-  error = bse_midi_device_open (server->midi_device);
-  if (error)
-    {
-      if (choice != BSE_TYPE_MIDI_DEVICE_NULL)
-	sfi_info ("failed to open midi device %s (reverting to null device): %s",
-		  bse_object_debug_name (server->midi_device), bse_error_blurb (error));
-      g_object_unref (server->midi_device);
-      server->midi_device = NULL;
-      if (choice != BSE_TYPE_MIDI_DEVICE_NULL)
-	goto retry_with_null_device;
-    }
-  return error;
+  return server->midi_device ? BSE_ERROR_NONE : error;
 }
 
 BseErrorType
@@ -523,48 +524,64 @@ bse_server_open_devices (BseServer *self)
       return BSE_ERROR_NONE;
     }
 
+  /* lock playback/capture/latency settings */
+  bse_gconfig_lock ();
+  /* calculate block_size for pcm setup */
+  guint block_size, latency = BSE_GCONFIG (synth_latency), mix_freq = BSE_GCONFIG (synth_mixing_freq);
+  bse_engine_constrain (latency, mix_freq, BSE_GCONFIG (synth_control_freq), &block_size, NULL);
+  /* try opening devices */
   if (!error)
-    error = bse_server_open_pcm_device (self);
+    error = server_open_pcm_device (self, mix_freq, latency, block_size);
+  guint aligned_freq = bse_pcm_device_frequency_align (mix_freq);
+  if (error && aligned_freq != mix_freq)
+    {
+      mix_freq = aligned_freq;
+      bse_engine_constrain (latency, mix_freq, BSE_GCONFIG (synth_control_freq), &block_size, NULL);
+      BseErrorType new_error = server_open_pcm_device (self, mix_freq, latency, block_size);
+      error = new_error ? error : 0;
+    }
   if (!error)
-    error = bse_server_open_midi_device (self);
+    error = server_open_midi_device (self);
   if (!error)
     {
-      GslTrans *trans = gsl_trans_open ();
-      bse_pcm_handle_set_watermark (bse_pcm_device_get_handle (self->pcm_device), BSE_GCONFIG (synth_latency));
-      engine_init (self, bse_pcm_device_get_handle (self->pcm_device)->mix_freq);
-      self->pcm_imodule = bse_pcm_imodule_insert (bse_pcm_device_get_handle (self->pcm_device), trans);
+      BseTrans *trans = bse_trans_open ();
+      engine_init (self, bse_pcm_device_get_mix_freq (self->pcm_device));
+      BsePcmHandle *pcm_handle = bse_pcm_device_get_handle (self->pcm_device, bse_engine_block_size());
+      self->pcm_imodule = bse_pcm_imodule_insert (pcm_handle, trans);
       if (self->wave_file)
 	{
 	  BseErrorType error;
 	  self->pcm_writer = g_object_new (BSE_TYPE_PCM_WRITER, NULL);
-	  error = bse_pcm_writer_open (self->pcm_writer, self->wave_file, 2, gsl_engine_sample_freq ());
+	  error = bse_pcm_writer_open (self->pcm_writer, self->wave_file, 2, bse_engine_sample_freq ());
 	  if (error)
 	    {
-	      sfi_info ("failed to open WAV file \"%s\": %s", self->wave_file, bse_error_blurb (error));
+              sfi_error_msg (_("Show recording file errors"),
+                             _("Failed to open output file \"%s\": %s"),
+                             self->wave_file, bse_error_blurb (error));
 	      g_object_unref (self->pcm_writer);
 	      self->pcm_writer = NULL;
 	    }
 	}
-      self->pcm_omodule = bse_pcm_omodule_insert (bse_pcm_device_get_handle (self->pcm_device),
-						  self->pcm_writer, trans);
-      gsl_trans_commit (trans);
+      self->pcm_omodule = bse_pcm_omodule_insert (pcm_handle, self->pcm_writer, trans);
+      bse_trans_commit (trans);
       self->dev_use_count++;
     }
   else
     {
       if (self->midi_device)
 	{
-	  bse_midi_device_suspend (self->midi_device);
+	  bse_device_close (BSE_DEVICE (self->midi_device));
 	  g_object_unref (self->midi_device);
 	  self->midi_device = NULL;
 	}
       if (self->pcm_device)
 	{
-	  bse_pcm_device_suspend (self->pcm_device);
+	  bse_device_close (BSE_DEVICE (self->pcm_device));
 	  g_object_unref (self->pcm_device);
 	  self->pcm_device = NULL;
 	}
     }
+  bse_gconfig_unlock ();        /* engine_init() holds another lock count on success */
   return error;
 }
 
@@ -577,11 +594,14 @@ bse_server_close_devices (BseServer *self)
   self->dev_use_count--;
   if (!self->dev_use_count)
     {
-      GslTrans *trans = gsl_trans_open ();
+      BseTrans *trans = bse_trans_open ();
       bse_pcm_imodule_remove (self->pcm_imodule, trans);
       self->pcm_imodule = NULL;
       bse_pcm_omodule_remove (self->pcm_omodule, trans);
       self->pcm_omodule = NULL;
+      bse_trans_commit (trans);
+      /* wait until transaction has been processed */
+      bse_engine_wait_on_trans ();
       if (self->pcm_writer)
 	{
 	  if (self->pcm_writer->open)
@@ -589,11 +609,8 @@ bse_server_close_devices (BseServer *self)
 	  g_object_unref (self->pcm_writer);
 	  self->pcm_writer = NULL;
 	}
-      gsl_trans_commit (trans);
-      /* wait until transaction has been processed */
-      gsl_engine_wait_on_trans ();
-      bse_pcm_device_suspend (self->pcm_device);
-      bse_midi_device_suspend (self->midi_device);
+      bse_device_close (BSE_DEVICE (self->pcm_device));
+      bse_device_close (BSE_DEVICE (self->midi_device));
       engine_shutdown (self);
       g_object_unref (self->pcm_device);
       self->pcm_device = NULL;
@@ -602,7 +619,7 @@ bse_server_close_devices (BseServer *self)
     }
 }
 
-GslModule*
+BseModule*
 bse_server_retrieve_pcm_output_module (BseServer   *self,
 				       BseSource   *source,
 				       const gchar *uplink_name)
@@ -619,7 +636,7 @@ bse_server_retrieve_pcm_output_module (BseServer   *self,
 
 void
 bse_server_discard_pcm_output_module (BseServer *self,
-				      GslModule *module)
+				      BseModule *module)
 {
   g_return_if_fail (BSE_IS_SERVER (self));
   g_return_if_fail (module != NULL);
@@ -629,7 +646,7 @@ bse_server_discard_pcm_output_module (BseServer *self,
   bse_server_close_devices (self);
 }
 
-GslModule*
+BseModule*
 bse_server_retrieve_pcm_input_module (BseServer   *self,
 				      BseSource   *source,
 				      const gchar *uplink_name)
@@ -646,7 +663,7 @@ bse_server_retrieve_pcm_input_module (BseServer   *self,
 
 void
 bse_server_discard_pcm_input_module (BseServer *self,
-				     GslModule *module)
+				     BseModule *module)
 {
   g_return_if_fail (BSE_IS_SERVER (self));
   g_return_if_fail (module != NULL);
@@ -705,15 +722,26 @@ bse_server_script_error (BseServer   *server,
 }
 
 void
-bse_server_user_message (BseServer     *server,
-			 BseUserMsgType msg_type,
-			 const gchar   *message)
+bse_server_user_message (BseServer          *server,
+                         const gchar        *log_domain,
+                         BseUserMsgType      msg_type,
+                         const gchar        *config_blurb,
+                         const gchar        *message,
+                         gint                pid,
+                         const gchar        *process_name)
 {
   g_return_if_fail (BSE_IS_SERVER (server));
   g_return_if_fail (message != NULL);
-  
-  g_signal_emit (server, signal_user_message, 0,
-		 msg_type, message);
+
+  BseUserMsg umsg = { 0, };
+  umsg.log_domain = (char*) log_domain;
+  umsg.msg_type = msg_type;
+  umsg.config_blurb = (char*) config_blurb;
+  umsg.message = (char*) message;
+  umsg.pid = pid;
+  umsg.process = (char*) process_name;
+
+  g_signal_emit (server, signal_user_message, 0, &umsg);
 }
 
 void
@@ -914,7 +942,7 @@ iowatch_dispatch (GSource    *source,
   WSource *wsource = (WSource*) source;
   
   BSE_THREADS_ENTER ();
-  wsource->watch_func (wsource->data, &wsource->pfd);
+  wsource->watch_func (wsource->data, 1, &wsource->pfd);
   BSE_THREADS_LEAVE ();
   
   return TRUE;
@@ -972,8 +1000,8 @@ iowatch_remove (BseServer *server,
 typedef struct {
   GSource       source;
   guint         n_fds;
-  GPollFD       fds[GSL_ENGINE_MAX_POLLFDS];
-  GslEngineLoop loop;
+  GPollFD       fds[BSE_ENGINE_MAX_POLLFDS];
+  BseEngineLoop loop;
 } PSource;
 
 static gboolean
@@ -984,7 +1012,7 @@ engine_prepare (GSource *source,
   gboolean need_dispatch;
   
   BSE_THREADS_ENTER ();
-  need_dispatch = gsl_engine_prepare (&psource->loop);
+  need_dispatch = bse_engine_prepare (&psource->loop);
   if (psource->loop.fds_changed)
     {
       guint i;
@@ -1018,7 +1046,7 @@ engine_check (GSource *source)
   for (i = 0; i < psource->n_fds; i++)
     psource->loop.fds[i].revents = psource->fds[i].revents;
   psource->loop.revents_filled = TRUE;
-  need_dispatch = gsl_engine_check (&psource->loop);
+  need_dispatch = bse_engine_check (&psource->loop);
   BSE_THREADS_LEAVE ();
   
   return need_dispatch;
@@ -1030,7 +1058,7 @@ engine_dispatch (GSource    *source,
 		 gpointer    user_data)
 {
   BSE_THREADS_ENTER ();
-  gsl_engine_dispatch ();
+  bse_engine_dispatch ();
   BSE_THREADS_LEAVE ();
   
   return TRUE;
@@ -1050,7 +1078,7 @@ engine_init (BseServer *server,
   
   g_return_if_fail (server->engine_source == NULL);
   
-  bse_gconfig_lock ();		// FIXME: globals mix_freq
+  bse_gconfig_lock ();
   server->engine_source = g_source_new (&engine_gsource_funcs, sizeof (PSource));
   g_source_set_priority (server->engine_source, BSE_PRIORITY_HIGH);
   
@@ -1059,13 +1087,13 @@ engine_init (BseServer *server,
       guint mypid = bse_main_getpid();
       int current_priority;
       engine_is_initialized = TRUE;
-      gsl_engine_init (TRUE);
+      bse_engine_init (TRUE);
       /* lower priorities compared to engine if our priority range permits */
       current_priority = getpriority (PRIO_PROCESS, mypid);
       if (current_priority <= -2 && mypid)
         setpriority (PRIO_PROCESS, mypid, current_priority + 1);
     }
-  gsl_engine_configure (BSE_GCONFIG (synth_latency), mix_freq, BSE_GCONFIG (synth_control_freq));
+  bse_engine_configure (BSE_GCONFIG (synth_latency), mix_freq, BSE_GCONFIG (synth_control_freq));
 
   g_source_attach (server->engine_source, bse_main_context);
 }
@@ -1077,7 +1105,7 @@ engine_shutdown (BseServer *server)
   
   g_source_destroy (server->engine_source);
   server->engine_source = NULL;
-  gsl_engine_garbage_collect ();
+  bse_engine_garbage_collect ();
   // FIXME: need to be able to completely unintialize engine here
   bse_gconfig_unlock ();
 }
