@@ -47,8 +47,9 @@ struct _BseServer
   
   guint		   dev_use_count;
   BsePcmDevice    *pcm_device;
-  GslModule       *pcm_imodule;
-  GslModule       *pcm_omodule;
+  guint            pcm_input_checked : 1;
+  BseModule       *pcm_imodule;
+  BseModule       *pcm_omodule;
   BsePcmWriter	  *pcm_writer;
   BseMidiDevice	  *midi_device;
   
@@ -68,23 +69,24 @@ BseProject*	bse_server_find_project			(BseServer	*server,
 							 const gchar	*name);
 BseErrorType	bse_server_open_devices			(BseServer	*server);
 void		bse_server_close_devices		(BseServer	*server);
-GslModule*	bse_server_retrieve_pcm_output_module	(BseServer	*server,
+BseModule*	bse_server_retrieve_pcm_output_module	(BseServer	*server,
 							 BseSource	*source,
 							 const gchar	*uplink_name);
 void		bse_server_discard_pcm_output_module	(BseServer	*server,
-							 GslModule	*module);
-GslModule*	bse_server_retrieve_pcm_input_module	(BseServer	*server,
+							 BseModule	*module);
+BseModule*	bse_server_retrieve_pcm_input_module	(BseServer	*server,
 							 BseSource	*source,
 							 const gchar	*uplink_name);
 void		bse_server_discard_pcm_input_module	(BseServer	*server,
-							 GslModule	*module);
-GslModule*	bse_server_retrieve_midi_input_module	(BseServer	*server,
+							 BseModule	*module);
+void		bse_server_require_pcm_input    	(BseServer	*server);
+BseModule*	bse_server_retrieve_midi_input_module	(BseServer	*server,
 							 const gchar	*downlink_name,
 							 guint		 midi_channel_id,
 							 guint		 nth_note,
 							 guint		 signals[4]);
 void		bse_server_discard_midi_input_module	(BseServer	*server,
-							 GslModule	*module);
+							 BseModule	*module);
 void		bse_server_add_io_watch			(BseServer	*server,
 							 gint		 fd,
 							 GIOCondition	 events,
@@ -99,24 +101,28 @@ void		bse_server_registration			(BseServer          *server,
 							 BseRegistrationType rtype,
 							 const gchar	    *what,
 							 const gchar	    *error);
-void		bse_server_script_start			(BseServer       *server,
-							 BseJanitor	 *janitor);
-void		bse_server_script_error			(BseServer	 *server,
-							 const gchar	 *script_name,
-							 const gchar	 *proc_name,
-							 const gchar     *reason);
-void		bse_server_user_message			(BseServer	*server,
-							 BseUserMsgType  msg_type,
-							 const gchar    *message);
-BseErrorType	bse_server_run_remote			(BseServer	   *server,
-							 const gchar	   *process_name,
-							 SfiRing	   *params,
-							 const gchar       *script_name,
-							 const gchar       *proc_name,
-							 BseJanitor	  **janitor_p);
-void		bse_server_queue_kill_wire		(BseServer	*server,
-							 SfiComWire	*wire);
-void		bse_server_notify_gconfig		(BseServer	*server);
+void		bse_server_script_start			(BseServer          *server,
+							 BseJanitor	    *janitor);
+void		bse_server_script_error			(BseServer	    *server,
+							 const gchar	    *script_name,
+							 const gchar	    *proc_name,
+							 const gchar        *reason);
+void		bse_server_user_message		        (BseServer	    *server,
+                                                         const gchar        *log_domain,
+                                                         BseUserMsgType      msg_type,
+                                                         const gchar        *config_blurb,
+                                                         const gchar        *message,
+                                                         gint                pid,
+                                                         const gchar        *process_name);
+BseErrorType	bse_server_run_remote			(BseServer	    *server,
+							 const gchar	    *process_name,
+							 SfiRing	    *params,
+							 const gchar        *script_name,
+							 const gchar        *proc_name,
+							 BseJanitor	   **janitor_p);
+void		bse_server_queue_kill_wire		(BseServer	    *server,
+							 SfiComWire	    *wire);
+void		bse_server_notify_gconfig		(BseServer	    *server);
 
 G_END_DECLS
 
