@@ -48,6 +48,9 @@ G_BEGIN_DECLS
 #define BSE_SOURCE_OCHANNEL_IDENT(src,id) (BSE_SOURCE (src)->channel_defs->ochannel_idents[(id)])
 #define BSE_SOURCE_OCHANNEL_LABEL(src,id) (BSE_SOURCE (src)->channel_defs->ochannel_labels[(id)])
 #define BSE_SOURCE_OCHANNEL_BLURB(src,id) (BSE_SOURCE (src)->channel_defs->ochannel_blurbs[(id)])
+/* BseSourceClass */
+#define BSE_SOURCE_CLASS_N_ICHANNELS(sc)  (BSE_SOURCE_CLASS (sc)->channel_defs.n_ichannels)
+#define BSE_SOURCE_CLASS_N_OCHANNELS(sc)  (BSE_SOURCE_CLASS (sc)->channel_defs.n_ochannels)
 /*< private >*/
 #define	BSE_SOURCE_INPUT(src,id)	  (BSE_SOURCE (src)->inputs + (guint) (id))
 #define	BSE_SOURCE_OCHANNEL_OSTREAM(s,oc) ((oc) < BSE_SOURCE_N_OCHANNELS (s) ? (oc) : 0xffffffff)
@@ -60,24 +63,26 @@ G_BEGIN_DECLS
                                            BSE_SOURCE (s)->channel_defs->ijstreams[(ic)] & ~BSE_SOURCE_JSTREAM_FLAG : \
                                            0xffffffff)
 #define	BSE_SOURCE_JSTREAM_FLAG		  ((guint) 1 << 31)
-
+#define BSE_SOURCE_PRIVATE_INPUTS(src)    ((BSE_OBJECT_FLAGS (src) & BSE_SOURCE_FLAG_PRIVATE_INPUTS) != 0)
 
 /* --- BseSource flags --- */
 typedef enum	/*< skip >*/
 {
-  BSE_SOURCE_FLAG_PREPARED		= 1 << (BSE_ITEM_FLAGS_USHIFT + 0),
-  BSE_SOURCE_FLAG_COLLECTED		= 1 << (BSE_ITEM_FLAGS_USHIFT + 1)
+  BSE_SOURCE_FLAG_PRIVATE_INPUTS	= 1 << (BSE_ITEM_FLAGS_USHIFT + 0),
+  BSE_SOURCE_FLAG_PREPARED		= 1 << (BSE_ITEM_FLAGS_USHIFT + 1),
+  BSE_SOURCE_FLAG_COLLECTED		= 1 << (BSE_ITEM_FLAGS_USHIFT + 2)
 } BseSourceFlags;
-#define BSE_SOURCE_FLAGS_USHIFT        (BSE_ITEM_FLAGS_USHIFT + 2)
+#define BSE_SOURCE_FLAGS_USHIFT        (BSE_ITEM_FLAGS_USHIFT + 3)
 
 
 /* --- typedefs & structures --- */
 typedef union  _BseSourceInput		BseSourceInput;
 typedef struct _BseSourceOutput		BseSourceOutput;
 typedef struct _BseSourceChannelDefs	BseSourceChannelDefs;
+typedef struct _BseSourceProbes		BseSourceProbes;
 typedef void (*BseSourceFreeContextData) (BseSource *source,
 					  gpointer   data,
-					  GslTrans  *trans);
+					  BseTrans  *trans);
 struct _BseSourceOutput
 {
   BseSource *osource;
@@ -100,6 +105,7 @@ struct _BseSource
   GSList	       *outputs;
   gpointer		contexts; /* bsearch array of type BseSourceContext */
   SfiReal		pos_x, pos_y;
+  BseSourceProbes      *probes;
 };
 struct _BseSourceChannelDefs
 {
@@ -120,16 +126,21 @@ struct _BseSourceClass
 
   BseSourceChannelDefs	 channel_defs;
   
+  void          (*property_updated)     (BseSource      *source,        /* overridable method */
+                                         guint           property_id,
+                                         guint64         tick_stamp,
+                                         double          value,
+                                         GParamSpec     *pspec);
   void		(*prepare)		(BseSource	*source);
   void		(*context_create)	(BseSource	*source,
 					 guint		 context_handle,
-					 GslTrans	*trans);
+					 BseTrans	*trans);
   void		(*context_connect)	(BseSource	*source,
 					 guint		 context_handle,
-					 GslTrans	*trans);
+					 BseTrans	*trans);
   void		(*context_dismiss)	(BseSource	*source,
 					 guint		 context_handle,
-					 GslTrans	*trans);
+					 BseTrans	*trans);
   void		(*reset)		(BseSource	*source);
 
   /*< private >*/
@@ -141,10 +152,12 @@ struct _BseSourceClass
 			 guint		 ichannel,
 			 BseSource	*osource,
 			 guint		 ochannel);
-  GslClass *gsl_class;
+  BseModuleClass *engine_class;
+  gboolean        filtered_properties;
+  SfiRing        *unprepared_properties;
+  SfiRing        *automation_properties;
 };
 
-
 /* --- prototypes -- */
 guint		bse_source_find_ichannel	(BseSource	*source,
 						 const gchar    *ichannel_ident);
@@ -158,17 +171,17 @@ BseErrorType	bse_source_unset_input		(BseSource	*source,
 						 guint		 ichannel,
 						 BseSource	*osource,
 						 guint		 ochannel);
-#define	bse_source_must_set_input(is,ic,os,oc)	G_STMT_START {   \
-  BseSource *_is = (is), *_os = (os);				 \
-  guint _ic = (ic), _oc = (oc);					 \
-  BseErrorType _err = bse_source_set_input (_is, _ic, _os, _oc); \
-  if (_err != BSE_ERROR_NONE)					 \
-    g_warning ("%s: failed to connect module %s channel %u "	 \
-	       "to module %s channel %u: %s", G_STRLOC,		 \
-	       bse_object_debug_name (_is), _ic,		 \
-	       bse_object_debug_name (_os), _oc,		 \
-               bse_error_blurb (_err));				 \
-} G_STMT_END
+gboolean        bse_source_get_input            (BseSource      *source,
+                                                 guint           ichannel,
+                                                 BseSource     **osourcep,
+                                                 guint          *ochannelp);
+void           	bse_source_must_set_input_loc	(BseSource	*source,
+						 guint		 ichannel,
+						 BseSource	*osource,
+						 guint		 ochannel,
+                                                 const gchar    *strloc);
+#define bse_source_must_set_input(is,ic,os,oc)  \
+  bse_source_must_set_input_loc (is, ic, os, oc, G_STRLOC)
 
 
 /* --- source implementations --- */
@@ -184,34 +197,35 @@ guint	    bse_source_class_add_ochannel      	(BseSourceClass	*source_class,
 						 const gchar	*ident,
 						 const gchar	*label,
 						 const gchar	*blurb);
+void        bse_source_class_inherit_channels   (BseSourceClass *source_class);
 void		bse_source_set_context_imodule	(BseSource	*source,
 						 guint		 context_handle,
-						 GslModule	*imodule);
+						 BseModule	*imodule);
 void		bse_source_set_context_omodule	(BseSource	*source,
 						 guint		 context_handle,
-						 GslModule	*omodule);
-GslModule*	bse_source_get_context_imodule	(BseSource	*source,
+						 BseModule	*omodule);
+BseModule*	bse_source_get_context_imodule	(BseSource	*source,
 						 guint		 context_handle);
-GslModule*	bse_source_get_context_omodule	(BseSource	*source,
+BseModule*	bse_source_get_context_omodule	(BseSource	*source,
 						 guint		 context_handle);
 void		bse_source_flow_access_module	(BseSource	*source,
 						 guint		 context_handle,
 						 guint64	 tick_stamp,
-						 GslAccessFunc   access_func,
+						 BseEngineAccessFunc   access_func,
 						 gpointer	 data,
-						 GslFreeFunc	 data_free_func,
-						 GslTrans	*trans);
+						 BseFreeFunc	 data_free_func,
+						 BseTrans	*trans);
 void		bse_source_flow_access_modules	(BseSource	*source,
 						 guint64	 tick_stamp,
-						 GslAccessFunc   access_func,
+						 BseEngineAccessFunc   access_func,
 						 gpointer	 data,
-						 GslFreeFunc	 data_free_func,
-						 GslTrans	*trans);
+						 BseFreeFunc	 data_free_func,
+						 BseTrans	*trans);
 void		bse_source_access_modules	(BseSource	*source,
-						 GslAccessFunc   access_func,
+						 BseEngineAccessFunc   access_func,
 						 gpointer	 data,
-						 GslFreeFunc	 data_free_func,
-						 GslTrans	*trans);
+						 BseFreeFunc	 data_free_func,
+						 BseTrans	*trans);
 BseErrorType    bse_source_check_input          (BseSource      *source,
                                                  guint           ichannel,
                                                  BseSource      *osource,
@@ -223,49 +237,72 @@ void       bse_source_input_backup_to_undo      (BseSource      *source,
                                                  BseSource      *osource,
                                                  guint           ochannel);
 /* convenience */
-void    	bse_source_class_cache_gsl_class(BseSourceClass	*source_class,
-						 const GslClass *gsl_class);
-void		bse_source_set_context_module	(BseSource	*source,
+void   	   bse_source_class_cache_engine_class  (BseSourceClass	*source_class,
+						 const BseModuleClass *engine_class);
+void	   bse_source_set_context_module	(BseSource	*source,
 						 guint		 context_handle,
-						 GslModule	*module);
-void		bse_source_update_modules	(BseSource	*source,
+						 BseModule	*module);
+void	   bse_source_update_modules	        (BseSource	*source,
 						 guint		 member_offset,
 						 gpointer	 member_data,
 						 guint		 member_size,
-						 GslTrans	*trans);
-void		bse_source_clear_ichannels	(BseSource	*source);
-void		bse_source_clear_ochannels	(BseSource	*source);
+						 BseTrans	*trans);
+void	   bse_source_clear_ichannels	        (BseSource	*source);
+void	   bse_source_clear_ochannels	        (BseSource	*source);
 
+/* automation */
+typedef struct {
+  GParamSpec       *pspec;
+  guint             midi_channel;
+  BseMidiSignalType signal_type;
+} BseAutomationProperty;
+BseErrorType                 bse_source_set_automation_property   (BseSource         *source,
+                                                                   const gchar       *prop_name,
+                                                                   guint              midi_channel,
+                                                                   BseMidiSignalType  signal_type);
+void                         bse_source_get_automation_property   (BseSource         *source,
+                                                                   const gchar       *prop_name,
+                                                                   guint             *pmidi_channel,
+                                                                   BseMidiSignalType *psignal_type);
+BseAutomationProperty*       bse_source_get_automation_properties (BseSource         *source,
+                                                                   guint             *n_props);
 
 /* --- internal --- */
-SfiRing* bse_source_collect_inputs_recursive	(BseSource	*source);
-void     bse_source_free_collection		(SfiRing	*ring);
+SfiRing* bse_source_collect_inputs_flat 	(BseSource	*source);       /* sets mark */
+SfiRing* bse_source_collect_inputs_recursive	(BseSource	*source);       /* sets mark */
+void     bse_source_free_collection		(SfiRing	*ring);         /* unsets mark */
+gboolean bse_source_test_input_recursive        (BseSource      *source,        /* sets+unsets mark */
+                                                 BseSource      *test);
+gpointer bse_source_get_context_data		(BseSource	*source,
+						 guint		 context_handle);
 void	 bse_source_create_context_with_data	(BseSource	*source,
 						 guint		 context_handle,
 						 gpointer	 data,
 						 BseSourceFreeContextData free_data,
-						 GslTrans	*trans);
-gpointer bse_source_get_context_data		(BseSource	*source,
-						 guint		 context_handle);
+						 BseTrans	*trans);
 void		bse_source_create_context	(BseSource	*source,
 						 guint		 context_handle,
-						 GslTrans	*trans);
+						 BseTrans	*trans);
 void		bse_source_connect_context	(BseSource	*source,
 						 guint		 context_handle,
-						 GslTrans	*trans);
+						 BseTrans	*trans);
 void		bse_source_dismiss_context	(BseSource	*source,
 						 guint		 context_handle,
-						 GslTrans	*trans);
+						 BseTrans	*trans);
 void		bse_source_recreate_context	(BseSource	*source,
 						 guint		 context_handle,
-						 GslTrans	*trans);
+						 BseTrans	*trans);
 void		bse_source_prepare		(BseSource	*source);
 void		bse_source_reset		(BseSource	*source);
 guint*		bse_source_context_ids		(BseSource	*source,
 						 guint		*n_ids);
 gboolean	bse_source_has_context		(BseSource	*source,
 						 guint		 context_handle);
-
+SfiRing*        bse_source_list_omodules        (BseSource      *source);
+/* implemented in bseprobe.cc */
+void    bse_source_clear_probes                 (BseSource      *source);
+void    bse_source_class_add_probe_signals      (BseSourceClass *klass);
+void    bse_source_probes_modules_changed       (BseSource      *source);
 
 G_END_DECLS
 
