@@ -15,6 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
+#include "topconfig.h"
 #include "bseutils.h"
 
 #include <string.h>
@@ -22,7 +23,6 @@
 #include <sys/types.h>
 #include <dirent.h>
 #include <sys/stat.h>
-#include "gslieee754.h"
 
 
 /* --- record utils --- */
@@ -139,6 +139,52 @@ bse_note_sequence_length (BseNoteSequence *rec)
   return rec->notes->n_notes;
 }
 
+void
+bse_property_candidate_relabel (BsePropertyCandidates *pc,
+                                const gchar           *label,
+                                const gchar           *tooltip)
+{
+  g_free (pc->label);
+  pc->label = g_strdup (label);
+  g_free (pc->tooltip);
+  pc->tooltip = g_strdup (tooltip);
+}
+
+void
+bse_item_seq_remove (BseItemSeq *iseq,
+                     BseItem    *item)
+{
+  guint i;
+ restart:
+  for (i = 0; i < iseq->n_items; i++)
+    if (iseq->items[i] == item)
+      {
+        iseq->n_items--;
+        g_memmove (iseq->items + i, iseq->items + i + 1, (iseq->n_items - i) * sizeof (iseq->items[0]));
+        goto restart;
+      }
+}
+
+SfiRing*
+bse_item_seq_to_ring (BseItemSeq *iseq)
+{
+  SfiRing *ring = NULL;
+  guint i;
+  if (iseq)
+    for (i = 0; i < iseq->n_items; i++)
+      ring = sfi_ring_append (ring, iseq->items[i]);
+  return ring;
+}
+
+BseItemSeq*
+bse_item_seq_from_ring (SfiRing *ring)
+{
+  BseItemSeq *iseq = bse_item_seq_new();
+  SfiRing *node;
+  for (node = ring; node; node = sfi_ring_walk (node, ring))
+    bse_item_seq_append (iseq, node->data);
+  return iseq;
+}
 
 /* --- balance calculation --- */
 double
@@ -183,7 +229,22 @@ bse_balance_set (double balance,
 
 
 /* --- icons --- */
-BseIcon*
+typedef enum                    /*< skip >*/
+{
+  BSE_PIXDATA_RGB               = 3,
+  BSE_PIXDATA_RGBA              = 4,
+  BSE_PIXDATA_RGB_MASK          = 0x07,
+  BSE_PIXDATA_1BYTE_RLE         = (1 << 3),
+  BSE_PIXDATA_ENCODING_MASK     = 0x08
+} BsePixdataType;
+typedef struct
+{
+  BsePixdataType type : 8;
+  guint          width : 12;
+  guint          height : 12;
+  const guint8  *encoded_pix_data;
+} BsePixdata;
+static BseIcon*
 bse_icon_from_pixdata (const BsePixdata *pixdata)
 {
   BseIcon *icon;
@@ -379,6 +440,12 @@ bse_string_equals (gconstpointer string1,
     return string1 == string2;
 }
 
+const gchar*
+bse_intern_path_user_data (const gchar *dir)
+{
+  return g_intern_strconcat (BSE_PATH_USER_DATA ("/"), dir, NULL);
+}
+
 void
 bse_bbuffer_puts (gchar        bbuffer[BSE_BBUFFER_SIZE],
 		  const gchar *string)
