@@ -1,7 +1,7 @@
 
-/**
- ** Generated data (by mkcproc.pl)
- **/
+/*
+ * Generated data (by mkcproc.pl)
+ */
 #line 1 "bsewaverepo.proc"
 
 #include <bse/bseplugin.h>
@@ -49,7 +49,7 @@ load_file (BseWaveRepo *wrepo,
   wave = g_object_new (BSE_TYPE_WAVE,
 		       "uname", fname,
 		       NULL);
-  bse_wave_set_description_bits (wave, wdsc, TRUE);
+  bse_wave_set_description_bits (wave, wdsc, FALSE);
   g_free (fname);
   for (i = 0; i < wdsc->n_chunks; i++)
     {
@@ -302,6 +302,6 @@ BseExportNode* bse__builtin_init_bsewaverepo_gen_proc_c (void)
   return (BseExportNode*) &__enode_bse_sample_file_info;
 }
 
-/**
- ** Generated data ends here
- **/
+/*
+ * Generated data ends here
+ */
