@@ -32,7 +32,7 @@
 
 
 /* --- variables --- */
-volatile guint64     gsl_externvar_tick_stamp = 0;
+volatile guint64     bse_engine_exvar_tick_stamp = 0;   /* initialized to 1 upon gsl_init(), so 0==invalid */
 static guint64	     tick_stamp_system_time = 0;
 static guint         global_tick_stamp_leaps = 0;
 
@@ -45,16 +45,16 @@ static SfiMutex     global_tick_stamp_mutex = { 0, };
  *
  * Retrieve the global GSL tick counter stamp.
  * GSL increments its global tick stamp at certain intervals,
- * by specific amounts (refer to gsl_engine_init() for further
+ * by specific amounts (refer to bse_engine_init() for further
  * details). The tick stamp is a non-wrapping, unsigned 64bit
  * integer greater than 0. Threads can schedule sleep interruptions
  * at certain tick stamps with sfi_thread_awake_after() and
  * sfi_thread_awake_before(). Tick stamp updating occours at
  * GSL engine block processing boundaries, so code that can
  * guarantee to not run across those boundaries (for instance
- * GslProcessFunc() functions) may use the macro %GSL_TICK_STAMP
+ * BseProcessFunc() functions) may use the macro %GSL_TICK_STAMP
  * to retrieve the current tick in a faster manner (not involving
- * mutex locking). See also gsl_module_tick_stamp().
+ * mutex locking). See also bse_module_tick_stamp().
  * This function is MT-safe and may be called from any thread.
  */
 guint64
@@ -63,7 +63,7 @@ gsl_tick_stamp (void)
   guint64 stamp;
 
   GSL_SPIN_LOCK (&global_tick_stamp_mutex);
-  stamp = gsl_externvar_tick_stamp;
+  stamp = bse_engine_exvar_tick_stamp;
   GSL_SPIN_UNLOCK (&global_tick_stamp_mutex);
 
   return stamp;
@@ -90,7 +90,7 @@ gsl_tick_stamp_last (void)
   GslTickStampUpdate ustamp;
 
   GSL_SPIN_LOCK (&global_tick_stamp_mutex);
-  ustamp.tick_stamp = gsl_externvar_tick_stamp;
+  ustamp.tick_stamp = bse_engine_exvar_tick_stamp;
   ustamp.system_time = tick_stamp_system_time;
   GSL_SPIN_UNLOCK (&global_tick_stamp_mutex);
 
@@ -106,10 +106,10 @@ _gsl_tick_stamp_inc (void)
   g_return_if_fail (global_tick_stamp_leaps > 0);
 
   systime = sfi_time_system ();
-  newstamp = gsl_externvar_tick_stamp + global_tick_stamp_leaps;
+  newstamp = bse_engine_exvar_tick_stamp + global_tick_stamp_leaps;
 
   GSL_SPIN_LOCK (&global_tick_stamp_mutex);
-  gsl_externvar_tick_stamp = newstamp;
+  bse_engine_exvar_tick_stamp = newstamp;
   tick_stamp_system_time = systime;
   GSL_SPIN_UNLOCK (&global_tick_stamp_mutex);
 
@@ -202,50 +202,11 @@ gsl_byte_order_from_string (const gchar *string)
 }
 
 GslErrorType
-gsl_check_file (const gchar *file_name,
+gsl_file_check (const gchar *file_name,
 		const gchar *mode)
 {
-  guint access_mask = 0;
-  guint check_file, check_dir, check_link;
-  
-  if (strchr (mode, 'r'))	/* readable */
-    access_mask |= R_OK;
-  if (strchr (mode, 'w'))	/* writable */
-    access_mask |= W_OK;
-  if (strchr (mode, 'x'))	/* executable */
-    access_mask |= X_OK;
-
-  if (access_mask && access (file_name, access_mask) < 0)
-    goto have_errno;
-  
-  check_file = strchr (mode, 'f') != NULL;	/* open as file */
-  check_dir  = strchr (mode, 'd') != NULL;	/* open as directory */
-  check_link = strchr (mode, 'l') != NULL;	/* open as link */
-
-  if (check_file || check_dir || check_link)
-    {
-      struct stat st;
-      
-      if (check_link)
-	{
-	  if (lstat (file_name, &st) < 0)
-	    goto have_errno;
-	}
-      else if (stat (file_name, &st) < 0)
-	goto have_errno;
-
-      if (check_file && S_ISDIR (st.st_mode))
-        return GSL_ERROR_IS_DIR;
-
-      if ((check_file && !S_ISREG (st.st_mode)) ||
-	  (check_dir && !S_ISDIR (st.st_mode)) ||
-	  (check_link && !S_ISLNK (st.st_mode)))
-	return GSL_ERROR_OPEN_FAILED;
-    }
-
-  return GSL_ERROR_NONE;
-  
- have_errno:
+  if (sfi_file_check (file_name, mode))
+    return GSL_ERROR_NONE;
   return gsl_error_from_errno (errno, GSL_ERROR_OPEN_FAILED);
 }
 
@@ -257,8 +218,6 @@ gsl_error_from_errno (gint         sys_errno,
     {
     case ELOOP:
     case ENAMETOOLONG:
-    case ENODEV:
-    case ENOTDIR:
     case ENOENT:        return GSL_ERROR_NOT_FOUND;
     case EISDIR:        return GSL_ERROR_IS_DIR;
     case EROFS:
@@ -276,8 +235,10 @@ gsl_error_from_errno (gint         sys_errno,
     case EBUSY:         return GSL_ERROR_BUSY;
     case EAGAIN:
     case EINTR:		return GSL_ERROR_TEMP;
-    case EFAULT:
-    case EBADF:         return GSL_ERROR_INTERNAL;
+    case EFAULT:        return GSL_ERROR_INTERNAL;
+    case EBADF:
+    case ENOTDIR:
+    case ENODEV:
     case EINVAL:
     default:            return fallback;
     }
@@ -476,7 +437,7 @@ gsl_init (const GslConfigValue values[])
 
   g_return_if_fail (gsl_config == NULL);	/* assert single initialization */
 
-  gsl_externvar_tick_stamp = 1;
+  bse_engine_exvar_tick_stamp = 1;
 
   /* configure permanent config record */
   if (config)
@@ -516,10 +477,8 @@ gsl_init (const GslConfigValue values[])
 
   /* initialize subsystems */
   sfi_mutex_init (&global_tick_stamp_mutex);
-  _gsl_init_signal ();
   _gsl_init_fd_pool ();
   _gsl_init_data_caches ();
-  _gsl_init_engine_utils ();
   _gsl_init_loader_gslwave ();
   _gsl_init_loader_aiff ();
   _gsl_init_loader_wav ();
