@@ -75,7 +75,7 @@ GslErrorType gsl_error_from_errno	(gint		 sys_errno,
 GslErrorType gsl_error_select           (guint           n_errors,
                                          GslErrorType    first_error,
                                          ...);
-GslErrorType gsl_check_file		(const gchar	*file_name,
+GslErrorType gsl_file_check		(const gchar	*file_name,
 					 const gchar	*mode);
 
 
@@ -109,18 +109,16 @@ guint            gsl_progress_printerr  (gpointer          message,
 /* --- implementation details --- */
 void	       _gsl_tick_stamp_inc	(void);
 void	       _gsl_tick_stamp_set_leap (guint		 ticks);
-void	_gsl_init_signal		(void);
 void	_gsl_init_fd_pool		(void);
 void	_gsl_init_data_caches		(void);
-void	_gsl_init_engine_utils		(void);
 void	_gsl_init_loader_gslwave	(void);
 void	_gsl_init_loader_aiff		(void);
 void	_gsl_init_loader_wav		(void);
 void	_gsl_init_loader_oggvorbis	(void);
 void	_gsl_init_loader_mad		(void);
 #define		GSL_N_IO_RETRIES	(5)
-#define		_GSL_TICK_STAMP_VAL()	(gsl_externvar_tick_stamp + 0)
-extern volatile guint64	gsl_externvar_tick_stamp;
+#define		_GSL_TICK_STAMP_VAL()	(bse_engine_exvar_tick_stamp + 0)
+extern volatile guint64	bse_engine_exvar_tick_stamp;
 
 
 G_END_DECLS
