@@ -96,7 +96,7 @@ skipchr (const guint8 *mem,
   const guint8 *p = mem, *bound = p + maxlen;
 
   while (p < bound)
-    if_reject (*p++ != byte)
+    if (UNLIKELY (*p++ != byte))
       return (guint8*) p - 1;
   return NULL;
 }
@@ -197,7 +197,7 @@ gsl_wave_file_info_load (const gchar  *file_name,
   else /* no loader match */
     {
       /* try to provide apropriate error code */
-      error = gsl_check_file (file_name, "rf");
+      error = gsl_file_check (file_name, "rf");
       if (!error)
 	error = GSL_ERROR_FORMAT_UNKNOWN;
     }
