@@ -17,12 +17,10 @@
  * Boston, MA 02111-1307, USA.
  */
 #define GSL_EXTENSIONS
-#include <bse/gslmath.h>
 #include <bse/gslcommon.h>
-#include <bse/gslmath.h>
 #include <bse/gslfilter.h>
 #include <bse/gslloader.h>
-#include <bse/gslsignal.h>
+#include <bse/bsemathsignal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -92,7 +90,7 @@ main (int   argc,
       gdouble r = 0;
       for (i = 0; i < 10; i++)
 	for (f = -1; f <= 1.0; f += 1.0 / 10000000.0)
-	  r += gsl_approx_exp2 (f);
+	  r += bse_approx_exp2 (f);
       return (r > 0) & 8;
     }
   else if (strcmp (arg, "old-exp2-run") == 0)
@@ -101,7 +99,7 @@ main (int   argc,
       gdouble r = 0;
       for (i = 0; i < 10; i++)
 	for (f = -1; f <= 1.0; f += 1.0 / 10000000.0)
-	  r += _gsl_signal_exp2_fraction (f); // gsl_signal_exp2 (f);
+	  r += _bse_signal_exp2_fraction (f); // bse_signal_exp2 (f);
       return (r > 0) & 8;
     }
   else if (strcmp (arg, "libc-exp-run") == 0)
@@ -144,17 +142,18 @@ main (int   argc,
       const gchar *file = pshift ();
       
       g_print ("file test for \"%s\":\n", file);
-      g_print ("  is readable   : %s\n", gsl_strerror (gsl_check_file (file, "r")));
-      g_print ("  is writable   : %s\n", gsl_strerror (gsl_check_file (file, "w")));
-      g_print ("  is executable : %s\n", gsl_strerror (gsl_check_file (file, "x")));
-      g_print ("  is file       : %s\n", gsl_strerror (gsl_check_file (file, "f")));
-      g_print ("  is directory  : %s\n", gsl_strerror (gsl_check_file (file, "d")));
-      g_print ("  is link       : %s\n", gsl_strerror (gsl_check_file (file, "l")));
+      g_print ("  is readable   : %s\n", gsl_strerror (gsl_file_check (file, "r")));
+      g_print ("  is writable   : %s\n", gsl_strerror (gsl_file_check (file, "w")));
+      g_print ("  is executable : %s\n", gsl_strerror (gsl_file_check (file, "x")));
+      g_print ("  is file       : %s\n", gsl_strerror (gsl_file_check (file, "f")));
+      g_print ("  is directory  : %s\n", gsl_strerror (gsl_file_check (file, "d")));
+      g_print ("  is link       : %s\n", gsl_strerror (gsl_file_check (file, "l")));
     }
   else if (strcmp (arg, "ring-test") == 0)
     {
       ring_test ();
     }
+#if 0
   else if (strcmp (arg, "rf") == 0)
     {
       double x, y, z;
@@ -182,29 +181,29 @@ main (int   argc,
     }
   else if (strcmp (arg, "snc") == 0)
     {
-      GslComplex u, emmc;
+      BseComplex u, emmc;
       u.re = atof (pshift ());
       u.im = atof (pshift ());
       emmc.re = atof (pshift ());
       emmc.im = atof (pshift ());
       
       g_print ("snc(%s, %s) = %s\n",
-	       gsl_complex_str (u),
-	       gsl_complex_str (emmc),
-	       gsl_complex_str (gsl_complex_ellip_sn (u, emmc)));
+	       bse_complex_str (u),
+	       bse_complex_str (emmc),
+	       bse_complex_str (bse_complex_ellip_sn (u, emmc)));
     }
   else if (strcmp (arg, "sci_snc") == 0)
     {
-      GslComplex u, k2;
+      BseComplex u, k2;
       u.re = atof (pshift ());
       u.im = atof (pshift ());
       k2.re = atof (pshift ());
       k2.im = atof (pshift ());
       
       g_print ("sci_snc(%s, %s) = %s\n",
-	       gsl_complex_str (u),
-	       gsl_complex_str (k2),
-	       gsl_complex_str (gsl_complex_ellip_sn (u, gsl_complex_sub (gsl_complex (1.0, 0), k2))));
+	       bse_complex_str (u),
+	       bse_complex_str (k2),
+	       bse_complex_str (bse_complex_ellip_sn (u, bse_complex_sub (bse_complex (1.0, 0), k2))));
     }
   else if (strcmp (arg, "asn") == 0)
     {
@@ -216,15 +215,15 @@ main (int   argc,
     }
   else if (strcmp (arg, "asnc") == 0)
     {
-      GslComplex y, emmc;
+      BseComplex y, emmc;
       y.re = atof (pshift ());
       y.im = atof (pshift ());
       emmc.re = atof (pshift ());
       emmc.im = atof (pshift ());
       
       g_print ("asnc(%s, %s) = %s\n",
-	       gsl_complex_str (y), gsl_complex_str (emmc),
-	       gsl_complex_str (gsl_complex_ellip_asn (y, emmc)));
+	       bse_complex_str (y), bse_complex_str (emmc),
+	       bse_complex_str (bse_complex_ellip_asn (y, emmc)));
       g_print ("asn(%f, %f = %."PREC"f\n",
 	       y.re, emmc.re, gsl_ellip_asn (y.re, emmc.re));
     }
@@ -244,70 +243,112 @@ main (int   argc,
     }
   else if (strcmp (arg, "sci_asnc") == 0)
     {
-      GslComplex y, k2;
+      BseComplex y, k2;
       y.re = atof (pshift ());
       y.im = atof (pshift ());
       k2.re = atof (pshift ());
       k2.im = atof (pshift ());
       g_print ("sci_asnc(%s, %s) = %s\n",
-	       gsl_complex_str (y), gsl_complex_str (k2),
-	       gsl_complex_str (gsl_complex_ellip_asn (y, gsl_complex_sub (gsl_complex (1.0, 0), k2))));
+	       bse_complex_str (y), bse_complex_str (k2),
+	       bse_complex_str (bse_complex_ellip_asn (y, bse_complex_sub (bse_complex (1.0, 0), k2))));
       g_print ("asn(%f, %f = %."PREC"f\n",
 	       y.re, k2.re, gsl_ellip_asn (y.re, 1.0 - k2.re));
     }
+  else if (strncmp (arg, "poly", 4) == 0)
+    {
+      guint order;
+      arg = arg + 4;
+      order = 2;
+      {
+	double a[100] = { 1, 2, 1 }, b[100] = { 1, -3./2., 0.5 };
+	g_print ("# Test order=%u  norm=%f:\n",
+		 order,
+		 bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
+	g_print ("H%u(z)=%s/%s\n", order,
+		 bse_poly_str (order, a, "z"),
+		 bse_poly_str (order, b, "z"));
+	if (*arg)
+	  {
+	    BseComplex root, roots[100];
+	    guint i;
+	    
+	    if (*arg == 'r')
+	      {
+		g_print ("#roots:\n");
+		bse_poly_complex_roots (order, a, roots);
+		for (i = 0; i < order; i++)
+		  {
+		    root = bse_complex_div (bse_complex (1, 0), roots[i]);
+		    g_print ("%+.14f %+.14f # %.14f\n", root.re, root.im, bse_complex_abs (root));
+		  }
+	      }
+	    if (*arg == 'p')
+	      {
+		g_print ("#poles:\n");
+		bse_poly_complex_roots (order, b, roots);
+		for (i = 0; i < order; i++)
+		  {
+		    root = bse_complex_div (bse_complex (1, 0), roots[i]);
+		    g_print ("%+.14f %+.14f # %.14f\n", root.re, root.im, bse_complex_abs (root));
+		  }
+	      }
+	  }
+      }
+    }
+#endif
   else if (strcmp (arg, "sin") == 0)
     {
-      GslComplex phi;
+      BseComplex phi;
       phi.re = atof (pshift ());
       phi.im = atof (pshift ());
       g_print ("sin(%s) = %s\n",
-	       gsl_complex_str (phi),
-	       gsl_complex_str (gsl_complex_sin (phi)));
+	       bse_complex_str (phi),
+	       bse_complex_str (bse_complex_sin (phi)));
     }
   else if (strcmp (arg, "cos") == 0)
     {
-      GslComplex phi;
+      BseComplex phi;
       phi.re = atof (pshift ());
       phi.im = atof (pshift ());
       g_print ("cos(%s) = %s\n",
-	       gsl_complex_str (phi),
-	       gsl_complex_str (gsl_complex_cos (phi)));
+	       bse_complex_str (phi),
+	       bse_complex_str (bse_complex_cos (phi)));
     }
   else if (strcmp (arg, "tan") == 0)
     {
-      GslComplex phi;
+      BseComplex phi;
       phi.re = atof (pshift ());
       phi.im = atof (pshift ());
       g_print ("tan(%s) = %s\n",
-	       gsl_complex_str (phi),
-	       gsl_complex_str (gsl_complex_tan (phi)));
+	       bse_complex_str (phi),
+	       bse_complex_str (bse_complex_tan (phi)));
     }
   else if (strcmp (arg, "sinh") == 0)
     {
-      GslComplex phi;
+      BseComplex phi;
       phi.re = atof (pshift ());
       phi.im = atof (pshift ());
       g_print ("sinh(%s) = %s\n",
-	       gsl_complex_str (phi),
-	       gsl_complex_str (gsl_complex_sinh (phi)));
+	       bse_complex_str (phi),
+	       bse_complex_str (bse_complex_sinh (phi)));
     }
   else if (strcmp (arg, "cosh") == 0)
     {
-      GslComplex phi;
+      BseComplex phi;
       phi.re = atof (pshift ());
       phi.im = atof (pshift ());
       g_print ("cosh(%s) = %s\n",
-	       gsl_complex_str (phi),
-	       gsl_complex_str (gsl_complex_cosh (phi)));
+	       bse_complex_str (phi),
+	       bse_complex_str (bse_complex_cosh (phi)));
     }
   else if (strcmp (arg, "tanh") == 0)
     {
-      GslComplex phi;
+      BseComplex phi;
       phi.re = atof (pshift ());
       phi.im = atof (pshift ());
       g_print ("tanh(%s) = %s\n",
-	       gsl_complex_str (phi),
-	       gsl_complex_str (gsl_complex_tanh (phi)));
+	       bse_complex_str (phi),
+	       bse_complex_str (bse_complex_tanh (phi)));
     }
   else if (strcmp (arg, "midi2freq") == 0)
     {
@@ -316,7 +357,7 @@ main (int   argc,
       note = CLAMP (note, 0, 128);
       g_print ("midi2freq(%u) = %f\n",
 	       note,
-	       gsl_temp_freq (gsl_get_config ()->kammer_freq,
+	       bse_temp_freq (gsl_get_config ()->kammer_freq,
 			      note - gsl_get_config ()->midi_kammer_note));
     }
   else if (strcmp (arg, "blp") == 0)
@@ -325,13 +366,13 @@ main (int   argc,
       order = atoi (pshift ()); order = MAX (order, 1);
       f = atof (pshift ());
       e = atof (pshift ());
-      f *= GSL_PI / 2.;
+      f *= PI / 2.;
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
       gsl_filter_butter_lp (order, f, e, a, b);
       g_print ("# Lowpass Butterworth filter order=%u freq=%f epsilon(s^2)=%f norm0=%f:\n",
 	       order, f, e,
-	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
+	       bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
       filter_label = "BL";
     }
   else if (strcmp (arg, "bhp") == 0)
@@ -340,14 +381,14 @@ main (int   argc,
       order = atoi (pshift ()); order = MAX (order, 1);
       f = atof (pshift ());
       e = atof (pshift ());
-      f *= GSL_PI / 2.;
+      f *= PI / 2.;
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
       
       gsl_filter_butter_hp (order, f, e, a, b);
       g_print ("# Highpass Butterworth filter order=%u freq=%f epsilon(s^2)=%f norm0=%f:\n",
 	       order, f, e,
-	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
+	       bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
       filter_label = "BH";
     }
   else if (strcmp (arg, "bbp") == 0)
@@ -357,15 +398,15 @@ main (int   argc,
       f1 = atof (pshift ());
       f2 = atof (pshift ());
       e = atof (pshift ());
-      f1 *= GSL_PI / 2.;
-      f2 *= GSL_PI / 2.;
+      f1 *= PI / 2.;
+      f2 *= PI / 2.;
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
       
       gsl_filter_butter_bp (order, f1, f2, e, a, b);
       g_print ("# Bandpass Butterworth filter order=%u freq1=%f freq2=%f epsilon(s^2)=%f norm0=%f:\n",
 	       order, f1, f2, e,
-	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
+	       bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
       filter_label = "BP";
     }
   else if (strcmp (arg, "bbs") == 0)
@@ -375,15 +416,15 @@ main (int   argc,
       f1 = atof (pshift ());
       f2 = atof (pshift ());
       e = atof (pshift ());
-      f1 *= GSL_PI / 2.;
-      f2 *= GSL_PI / 2.;
+      f1 *= PI / 2.;
+      f2 *= PI / 2.;
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
       
       gsl_filter_butter_bs (order, f1, f2, e, a, b);
       g_print ("# Bandstop Butterworth filter order=%u freq1=%f freq2=%f epsilon(s^2)=%f norm0=%f:\n",
 	       order, f1, f2, e,
-	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
+	       bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
       filter_label = "BS";
     }
   else if (strcmp (arg, "t1l") == 0)
@@ -392,14 +433,14 @@ main (int   argc,
       order = atoi (pshift ()); order = MAX (order, 1);
       f = atof (pshift ());
       e = atof (pshift ());
-      f *= GSL_PI / 2.;
+      f *= PI / 2.;
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
       
       gsl_filter_tscheb1_lp (order, f, e, a, b);
       g_print ("# Lowpass Tschebyscheff Type1 order=%u freq=%f epsilon(s^2)=%f norm0=%f:\n",
 	       order, f, e,
-	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
+	       bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
       filter_label = "T1L";
     }
   else if (strcmp (arg, "t1h") == 0)
@@ -408,7 +449,7 @@ main (int   argc,
       order = atoi (pshift ()); order = MAX (order, 1);
       f = atof (pshift ());
       e = atof (pshift ());
-      f *= GSL_PI / 2.;
+      f *= PI / 2.;
       
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
@@ -416,7 +457,7 @@ main (int   argc,
       gsl_filter_tscheb1_hp (order, f, e, a, b);
       g_print ("# Highpass Tschebyscheff Type1 order=%u freq=%f epsilon(s^2)=%f norm0=%f:\n",
 	       order, f, e,
-	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
+	       bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
       filter_label = "T1H";
     }
   else if (strcmp (arg, "t1s") == 0)
@@ -426,8 +467,8 @@ main (int   argc,
       fc = atof (pshift ());
       fr = atof (pshift ());
       e = atof (pshift ());
-      fc *= GSL_PI / 2.;
-      fr *= GSL_PI / 2.;
+      fc *= PI / 2.;
+      fr *= PI / 2.;
       
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
@@ -435,7 +476,7 @@ main (int   argc,
       gsl_filter_tscheb1_bs (order, fc, fr, e, a, b);
       g_print ("# Bandstop Tschebyscheff Type1 order=%u freq_c=%f freq_r=%f epsilon(s^2)=%f norm=%f:\n",
 	       order, fc, fr, e,
-	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
+	       bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
       filter_label = "T1S";
     }
   else if (strcmp (arg, "t1p") == 0)
@@ -445,8 +486,8 @@ main (int   argc,
       fc = atof (pshift ());
       fr = atof (pshift ());
       e = atof (pshift ());
-      fc *= GSL_PI / 2.;
-      fr *= GSL_PI / 2.;
+      fc *= PI / 2.;
+      fr *= PI / 2.;
       
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
@@ -454,7 +495,7 @@ main (int   argc,
       gsl_filter_tscheb1_bp (order, fc, fr, e, a, b);
       g_print ("# Bandpass Tschebyscheff Type1 order=%u freq_c=%f freq_r=%f epsilon(s^2)=%f norm=%f:\n",
 	       order, fc, fr, e,
-	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
+	       bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
       filter_label = "T1P";
     }
   else if (strcmp (arg, "t2l") == 0)
@@ -464,7 +505,7 @@ main (int   argc,
       f = atof (pshift ());
       st = atof (pshift ());
       e = atof (pshift ());
-      f *= GSL_PI / 2.;
+      f *= PI / 2.;
       
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
@@ -472,7 +513,7 @@ main (int   argc,
       gsl_filter_tscheb2_lp (order, f, st, e, a, b);
       g_print ("# Lowpass Tschebyscheff Type2 order=%u freq=%f steepness=%f (%f) epsilon(s^2)=%f norm=%f:\n",
 	       order, f, st, f * (1.+st), e,
-	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
+	       bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
       filter_label = "T2L";
     }
   else if (strcmp (arg, "t2h") == 0)
@@ -482,15 +523,15 @@ main (int   argc,
       f = atof (pshift ());
       st = atof (pshift ());
       e = atof (pshift ());
-      f *= GSL_PI / 2.;
+      f *= PI / 2.;
       
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
       
       gsl_filter_tscheb2_hp (order, f, st, e, a, b);
       g_print ("# Highpass Tschebyscheff Type2 order=%u freq=%f steepness=%f (%f, %f) epsilon(s^2)=%f norm=%f:\n",
-	       order, f, st, GSL_PI - f, (GSL_PI - f) * (1.+st), e,
-	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
+	       order, f, st, PI - f, (PI - f) * (1.+st), e,
+	       bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
       filter_label = "T2H";
     }
   else if (strcmp (arg, "t2p") == 0)
@@ -501,8 +542,8 @@ main (int   argc,
       f2 = atof (pshift ());
       st = atof (pshift ());
       e = atof (pshift ());
-      f1 *= GSL_PI / 2.;
-      f2 *= GSL_PI / 2.;
+      f1 *= PI / 2.;
+      f2 *= PI / 2.;
       
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
@@ -510,7 +551,7 @@ main (int   argc,
       gsl_filter_tscheb2_bp (order, f1, f2, st, e, a, b);
       g_print ("# Bandpass Tschebyscheff Type2 order=%u freq1=%f freq2=%f steepness=%f epsilon(s^2)=%f norm=%f:\n",
 	       order, f1, f2, st, e,
-	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
+	       bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
       filter_label = "T2P";
     }
   else if (strcmp (arg, "t2s") == 0)
@@ -521,8 +562,8 @@ main (int   argc,
       f2 = atof (pshift ());
       st = atof (pshift ());
       e = atof (pshift ());
-      f1 *= GSL_PI / 2.;
-      f2 *= GSL_PI / 2.;
+      f1 *= PI / 2.;
+      f2 *= PI / 2.;
       
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
@@ -530,7 +571,7 @@ main (int   argc,
       gsl_filter_tscheb2_bs (order, f1, f2, st, e, a, b);
       g_print ("# Bandstop Tschebyscheff Type2 order=%u freq1=%f freq2=%f steepness=%f epsilon(s^2)=%f norm=%f:\n",
 	       order, f1, f2, st, e,
-	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
+	       bse_poly_eval (order, a, 1) / bse_poly_eval (order, b, 1));
       filter_label = "T2S";
     }
   else if (strcmp (arg, "scan") == 0)
@@ -554,7 +595,7 @@ main (int   argc,
 	  
 	  if (f[0] && v[0])
 	    {
-	      freq[n_points] = atof (f) * GSL_PI;
+	      freq[n_points] = atof (f) * PI;
 	      value[n_points] = atof (v);
 	      n_points++;
 	    }
@@ -562,48 +603,7 @@ main (int   argc,
       while (f[0] && v[0]);
       
       gsl_filter_fir_approx (iorder, a, n_points, freq, value);
-      g_print ("FIR%u(z)=%s\n", iorder, gsl_poly_str (iorder, a, "z"));
-    }
-  else if (strncmp (arg, "poly", 4) == 0)
-    {
-      guint order;
-      arg = arg + 4;
-      order = 2;
-      {
-	double a[100] = { 1, 2, 1 }, b[100] = { 1, -3./2., 0.5 };
-	g_print ("# Test order=%u  norm=%f:\n",
-		 order,
-		 gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
-	g_print ("H%u(z)=%s/%s\n", order,
-		 gsl_poly_str (order, a, "z"),
-		 gsl_poly_str (order, b, "z"));
-	if (*arg)
-	  {
-	    GslComplex root, roots[100];
-	    guint i;
-	    
-	    if (*arg == 'r')
-	      {
-		g_print ("#roots:\n");
-		gsl_poly_complex_roots (order, a, roots);
-		for (i = 0; i < order; i++)
-		  {
-		    root = gsl_complex_div (gsl_complex (1, 0), roots[i]);
-		    g_print ("%+.14f %+.14f # %.14f\n", root.re, root.im, gsl_complex_abs (root));
-		  }
-	      }
-	    if (*arg == 'p')
-	      {
-		g_print ("#poles:\n");
-		gsl_poly_complex_roots (order, b, roots);
-		for (i = 0; i < order; i++)
-		  {
-		    root = gsl_complex_div (gsl_complex (1, 0), roots[i]);
-		    g_print ("%+.14f %+.14f # %.14f\n", root.re, root.im, gsl_complex_abs (root));
-		  }
-	      }
-	  }
-      }
+      g_print ("FIR%u(z)=%s\n", iorder, bse_poly_str (iorder, a, "z"));
     }
   else
     usage ();
@@ -624,8 +624,8 @@ main (int   argc,
       else if (filter_mode == FILTER_GNUPLOT)
 	{
 	  g_print ("%s%u(z)=%s/%s\n", filter_label, order,
-		   gsl_poly_str (order, a, "z"),
-		   gsl_poly_str (order, b, "z"));
+		   bse_poly_str (order, a, "z"),
+		   bse_poly_str (order, b, "z"));
 	}
       else
 	g_error ("unknown filter_mode");
@@ -692,7 +692,8 @@ print_int_ring (SfiRing *ring)
 
 static gint
 ints_cmp (gconstpointer d1,
-	  gconstpointer d2)
+	  gconstpointer d2,
+          gpointer      data)
 {
   gint i1 = (gint) d1;
   gint i2 = (gint) d2;
@@ -731,7 +732,7 @@ ring_test (void)
 	ring = sfi_ring_append (ring, (gpointer) (data_array[n][i]));
       g_print ("source: ");
       print_int_ring (ring);
-      ring = sfi_ring_sort (ring, ints_cmp);
+      ring = sfi_ring_sort (ring, ints_cmp, NULL);
       g_print (" sorted: ");
       print_int_ring (ring);
       g_print ("\n");
