@@ -29,7 +29,7 @@ namespace {
 using namespace Sfidl;
 using namespace std;
 
-class CodeGeneratorCoreC : public CodeGeneratorCBase {
+class CodeGeneratorCoreC : public CodeGenerator {
   const char*
   intern (const string &str)
   {
@@ -82,13 +82,13 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
   void
   CodeGeneratorCoreC::help()
   {
-    CodeGeneratorCBase::help();
+    CodeGenerator::help();
     fprintf (stderr, " --init <name>               set the name of the init function\n");
   }
   OptionVector
   getOptions()
   {
-    OptionVector opts = CodeGeneratorCBase::getOptions();
+    OptionVector opts = CodeGenerator::getOptions();
     
     opts.push_back (make_pair ("--init", true));
     
@@ -104,7 +104,7 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
       }
     else
       {
-        CodeGeneratorCBase::setOption (option, value);
+        CodeGenerator::setOption (option, value);
       }
   }
 
@@ -176,6 +176,43 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
       default:          return intern (var);
       }
   }
+  string
+  construct_pspec (const Param &pdef)
+  {
+    string pspec;
+    const string group = (pdef.group != "") ? pdef.group.escaped() : "NULL";
+    string pname, parg;
+    switch (parser.typeOf (pdef.type))
+      {
+      case CHOICE:
+        pname = "Choice";
+        parg = makeLowerName (pdef.type) + "_get_values()";
+        break;
+      case RECORD:
+        pname = "Record";
+        parg = makeLowerName (pdef.type) + "_fields";
+        break;
+      case SEQUENCE:
+        pname = "Sequence";
+        parg = makeLowerName (pdef.type) + "_content";
+        break;
+      case OBJECT:
+        pname = "Object";
+        break;
+      default:
+        pname = pdef.pspec;
+        break;
+      }
+    pspec = "sfidl_pspec_" + pname;
+    if (parg != "")
+      parg = string (", ") + parg;
+    if (pdef.args == "")
+      pspec += "_default (" + group + ",\"" + pdef.name + parg + "\")";
+    else
+      pspec += " (" + group + ",\"" + pdef.name + "\"," + pdef.args + parg + ")";
+    return pspec;
+  }
+
   void
   generate_enum_type_id_prototypes ()
   {
@@ -239,9 +276,6 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
             printf ("  %s = %d,\n", ename.c_str(), value);
           }
         printf ("} %s;\n", mname.c_str());
-        
-        printf ("const gchar* %s_to_choice (%s value);\n", lname.c_str(), mname.c_str());
-        printf ("%s %s_from_choice (const gchar *choice);\n", mname.c_str(), lname.c_str());
       }
   }
   void
@@ -303,7 +337,7 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
         printf ("  return choice_values;\n");
         printf ("}\n");
         
-        printf ("GType %s = 0;\n", makeGTypeName (ei->name).c_str());
+        printf ("GType %s = 0;\n", make_TYPE_MACRO (ei->name));
         printf ("\n");
         
         enumCount++;
@@ -523,7 +557,7 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
         printf ("%s_rec2boxed (const GValue *src_value, GValue *dest_value)\n", name.c_str());
         printf ("{\n");
         printf ("  SfiRec *rec = sfi_value_get_rec (src_value);\n");
-        printf ("  g_value_set_boxed_take_ownership (dest_value,\n");
+        printf ("  g_value_take_boxed (dest_value,\n");
         printf ("    rec ? %s_from_rec (rec) : NULL);\n", name.c_str());
         printf ("}\n");
         
@@ -535,7 +569,7 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
         printf ("  %s_rec2boxed,\n", name.c_str());
         printf ("  %s_info_strings\n", name.c_str());
         printf ("};\n");
-        printf ("GType %s = 0;\n", makeGTypeName (ri->name).c_str());
+        printf ("GType %s = 0;\n", make_TYPE_MACRO (ri->name));
       }
   }
   void
@@ -559,7 +593,7 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
         if (parser.fromInclude (si->name)) continue;
         
         string mname = makeMixedName (si->name.c_str());
-        string array = typeArray (si->content.type);
+        string array = string (TypeField (si->content.type)) + "*";
         string elements = si->content.name;
         
         printf ("struct _%s {\n", mname.c_str());
@@ -709,7 +743,6 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
         printf ("  return hack_cast (sh, sh.steal());\n");
         printf ("}\n");
         
-        string elementFromValue = createTypeCode (si->content.type, "element", MODEL_FROM_VALUE);
         printf ("%s\n", ret.c_str());
         printf ("%s_from_seq (SfiSeq *seq)\n", lname.c_str());
         printf ("{\n");
@@ -717,7 +750,6 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
         printf ("  return hack_cast (sh, sh.steal());\n");
         printf ("}\n");
         
-        string elementToValue = createTypeCode (si->content.type, "seq->" + elements + "[i]", MODEL_TO_VALUE);
         printf ("SfiSeq*\n");
         printf ("%s_to_seq (%s cseq)\n", lname.c_str(), arg.c_str());
         printf ("{\n");
@@ -772,7 +804,7 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
         printf ("%s_seq2boxed (const GValue *src_value, GValue *dest_value)\n", name.c_str());
         printf ("{\n");
         printf ("  SfiSeq *seq = sfi_value_get_seq (src_value);\n");
-        printf ("  g_value_set_boxed_take_ownership (dest_value,\n");
+        printf ("  g_value_take_boxed (dest_value,\n");
         printf ("    seq ? %s_from_seq (seq) : NULL);\n", name.c_str());
         printf ("}\n");
         
@@ -784,7 +816,7 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
         printf ("  %s_seq2boxed,\n", name.c_str());
         printf ("  %s_info_strings\n", name.c_str());
         printf ("};\n");
-        printf ("GType %s = 0;\n", makeGTypeName (si->name).c_str());
+        printf ("GType %s = 0;\n", make_TYPE_MACRO (si->name));
       }
   }
   void
@@ -824,7 +856,7 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
               {
                 if (generateIdlLineNumbers)
                   printf ("#line %u \"%s\"\n", pi->line, parser.fileName().c_str());
-                printf ("  %s_field[%d] = %s;\n", name.c_str(), f, makeParamSpec (*pi).c_str());
+                printf ("  %s_field[%d] = %s;\n", name.c_str(), f, construct_pspec (*pi).c_str());
               }
           }
         if (parser.isSequence (*ti))
@@ -835,14 +867,14 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
             
             if (generateIdlLineNumbers)
               printf ("#line %u \"%s\"\n", sdef.content.line, parser.fileName().c_str());
-            printf ("  %s_content = %s;\n", name.c_str(), makeParamSpec (sdef.content).c_str());
+            printf ("  %s_content = %s;\n", name.c_str(), construct_pspec (sdef.content).c_str());
           }
       }
     for (vector<Choice>::const_iterator ei = parser.getChoices().begin(); ei != parser.getChoices().end(); ei++)
       {
         if (parser.fromInclude (ei->name)) continue;
         
-        string gname = makeGTypeName(ei->name);
+        string gname = make_TYPE_MACRO (ei->name);
         string name = makeLowerName(ei->name);
         string mname = makeMixedName(ei->name);
         
@@ -857,7 +889,7 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
       {
         if (parser.fromInclude (ri->name)) continue;
         
-        string gname = makeGTypeName(ri->name);
+        string gname = make_TYPE_MACRO (ri->name);
         string name = makeLowerName(ri->name);
         
         printf ("  %s = sfi_boxed_make_record (&%s_boxed_info,\n", gname.c_str(), name.c_str());
@@ -868,7 +900,7 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
       {
         if (parser.fromInclude (si->name)) continue;
         
-        string gname = makeGTypeName(si->name);
+        string gname = make_TYPE_MACRO (si->name);
         string name = makeLowerName(si->name);
         
         printf ("  %s_boxed_info.element = %s_content;\n", name.c_str(), name.c_str());
@@ -881,7 +913,7 @@ class CodeGeneratorCoreC : public CodeGeneratorCBase {
   
 public:
   CodeGeneratorCoreC (const Parser& parser) :
-    CodeGeneratorCBase (parser)
+    CodeGenerator (parser)
   {
   }
 
