@@ -91,8 +91,7 @@ glue_untyped_pspec_constructor (const Parser &parser,
     {
     case OBJECT:
       {
-        /* FIXME(stw): the ParamSpec doesn't transport the type of the objects we require */
-        string pspec = "sfidl_pspec_Proxy";
+        string pspec = "sfidl_pspec_Object";
         if (param.args == "")
           pspec += "_default (" + group + ",\"" + param.name + "\")";
         else
@@ -350,7 +349,7 @@ public:
         }
       case OBJECT:
         {
-          string pspec = "sfidl_pspec_Object";
+          string pspec = "sfidl_pspec_TypedObject";
           if (param.args == "")
             pspec += "_default";
           pspec += " (" + group + ", \"" + param.name + "\", ";
@@ -555,9 +554,9 @@ public:
       }
   }
   void
-  generate_enum_implementations (NamespaceHelper& nspace)
+  generate_enum_declarations (NamespaceHelper& nspace)
   {
-    printf ("\n\n/* enum implementations */\n");
+    printf ("\n\n/* enum declarations */\n");
     for (vector<Choice>::const_iterator ci = parser.getChoices().begin(); ci != parser.getChoices().end(); ci++)
       {
         if (parser.fromInclude (ci->name))
@@ -859,6 +858,15 @@ public:
         push_type ("EFFECT", pure_TypeName (ci->name));
       }
   }
+  bool
+  class_has_automation_properties (const Class &klass)
+  {
+    /* figure whether automation properties are required */
+    for (vector<Param>::const_iterator pi = klass.properties.begin(); pi != klass.properties.end(); pi++)
+      if (g_option_check (pi->literal_options.c_str(), "automate"))
+        return true;
+    return false;
+  }
   void
   generate_class_definitions (NamespaceHelper& nspace)
   {
@@ -910,11 +918,9 @@ public:
         printf ("  static inline const char* type_name () { return \"%s\"; }\n", make_PrefixedTypeName (ci->name));
         
         /* i/j/o channel names */
-        int is_public = 0;
         if (ci->istreams.size())
           {
-            if (!is_public++)
-              printf ("public:\n");
+            printf ("public:\n");
             printf ("  enum {\n");
             for (vector<Stream>::const_iterator si = ci->istreams.begin(); si != ci->istreams.end(); si++)
               printf ("    ICHANNEL_%s,\n", pure_UPPER (si->ident));
@@ -922,8 +928,7 @@ public:
           }
         if (ci->jstreams.size())
           {
-            if (!is_public++)
-              printf ("public:\n");
+            printf ("public:\n");
             printf ("  enum {\n");
             for (vector<Stream>::const_iterator si = ci->jstreams.begin(); si != ci->jstreams.end(); si++)
               printf ("    JCHANNEL_%s,\n", pure_UPPER (si->ident));
@@ -931,21 +936,31 @@ public:
           }
         if (ci->ostreams.size())
           {
-            if (!is_public++)
-              printf ("public:\n");
+            printf ("public:\n");
             printf ("  enum {\n");
             for (vector<Stream>::const_iterator si = ci->ostreams.begin(); si != ci->ostreams.end(); si++)
               printf ("    OCHANNEL_%s,\n", pure_UPPER (si->ident));
             printf ("    N_OCHANNELS\n  };\n");
           }
         
-        /* "Properties" structure for synthesis modules */
-        if (ci->properties.size() && ci->istreams.size() + ci->jstreams.size() + ci->ostreams.size())
+        /* property IDs */
+        printf ("protected:\n  enum %s {\n", ctPropertyID);
+        if (ci->properties.begin() != ci->properties.end())
           {
-            if (!is_public++)
-              printf ("public:\n");
+            vector<Param>::const_iterator pi = ci->properties.begin();
+            printf ("    PROP_%s = 1,\n", pure_UPPER (pi->name));
+            for (pi++; pi != ci->properties.end(); pi++)
+              printf ("    PROP_%s,\n", pure_UPPER (pi->name));
+          }
+        printf ("  };\n");
+        
+        /* "Properties" structure for synthesis modules */
+        if (ci->istreams.size() + ci->jstreams.size() + ci->ostreams.size())
+          {
+            printf ("public:\n");
             printf ("  /* \"transport\" structure to configure synthesis modules from properties */\n");
             printf ("  struct %s {\n", ctProperties);
+            printf ("    typedef %s IDType;\n", ctPropertyID);
             for (vector<Param>::const_iterator pi = ci->properties.begin(); pi != ci->properties.end(); pi++)
               printf ("    %s %s;\n", TypeField (pi->type), pi->name.c_str());
             printf ("    explicit %s (%s *p) ", ctProperties, ctNameBase);
@@ -956,25 +971,35 @@ public:
             printf ("  };\n");
           }
         
+        /* auto-update type */
+        printf ("protected:\n");
+        printf ("  typedef %s AutoUpdateCategory;\n", class_has_automation_properties (*ci) ? "::Bse::SynthesisModule::NeedAutoUpdateTag" : "void");
+
         /* property fields */
         printf ("protected:\n");
         for (vector<Param>::const_iterator pi = ci->properties.begin(); pi != ci->properties.end(); pi++)
-          printf ("  %s %s;\n", TypeField (pi->type), pi->name.c_str());
-        
-        /* property IDs */
-        if (ci->properties.begin() != ci->properties.end())
           {
-            printf ("protected:\n  enum %s {\n", ctPropertyID);
-            vector<Param>::const_iterator pi = ci->properties.begin();
-            printf ("    PROP_%s = 1,\n", pure_UPPER (pi->name));
-            for (pi++; pi != ci->properties.end(); pi++)
-              printf ("    PROP_%s,\n", pure_UPPER (pi->name));
-            printf ("  };\n");
+            printf ("  %s %s;\n", TypeField (pi->type), pi->name.c_str());
+            if (g_option_check (pi->literal_options.c_str(), "automate"))
+              printf ("  guint64 last__%s;\n", pi->name.c_str());
           }
         
-        /* property setter */
+        /* get_property() */
         printf ("public:\n");
-        printf ("  void set_property (guint prop_id, const ::Bse::Value &value, GParamSpec *pspec)\n");
+        printf ("  void get_property (%s prop_id, ::Bse::Value &value, GParamSpec *pspec)\n", ctPropertyID);
+        printf ("  {\n");
+        printf ("    switch (prop_id) {\n");
+        for (vector<Param>::const_iterator pi = ci->properties.begin(); pi != ci->properties.end(); pi++)
+          {
+            printf ("    case PROP_%s:\n", pure_UPPER (pi->name));
+            printf ("      %s (&value, %s);\n", func_value_set_param (pi->type), pi->name.c_str());
+            printf ("    break;\n");
+          }
+        printf ("    };\n");
+        printf ("  }\n");
+        
+        /* set_property() */
+        printf ("  void set_property (%s prop_id, const ::Bse::Value &value, GParamSpec *pspec)\n", ctPropertyID);
         printf ("  {\n");
         printf ("    switch (prop_id) {\n");
         for (vector<Param>::const_iterator pi = ci->properties.begin(); pi != ci->properties.end(); pi++)
@@ -999,20 +1024,36 @@ public:
         printf ("    default: ;\n");
         printf ("    };\n");
         printf ("  }\n");
-        
-        /* property getter */
-        printf ("  void get_property (guint prop_id, ::Bse::Value &value, GParamSpec *pspec)\n");
+
+        /* editable_property() */
+        printf ("  virtual bool editable_property (%s prop_id, GParamSpec *pspec)\n", ctPropertyID);
+        printf ("  {\n");
+        printf ("    return true;\n");
+        printf ("  }\n");
+
+        /* get_candidates() */
+        printf ("  virtual void get_candidates (%s prop_id, ::Bse::PropertyCandidatesHandle &pch, GParamSpec *pspec)\n", ctPropertyID);
+        printf ("  {\n");
+        printf ("  }\n");
+
+        /* property_updated() */
+        printf ("  void property_updated (%s prop_id, guint64 tick_stamp, double prop_value, GParamSpec *pspec)\n", ctPropertyID);
         printf ("  {\n");
         printf ("    switch (prop_id) {\n");
         for (vector<Param>::const_iterator pi = ci->properties.begin(); pi != ci->properties.end(); pi++)
           {
+            if (!g_option_check (pi->literal_options.c_str(), "automate"))
+              continue;
             printf ("    case PROP_%s:\n", pure_UPPER (pi->name));
-            printf ("      %s (&value, %s);\n", func_value_set_param (pi->type), pi->name.c_str());
+            printf ("      if (tick_stamp >= ::std::max (last__%s, module_update_tick_stamp()))\n", pi->name.c_str());
+            printf ("        %s = prop_value;\n", pi->name.c_str());
+            printf ("      last__%s = tick_stamp;\n", pi->name.c_str());
             printf ("    break;\n");
           }
+        printf ("    default: ;\n");
         printf ("    };\n");
         printf ("  }\n");
-        
+
         /* static data */
         printf ("private:\n");
         printf ("  static struct StaticData {\n");
@@ -1082,6 +1123,9 @@ public:
         if (parser.fromInclude (ci->name))
           continue;
         nspace.setFromSymbol(ci->name);
+        const char *ctName = pure_TypeName (ci->name);
+        const char *ctNameBase = intern (ctName + string ("Base"));
+        const char *ctPropertyID = intern (ctName + string ("PropertyID"));
         const char *nname = nspace.printable_form (ci->name);
         vector<string> destroy_jobs;
         
@@ -1089,6 +1133,14 @@ public:
         printf ("void\n");
         printf ("%sBase::class_init (::Bse::CxxBaseClass *klass)\n", nname);
         printf ("{\n");
+        printf ("  klass->set_accessors (::Bse::cxx_get_property_trampoline<%s, %s>,\n", ctNameBase, ctPropertyID);
+        printf ("                        ::Bse::cxx_set_property_trampoline<%s, %s>,\n", ctNameBase, ctPropertyID);
+        printf ("                        ::Bse::cxx_editable_property_trampoline<%s, %s>,\n", ctNameBase, ctPropertyID);
+        printf ("                        ::Bse::cxx_get_candidates_trampoline<%s, %s>,\n", ctNameBase, ctPropertyID);
+        if (class_has_automation_properties (*ci))
+          printf ("                        ::Bse::cxx_property_updated_trampoline<%s, %s>);\n", ctNameBase, ctPropertyID);
+        else
+          printf ("                        NULL);\n");
         for (vector<Param>::const_iterator pi = ci->properties.begin(); pi != ci->properties.end(); pi++)
           printf ("  klass->add_param (PROP_%s, %s);\n", pure_UPPER (pi->name), typed_pspec_constructor (*pi).c_str());
         for (vector<Stream>::const_iterator si = ci->istreams.begin(); si != ci->istreams.end(); si++)
@@ -1260,9 +1312,9 @@ public:
     generate_enum_definitions (nsh);
     generate_sequence_definitions (nsh);
     generate_record_definitions (nsh);
-    generate_class_definitions (nsh);           /* adds to images, procs */
 
     /* (type) declarations */
+    generate_enum_declarations (nsh);
     generate_sequence_declarations (nsh);
     generate_record_declarations (nsh);
 
@@ -1271,8 +1323,10 @@ public:
       procs.push_back (&(*mi));                 /* collect procedures */
     generate_procedure_prototypes (nsh);
 
+    /* class bodies */
+    generate_class_definitions (nsh);           /* adds to images, procs */
+
     /* implementations */
-    generate_enum_implementations (nsh);
     generate_choice_implementations (nsh);
     generate_record_implementations (nsh);
     generate_sequence_implementations (nsh);
