@@ -90,6 +90,7 @@ enum ExtraToken {
   TOKEN_GROUP,
   TOKEN_USING,
   TOKEN_CONST,
+  TOKEN_CONST_IDENT,
   TOKEN_INFO,
   TOKEN_ISTREAM,
   TOKEN_JSTREAM,
@@ -100,7 +101,7 @@ enum ExtraToken {
 const char *token_symbols[] = {
   "namespace", "class", "choice", "record", "sequence",
   "property", "group", "using",
-  "Const", "Info", "IStream", "JStream", "OStream",
+  "Const", "ConstIdent", "Info", "IStream", "JStream", "OStream",
   0
 };
 
@@ -153,7 +154,7 @@ bool operator== (GTokenType t, ExtraToken e) { return (int) t == (int) e; }
     g_scanner_get_next_token(scanner); \
   if (g_scanner_get_next_token (scanner) != G_TOKEN_INT) \
     return G_TOKEN_INT; \
-  i = scanner->value.v_int; \
+  i = scanner->value.v_int64; \
   if (negate) i = -i; \
 }G_STMT_END
 #define parse_float_or_return(f) G_STMT_START{ \
@@ -165,7 +166,7 @@ bool operator== (GTokenType t, ExtraToken e) { return (int) t == (int) e; }
     t = g_scanner_get_next_token (scanner); \
   } \
   if (t == G_TOKEN_INT) \
-    f = scanner->value.v_int; \
+    f = scanner->value.v_int64; \
   else if (t == G_TOKEN_FLOAT) \
     f = scanner->value.v_float; \
   else \
@@ -252,7 +253,7 @@ Parser::findClass (const string& name) const
 
 Parser::Parser () : options (*Options::the())
 {
-  scanner = g_scanner_new (&scanner_config_template);
+  scanner = g_scanner_new64 (&scanner_config_template);
   
   for (int n = 0; token_symbols[n]; n++)
     g_scanner_add_symbol (scanner, token_symbols[n], GUINT_TO_POINTER (G_TOKEN_LAST + 1 + n));
@@ -263,7 +264,7 @@ Parser::Parser () : options (*Options::the())
   scanner->user_data = this;
 }
 
-void Parser::printError(const gchar *format, ...)
+void Parser::printError (const gchar *format, ...)
 {
   va_list args;
   gchar *string;
@@ -278,6 +279,19 @@ void Parser::printError(const gchar *format, ...)
   g_free (string);
 }
 
+void Parser::printWarning (const gchar *format, ...)
+{
+  va_list args;
+  gchar *string;
+  
+  va_start (args, format);
+  string = g_strdup_vprintf (format, args);
+  va_end (args);
+  
+  g_scanner_warn (scanner, "%s", string);
+  g_free (string);
+}
+
 void Parser::scannerMsgHandler (GScanner *scanner, gchar *message, gboolean is_error)
 {
   g_return_if_fail (scanner != NULL);
@@ -287,7 +301,7 @@ void Parser::scannerMsgHandler (GScanner *scanner, gchar *message, gboolean is_e
   if (scanner->line > 0 && parser->scannerLineInfo.size() >= scanner->line)
     {
       const LineInfo& info = parser->scannerLineInfo[scanner->line-1];
-      fprintf (stderr, "%s:%d: ", info.filename.c_str(), info.line);
+      fprintf (stderr, "%s: ", info.location().c_str());
     }
   else
     {
@@ -295,6 +309,8 @@ void Parser::scannerMsgHandler (GScanner *scanner, gchar *message, gboolean is_e
     }
   if (is_error)
     fprintf (stderr, "error: ");
+  else
+    fprintf (stderr, "warning: ");
   fprintf (stderr, "%s\n", message);
 }
 
@@ -320,26 +336,33 @@ static bool fileExists(const string& filename)
   return false;
 }
 
-static void loadFile(const char *filename, vector<char>& v)
+static void loadFile (FILE *file, vector<char>& v)
 {
-  FILE *f = 0;
-
-  if (strcmp (filename, "-") == 0) /* stdin */
-    f = stdin;
-  else
-    f = fopen (filename,"r");
-
-  if(!f)
-    {
-      fprintf(stderr,"file '%s' not found\n",filename);
-      exit(1);
-    }
-
   char buffer[1024];
   long l;
-  while (!feof (f) && (l = fread(buffer,1,1024,f)) > 0)
+  while (!feof (file) && (l = fread(buffer,1,1024,file)) > 0)
     v.insert(v.end(),buffer, buffer+l);
-  fclose(f);
+}
+
+static void loadFile(const char *filename, vector<char>& v)
+{
+  if (strcmp (filename, "-") == 0) /* stdin */
+    {
+      loadFile (stdin, v);
+    }
+  else
+    {
+      FILE *file = fopen (filename,"r");
+
+      if (!file)
+	{
+	  fprintf(stderr,"file '%s' not found\n",filename);
+	  exit(1);
+	}
+
+      loadFile (file, v);
+      fclose (file);
+    }
 }
 
 bool Parser::haveIncluded (const string& filename) const
@@ -411,7 +434,7 @@ void Parser::preprocessContents (const string& input_filename)
 
   LineInfo linfo;
   linfo.line = 1;
-  linfo.filename = (input_filename == "-") ? "stdin" : input_filename;
+  linfo.filename = input_filename;
 
   vector<char> input;
   loadFile (input_filename.c_str(), input);
@@ -523,7 +546,8 @@ void Parser::preprocessContents (const string& input_filename)
 	    }
 	  if (location == "")
 	    {
-	      fprintf(stderr,"include file '%s' not found\n", filename.c_str());
+	      fprintf (stderr, "%s: ", linfo.location().c_str());
+	      fprintf (stderr, "include file '%s' not found\n", filename.c_str());
 	      exit(1);
 	    }
 	  preprocess (location, includeImpl);
@@ -762,6 +786,13 @@ GTokenType Parser::parseNamespace()
 		return expected_token;
 	    }
 	    break;
+	  case TOKEN_CONST_IDENT:
+	    {
+	      GTokenType expected_token = parseConstant (true);
+	      if (expected_token != G_TOKEN_NONE)
+		return expected_token;
+	    }
+	    break;
 	  case TOKEN_USING:
 	    {
 	      parse_or_return (TOKEN_USING);
@@ -833,13 +864,14 @@ GTokenType Parser::parseStringOrConst (string &s)
 
       for(vector<Constant>::iterator ci = constants.begin(); ci != constants.end(); ci++)
 	{
-	  if (ci->name == s)
+	  if (ci->name == s &&
+              ci->type != Constant::tIdent)     /* identifier constants can't be expanded to strings */
 	    {
 	      char *x = 0;
 	      switch (ci->type)
 		{
 		  case Constant::tInt:
-		    s = x = g_strdup_printf ("%d", ci->i);
+		    s = x = g_strdup_printf ("%lldLL", ci->i);
 		    g_free (x);
 		    break;
 		  case Constant::tFloat:
@@ -864,35 +896,51 @@ GTokenType Parser::parseStringOrConst (string &s)
   return G_TOKEN_NONE;
 }
 
-GTokenType Parser::parseConstant ()
+GTokenType Parser::parseConstant (bool isident)
 {
   /*
    * constant BAR = 3;
    */
   Constant cdef;
 
-  parse_or_return (TOKEN_CONST);
+  if (isident)
+    parse_or_return (TOKEN_CONST_IDENT);
+  else
+    parse_or_return (TOKEN_CONST);
   parse_or_return (G_TOKEN_IDENTIFIER);
   cdef.name = defineSymbol (scanner->value.v_identifier);
   cdef.file = fileName();
   
   parse_or_return ('=');
 
+  /* handle ConstIdent */
+  if (isident)
+    {
+      parse_or_return (G_TOKEN_IDENTIFIER);
+      cdef.str = scanner->value.v_identifier;
+      cdef.type = Constant::tIdent;
+      parse_or_return (';');
+      addConstantTodo (cdef);
+      return G_TOKEN_NONE;
+    }
+
   GTokenType t = g_scanner_peek_next_token (scanner);
 
-  bool negate = (t == GTokenType('-')); /* negative number? */
-  if (negate)
-  {
-    parse_or_return ('-');
-    t = g_scanner_peek_next_token (scanner);
-  }
+  /* allow positive/negative prefixing */
+  bool negate = FALSE;
+  while (t == '+' || t == '-')
+    {
+      t = g_scanner_get_next_token (scanner);
+      negate = negate ^ (t == '-');
+      t = g_scanner_peek_next_token (scanner);
+    }
 
   if (t == G_TOKEN_INT)
   {
     parse_or_return (G_TOKEN_INT);
 
     cdef.type = Constant::tInt;
-    cdef.i = scanner->value.v_int;
+    cdef.i = scanner->value.v_int64;
     if (negate)
       cdef.i = -cdef.i;
   }
@@ -921,7 +969,8 @@ GTokenType Parser::parseConstant ()
   return G_TOKEN_NONE;
 }
 
-GTokenType Parser::parseChoice ()
+GTokenType
+Parser::parseChoice ()
 {
   Choice choice;
   int value = 0, sequentialValue = 1;
@@ -962,76 +1011,109 @@ skip_ascii_at (GScanner *scanner)
     g_scanner_get_next_token (scanner);
 }
 
-GTokenType Parser::parseChoiceValue (ChoiceValue& comp, int& value, int& sequentialValue)
+#define to_lower(c)                             ( \
+        (guchar) (                                                      \
+          ( (((guchar)(c))>='A' && ((guchar)(c))<='Z') * ('a'-'A') ) |  \
+          ( (((guchar)(c))>=192 && ((guchar)(c))<=214) * (224-192) ) |  \
+          ( (((guchar)(c))>=216 && ((guchar)(c))<=222) * (248-216) ) |  \
+          ((guchar)(c))                                                 \
+        )                                                               \
+)
+
+
+GTokenType
+Parser::parseChoiceValue (ChoiceValue& comp, int& value, int& sequentialValue)
 {
-  /*
-    YES,
-    YES = 1,
-    YES = "Yes",
-    YES = (1, "Yes"),
-    YES = Neutral,
-    YES = (Neutral, "Yes"),
-  */
-  
   parse_or_return (G_TOKEN_IDENTIFIER);
   comp.name = defineSymbol(scanner->value.v_identifier);
   comp.file = fileName();
   comp.neutral = false;
+  string str;
+  comp.name.c_str(); // don't remove this
+  for (guint i = 0; i < comp.name.size(); i++)
+    if (comp.name[i] != ':' || comp.name[i + 1] != ':')
+      str += comp.name[i] == ':' ? '_' : to_lower(comp.name[i]);
+  comp.label = g_type_name_to_sname (str.c_str());
   
-  /* the hints are optional */
+  /*
+    YES,
+    YES = 1,
+    YES = "Yes",
+    YES = Neutral,
+    YES = (1),
+    YES = (1, "Yes"),
+    YES = (Neutral, "Yes"),
+    YES = (1, "Yes", "this is the Yes value"),
+    YES = ("Yes", "this is the Yes value"),
+  */
   if (g_scanner_peek_next_token (scanner) == GTokenType('='))
     {
-      int args = 1;
       parse_or_return ('=');
-      
       if (g_scanner_peek_next_token (scanner) == GTokenType('('))
-	{
-	  parse_or_return ('(');
-	  args = 2;
-	}
-      
-      switch (g_scanner_peek_next_token (scanner))
-	{
-        case G_TOKEN_IDENTIFIER:
-          {
-            parse_or_return (G_TOKEN_IDENTIFIER);
-            if (strcmp (scanner->value.v_string, "Neutral") == 0)
-              {
-                comp.neutral = true;
-              }
-            else
-              {
-                printError("expected 'Neutral' or nothing as choice value type");
-                return GTokenType('(');
-              }
-          }
-          break;
-        case G_TOKEN_INT:
-          {
-            parse_or_return (G_TOKEN_INT);
-            value = scanner->value.v_int;
-          }
-          break;
-        case G_TOKEN_STRING:
-          {
-            if (args == 1)
-              parse_string_or_return (comp.text);
-            else
-              return G_TOKEN_INT;
-          }
-          break;
-        default:
-          return G_TOKEN_INT;
-	}
-      
-      if (args == 2)
-	{
-	  parse_or_return (',');
-	  parse_string_or_return (comp.text);
-	  parse_or_return (')');
+        {
+          bool need_arg = true;
+          parse_or_return ('(');
+          if (g_scanner_peek_next_token (scanner) == G_TOKEN_INT)
+            {
+              parse_or_return (G_TOKEN_INT);
+              value = scanner->value.v_int64;
+              if (g_scanner_peek_next_token (scanner) == ',')
+                parse_or_return (',');
+              else
+                need_arg = false;
+            }
+          else if (g_scanner_peek_next_token (scanner) == G_TOKEN_IDENTIFIER &&
+                   strcmp (scanner->next_value.v_string, "Neutral") == 0)
+            {
+              parse_or_return (G_TOKEN_IDENTIFIER);
+              comp.neutral = true;
+              if (g_scanner_peek_next_token (scanner) == ',')
+                parse_or_return (',');
+              else
+                need_arg = false;
+            }
+          if (need_arg)
+            {
+              parse_istring_or_return (comp.label);
+              if (g_scanner_peek_next_token (scanner) == ',')
+                parse_or_return (',');
+              else
+                need_arg = false;
+            }
+          if (need_arg)
+            {
+              parse_istring_or_return (comp.blurb);
+              if (g_scanner_peek_next_token (scanner) == ',')
+                parse_or_return (',');
+              else
+                need_arg = false;
+            }
+          parse_or_return (')');
 	}
+      else if (g_scanner_peek_next_token (scanner) == G_TOKEN_INT)
+        {
+          parse_or_return (G_TOKEN_INT);
+          value = scanner->value.v_int64;
+        }
+      else if (g_scanner_peek_next_token (scanner) == G_TOKEN_IDENTIFIER &&
+               strcmp (scanner->next_value.v_string, "Neutral") == 0)
+        {
+          parse_or_return (G_TOKEN_IDENTIFIER);
+          comp.neutral = true;
+        }
+      else if (g_scanner_peek_next_token (scanner) == G_TOKEN_IDENTIFIER &&
+               strcmp (scanner->next_value.v_string, "_") == 0)
+        {
+          parse_istring_or_return (comp.label);
+        }
+      else if (g_scanner_peek_next_token (scanner) == G_TOKEN_STRING)
+        {
+          parse_istring_or_return (comp.label);
+        }
+      else
+        return G_TOKEN_INT;
     }
-  
+
   comp.value = value++;
   if (comp.neutral)
     comp.sequentialValue = 0;
@@ -1167,7 +1249,7 @@ Parser::parseStream (Stream&      stream,
   parse_or_return ('=');
 
   parse_or_return ('(');
-  parse_istring_or_return (stream.name);
+  parse_istring_or_return (stream.label);
   parse_or_return (',');
   parse_istring_or_return (stream.blurb);
   parse_or_return (')');
@@ -1176,6 +1258,121 @@ Parser::parseStream (Stream&      stream,
   return G_TOKEN_NONE;
 }
 
+/* like g_scanner_get_token, but accepts ':' within identifiers */
+GTokenType scanner_get_next_token_with_colon_identifiers (GScanner *scanner)
+{
+  char *cset_identifier_first_orig = scanner->config->cset_identifier_first;
+  char *cset_identifier_nth_orig   = scanner->config->cset_identifier_nth;
+
+  string identifier_first_with_colon = cset_identifier_first_orig + string (":");
+  string identifier_nth_with_colon = cset_identifier_nth_orig + string (":");
+
+  scanner->config->cset_identifier_first = const_cast<char *>(identifier_first_with_colon.c_str());
+  scanner->config->cset_identifier_nth   = const_cast<char *>(identifier_nth_with_colon.c_str());
+
+  GTokenType token = g_scanner_get_next_token (scanner);
+
+  scanner->config->cset_identifier_first = cset_identifier_first_orig;
+  scanner->config->cset_identifier_nth   = cset_identifier_nth_orig;
+
+  return token;
+}
+
+/* returns true for C++ style identifiers (Foo::BAR) - only the colons are checked, not individual chars */
+bool isCxxTypeName (const string& str)
+{
+  enum { valid, colon1, colon2, invalid } state = valid;
+
+  for (string::const_iterator i = str.begin(); i != str.end(); i++)
+    {
+      if (state == valid)
+	{
+	  if (*i == ':')
+	    state = colon1;
+	}
+      else if (state == colon1)
+	{
+	  if (*i == ':')
+	    state = colon2;
+	  else
+	    state = invalid;
+	}
+      else if (state == colon2)
+	{
+	  if (*i == ':')
+	    state = invalid;
+	  else
+	    state = valid;
+	}
+    }
+  return (state == valid) && (str.size() != 0);
+}
+
+static bool
+makeLiteralOptions (const string& options, string& literal_options)
+{
+  bool failed = false;
+  bool in_string = false;
+
+  /*
+   * this does handle
+   *
+   *    \":r\"	      => :r
+   *	\":r\" \":w\" => :r:w
+   *	(\":r\"
+   *	 \":w\")      => :r:w
+   *
+   * this does not handle
+   *
+   *    \":r\" _(\":i18n-hint\") \":w\"
+   *	\":r\" C_DEFINED_IDENTIFIER \":w\"
+   *	g_strdup (\":r:w\")
+   *	\":r\" /+ (imagine this would be a C-style-comment with real asterisks) +/ \":w\"
+   *
+   * however, the last case should not occur, as options have already been GScanner scanned
+   */
+  for (string::const_iterator oi = options.begin(); oi != options.end(); oi++)
+    {
+      bool last_was_backslash = false;
+      if (*oi == '\\')
+	{
+	  last_was_backslash = true;
+
+	  if ((oi + 1) != options.end())      // unescape stuff
+	    oi++;
+	  else
+	    failed = true;		      // option string ends with a single backslash? huh?
+	}
+      switch (*oi)
+	{
+	case '"':   if (!last_was_backslash)  // keep track of whether we are inside a string or not
+		      in_string = !in_string;
+		    else if (in_string)	      // if we got an \" inside a string, replace it with the real thing
+		      literal_options += *oi;
+		    else		      // we got an \" outside a string? why?
+		      failed = true;
+		    break;
+	case '(':			      // somewhat lazy way of handling string concatenation:
+	case ')':			      // will match (C compiler will check this anyways)
+	case ' ':			      // so we just ignore whitespace and paranthesis outside strings
+	case '\t':
+	case '\n':  if (in_string)
+		      literal_options += *oi; // inside strings, treat them as real
+		    break;
+	default:    if (in_string)	      // inside strings, just collect all characters that we see
+		      literal_options += *oi;
+		    else
+		      failed = true;	      // outside strings, we shouldn't see things
+		    break;
+	}
+    }
+
+  if (failed)
+    literal_options = ""; // what we've got so far might be unusable
+
+  return !failed;
+}
+
 GTokenType Parser::parseParamHints (Param &def)
 {
   if (g_scanner_peek_next_token (scanner) == G_TOKEN_IDENTIFIER)
@@ -1183,15 +1380,18 @@ GTokenType Parser::parseParamHints (Param &def)
       parse_or_return (G_TOKEN_IDENTIFIER);
       def.pspec = scanner->value.v_identifier;
     }
-  
+
   parse_or_return ('(');
 
   int bracelevel = 1;
   string args;
+  string current_arg;
+  int arg_count = 0;
   while (!g_scanner_eof (scanner) && bracelevel > 0)
     {
-      GTokenType t = g_scanner_get_next_token (scanner);
+      GTokenType t = scanner_get_next_token_with_colon_identifiers (scanner);
       gchar *token_as_string = 0, *x = 0;
+      bool current_arg_complete = false;
 
       if(int(t) > 0 && int(t) <= 255)
 	{
@@ -1203,34 +1403,45 @@ GTokenType Parser::parseParamHints (Param &def)
 	case '(':		  bracelevel++;
 	  break;
 	case ')':		  bracelevel--;
+				  if (bracelevel == 0)
+				    current_arg_complete = true;
+	  break;
+	case ',':		  current_arg_complete = true;
 	  break;
 	case G_TOKEN_STRING:	  x = g_strescape (scanner->value.v_string, 0);
 				  token_as_string = g_strdup_printf ("\"%s\"", x);
 				  g_free (x);
 	  break;
-	case G_TOKEN_INT:	  token_as_string = g_strdup_printf ("%lu", scanner->value.v_int);
+	case G_TOKEN_INT:	  token_as_string = g_strdup_printf ("%llu", scanner->value.v_int64);
 	  break;
 	case G_TOKEN_FLOAT:	  token_as_string = g_strdup_printf ("%.17g", scanner->value.v_float);
 	  break;
 	case G_TOKEN_IDENTIFIER:
           {
-            vector<Constant>::iterator ci;
-            string coname = qualifySymbol (scanner->value.v_identifier);
-            /* FIXME: there should be a generic const_to_string() function */
-            for (ci = constants.begin(); ci != constants.end(); ci++)
-              if (ci->name == coname)
-                break;
-            // else g_printerr("lookup failed: %s (%s)\n", scanner->value.v_identifier, ci->name.c_str());
+	    vector<Constant>::iterator ci = constants.end();
+	    if (isCxxTypeName (scanner->value.v_identifier))
+	      {
+		string coname = qualifySymbol (scanner->value.v_identifier);
+
+		/* FIXME: there should be a generic const_to_string() function */
+		for (ci = constants.begin(); ci != constants.end(); ci++)
+		  if (ci->name == coname)
+		    break;
+	      }
+
             if (ci == constants.end())
               token_as_string = g_strdup_printf ("%s", scanner->value.v_identifier);
             else switch (ci->type)
               {
               case Constant::tInt:
-                token_as_string = g_strdup_printf ("%d", ci->i);
+                token_as_string = g_strdup_printf ("%lldLL", ci->i);
                 break;
               case Constant::tFloat:
                 token_as_string = g_strdup_printf ("%.17g", ci->f);
                 break;
+              case Constant::tIdent:
+                token_as_string = g_strdup (ci->str.c_str());
+                break;
               case Constant::tString:
                 x = g_strescape (ci->str.c_str(), 0);
                 token_as_string = g_strdup_printf ("\"%s\"", x);
@@ -1246,6 +1457,22 @@ GTokenType Parser::parseParamHints (Param &def)
 	  if (!token_as_string)
 	    return GTokenType (')');
 	}
+      if (current_arg_complete)
+	{
+	  switch (arg_count++)
+	    {
+	    case 0:   def.label = current_arg;	 /* first pspec constructor argument */
+		      break;
+	    case 1:   def.blurb = current_arg;	 /* second pspec constructor argument */
+		      break;
+	    default:  def.options = current_arg; /* last pspec constructor argument */
+	    }
+	  current_arg = "";
+	}
+      else if (token_as_string)
+	{
+	  current_arg += token_as_string;
+	}
       if (token_as_string && bracelevel)
 	{
 	  if (args != "")
@@ -1255,6 +1482,8 @@ GTokenType Parser::parseParamHints (Param &def)
 	}
     }
   def.args = args;
+  if (!makeLiteralOptions (def.options, def.literal_options))
+    printWarning ("can't parse option string: %s", def.options.c_str());
   return G_TOKEN_NONE;
 }
 
@@ -1276,6 +1505,8 @@ GTokenType Parser::parseInfoOptional (Map<string,IString>& infos)
     parse_istring_or_return (value);
     parse_or_return (';');
 
+    for (guint i = 0; i < key.size(); i++)
+      key[i] = to_lower (key[i]);
     infos[key] = value;
   }
   return G_TOKEN_NONE;
@@ -1666,7 +1897,7 @@ void Parser::addType (const std::string& type, TypeDeclaration typeDecl)
     }
   else if (m == typeDecl)
     {
-      printError ("double definition of '%s' as same type\n", type.c_str());
+      printError ("double definition of '%s' as same type", type.c_str());
     }
   else if (m == (typeDecl | tdProto))
     {
@@ -1674,7 +1905,7 @@ void Parser::addType (const std::string& type, TypeDeclaration typeDecl)
     }
   else
     {
-      printError ("double definition of '%s' as different types\n", type.c_str());
+      printError ("double definition of '%s' as different types", type.c_str());
     }
 }
 
@@ -1712,21 +1943,22 @@ string Parser::defineSymbol (const string& name)
   return sym->fullName();
 }
 
-static list<string> symbolToList(string symbol)
+list<string> symbolToList (const string& symbol)
 {
   list<string> result;
   string current;
+
+  g_return_val_if_fail (isCxxTypeName (symbol), result);
   
-  string::iterator si;
-  for(si = symbol.begin(); si != symbol.end(); si++)
+  for (string::const_iterator si = symbol.begin(); si != symbol.end(); si++)
     {
-      if(*si != ':')
+      if (*si != ':')
 	{
 	  current += *si;
 	}
       else
 	{
-	  if(current != "")
+	  if (current != "")
 	    result.push_back(current);
 	  
 	  current = "";
