@@ -92,7 +92,7 @@ struct LineInfo {
 struct Constant {
   std::string name;
   std::string file;
-  enum { tString = 1, tFloat = 2, tInt = 3 } type;
+  enum { tString = 1, tFloat = 2, tInt = 3, tIdent = 4 } type;
 
   std::string str;
   float f;
@@ -108,6 +108,12 @@ struct Param {
   std::string pspec;
   int         line;
   std::string args;
+
+  std::string label; /* first argument of the param spec contructor */
+  std::string blurb; /* second argument of the param spec contructor */
+  std::string options; /* last argument of the param spec contructor */
+  std::string literal_options; /* the real option string; note that conversion might not work,
+				  if building the literal option string requires things like C function calls */
 };
 
 struct Stream {
@@ -268,6 +274,7 @@ protected:
 
   static void scannerMsgHandler (GScanner *scanner, gchar *message, gboolean is_error);
   void printError (const gchar *format, ...);
+  void printWarning (const gchar *format, ...);
 
   // preprocessor
 
@@ -290,7 +297,7 @@ protected:
 
   GTokenType parseTypeName (std::string& s);
   GTokenType parseStringOrConst (std::string &s);
-  GTokenType parseConstant ();
+  GTokenType parseConstant (bool isident = false);
   GTokenType parseNamespace ();
   GTokenType parseChoice ();
   GTokenType parseChoiceValue (ChoiceValue& comp, int& value, int& sequentialValue);
