@@ -149,10 +149,10 @@ SfiFBlock*
 sfi_fblock_new (void)
 {
   SfiFBlock *fblock = sfi_new_struct (SfiFBlock, 1);
-
   fblock->ref_count = 1;
   fblock->n_values = 0;
   fblock->values = NULL;
+  fblock->freefunc = g_free;
   return fblock;
 }
 
@@ -164,6 +164,19 @@ sfi_fblock_new_sized (guint size)
   return fblock;
 }
 
+SfiFBlock*
+sfi_fblock_new_foreign (guint     n_values,
+                        gfloat   *values,
+                        GFreeFunc freefunc)
+{
+  g_return_val_if_fail (n_values == 0 || values != NULL, NULL);
+  SfiFBlock *fblock = sfi_fblock_new ();
+  fblock->n_values = n_values;
+  fblock->values = values;
+  fblock->freefunc = freefunc;
+  return fblock;
+}
+
 SfiFBlock*
 sfi_fblock_ref (SfiFBlock *fblock)
 {
@@ -183,24 +196,39 @@ sfi_fblock_unref (SfiFBlock *fblock)
   fblock->ref_count--;
   if (fblock->ref_count == 0)
     {
-      g_free (fblock->values);
+      fblock->freefunc (fblock->values);
       sfi_delete_struct (SfiFBlock, fblock);
     }
 }
 
+static void
+fblock_resize (SfiFBlock *fblock,
+               guint      size)
+{
+  guint oldsize = fblock->n_values;
+  fblock->n_values = size;
+  if (fblock->freefunc == g_free)
+    fblock->values = g_renew (gfloat, fblock->values, fblock->n_values);
+  else
+    {
+      gfloat *values = g_new (gfloat, fblock->n_values);
+      memcpy (values, fblock->values, MIN (fblock->n_values, oldsize) * sizeof (values[0]));
+      fblock->freefunc (fblock->values);
+      fblock->values = values;
+      fblock->freefunc = g_free;
+    }
+}
+
 void
 sfi_fblock_resize (SfiFBlock *fblock,
 		   guint      size)
 {
-  guint i;
-
   g_return_if_fail (fblock != NULL);
 
-  i = fblock->n_values;
-  fblock->n_values = size;
-  fblock->values = g_renew (gfloat, fblock->values, fblock->n_values);
-  if (size > i)
-    memset (fblock->values + i, 0, sizeof (fblock->values[0]) * (size - i));
+  guint osize = fblock->n_values;
+  fblock_resize (fblock, size);
+  if (size > osize)
+    memset (fblock->values + osize, 0, sizeof (fblock->values[0]) * (size - osize));
 }
 
 SfiFBlock*
@@ -226,14 +254,10 @@ sfi_fblock_append (SfiFBlock    *fblock,
 
   if (n_values)
     {
-      guint i;
-
       g_return_if_fail (values != NULL);
-
-      i = fblock->n_values;
-      fblock->n_values += n_values;
-      fblock->values = g_renew (gfloat, fblock->values, fblock->n_values);
-      memcpy (fblock->values + i, values, n_values * sizeof (fblock->values[0]));
+      guint oldsize = fblock->n_values;
+      fblock_resize (fblock, oldsize + n_values);
+      memcpy (fblock->values + oldsize, values, n_values * sizeof (fblock->values[0]));
     }
 }
 
@@ -241,13 +265,9 @@ void
 sfi_fblock_append1 (SfiFBlock *fblock,
 		    gfloat     float0)
 {
-  guint i;
-
   g_return_if_fail (fblock != NULL);
-
-  i = fblock->n_values++;
-  fblock->values = g_renew (gfloat, fblock->values, fblock->n_values);
-  fblock->values[i] = float0;
+  fblock_resize (fblock, fblock->n_values + 1);
+  fblock->values[fblock->n_values - 1] = float0;
 }
 
 guint
@@ -413,6 +433,19 @@ sfi_seq_check (SfiSeq *seq,
   return TRUE;
 }
 
+gboolean
+sfi_seq_validate (SfiSeq     *seq,
+                  GParamSpec *pspec)
+{
+  g_return_val_if_fail (seq != NULL, FALSE);
+  g_return_val_if_fail (pspec != NULL, FALSE);
+
+  GValue *v = sfi_value_seq (seq);
+  gboolean changed = g_param_value_validate (pspec, v);
+  sfi_value_free (v);
+  return changed;
+}
+
 void
 sfi_seq_append_bool (SfiSeq      *seq,
 		     SfiBool      v_bool)
@@ -1280,6 +1313,18 @@ sfi_rec_get_proxy (SfiRec      *rec,
 }
 
 
+/* --- basic comparisons --- */
+gint
+sfi_pointer_cmp (gconstpointer   value1,
+                 gconstpointer   value2,
+                 gpointer        dummy)
+{
+  const char *p1 = value1;
+  const char *p2 = value2;
+  return p1 < p2 ? -1 : p1 != p2;
+}
+
+
 /* --- ring (circular-list) --- */
 static inline SfiRing*
 sfi_ring_prepend_link_i (SfiRing *head,
@@ -1357,25 +1402,89 @@ sfi_ring_append_uniq (SfiRing *head,
 }
 
 SfiRing*
-sfi_ring_copy (SfiRing *head)
+sfi_ring_insert_before (SfiRing        *head,
+                        SfiRing        *sibling,
+                        gpointer        data)
 {
-  SfiRing *walk, *dest = NULL;
+  if (!sibling)
+    return sfi_ring_append (head, data);
+  SfiRing *node = sfi_ring_prepend (sibling, data);
+  return sibling == head ? node : head;
+}
+
+SfiRing*
+sfi_ring_insert (SfiRing *head,
+                 gpointer data,
+                 gint     position)
+{
+  if (position < 0)
+    return sfi_ring_append (head, data);
+  else if (position == 0)
+    return sfi_ring_prepend (head, data);
+  SfiRing *node = sfi_ring_nth (head, position);
+  if (node)
+    return sfi_ring_insert_before (head, node, data);
+  else
+    return sfi_ring_append (head, data);
+}
+
+gint
+sfi_ring_position (const SfiRing  *head,
+                   const SfiRing  *node)
+{
+  guint i = 0;
+  const SfiRing *ring;
+  for (ring = head; ring; ring = sfi_ring_walk (ring, head), i++)
+    if (ring == node)
+      return i;
+  return -1;
+}
+
+gint
+sfi_ring_index (const SfiRing *head,
+                gconstpointer  data)
+{
+  guint i = 0;
+  const SfiRing *ring;
+  for (ring = head; ring; ring = sfi_ring_walk (ring, head), i++)
+    if (ring->data == data)
+      return i;
+  return -1;
+}
+
+SfiRing*
+sfi_ring_copy (const SfiRing *head)
+{
+  const SfiRing *walk;
+  SfiRing *dest = NULL;
   for (walk = head; walk; walk = sfi_ring_walk (walk, head))
     dest = sfi_ring_append (dest, walk->data);
   return dest;
 }
 
 SfiRing*
-sfi_ring_copy_deep (SfiRing        *head,
+sfi_ring_copy_deep (const SfiRing  *head,
 		    SfiRingDataFunc copy,
 		    gpointer        func_data)
 {
-  SfiRing *walk, *dest = NULL;
+  const SfiRing *walk;
+  SfiRing *dest = NULL;
   for (walk = head; walk; walk = sfi_ring_walk (walk, head))
     dest = sfi_ring_append (dest, copy (walk->data, func_data));
   return dest;
 }
 
+SfiRing*
+sfi_ring_copy_rest (const SfiRing *ring,
+                    const SfiRing *head)
+{
+  const SfiRing *walk;
+  SfiRing *dest = NULL;
+  for (walk = ring; walk; walk = sfi_ring_walk (walk, head))
+    dest = sfi_ring_append (dest, walk->data);
+  return dest;
+}
+
 SfiRing*
 sfi_ring_concat (SfiRing *head1,
 		 SfiRing *head2)
@@ -1452,77 +1561,6 @@ sfi_ring_remove_node (SfiRing *head,
   return head;
 }
 
-SfiRing*
-sfi_ring_merge_sorted (SfiRing     *head1,
-		       SfiRing     *head2,
-		       GCompareFunc func)
-{
-  if (head1 && head2)
-    {
-      SfiRing *tail1 = head1->prev;
-      SfiRing *tail2 = head2->prev;
-      SfiRing *tmp, *ring = NULL;
-      /* NULL terminate rings */
-      tail1->next = NULL;
-      tail2->next = NULL;
-      while (head1 && head2)
-	{
-	  gint cmp = func (head1->data, head2->data);
-	  if (cmp <= 0)
-	    {
-	      tmp = head1;
-	      head1 = head1->next;
-	    }
-	  else
-	    {
-	      tmp = head2;
-	      head2 = head2->next;
-	    }
-	  ring = sfi_ring_append_link_i (ring, tmp);
-	}
-      /* reform valid rings, concat sorted rest */
-      if (head1)
-	{
-	  tail1->next = head1;
-	  head1->prev = tail1;
-	  return sfi_ring_concat (ring, head1);
-	}
-      if (head2)
-	{
-	  tail2->next = head2;
-	  head2->prev = tail2;
-	  return sfi_ring_concat (ring, head2);
-	}
-      return ring;
-    }
-  else
-    return sfi_ring_concat (head1, head2);
-}
-
-SfiRing*
-sfi_ring_sort (SfiRing     *head,
-	       GCompareFunc func)
-{
-  g_return_val_if_fail (func != NULL, head);
-
-  if (head && head->next != head)
-    {
-      SfiRing *ring, *tmp, *tail = head->prev;
-      /* find middle node to get log2 recursion depth */
-      ring = tmp = head->next;
-      while (tmp != tail && tmp->next != tail)
-	{
-	  ring = ring->next;
-	  tmp = tmp->next->next;
-	}
-      sfi_ring_split (head, ring);
-      return sfi_ring_merge_sorted (sfi_ring_sort (head, func),
-				    sfi_ring_sort (ring, func),
-				    func);
-    }
-  return head;
-}
-
 SfiRing*
 sfi_ring_reverse (SfiRing *head)
 {
@@ -1564,9 +1602,9 @@ sfi_ring_remove (SfiRing *head,
 }
 
 guint
-sfi_ring_length (SfiRing *head)
+sfi_ring_length (const SfiRing *head)
 {
-  SfiRing *ring;
+  const SfiRing *ring;
   guint i = 0;
   
   for (ring = head; ring; ring = sfi_ring_walk (ring, head))
@@ -1575,11 +1613,11 @@ sfi_ring_length (SfiRing *head)
   return i;
 }
 
-gboolean
-sfi_ring_test_length (SfiRing *head,
-		      guint    test_length)
+gint    /* essentially compute length(ring) - test_length, clamped to -1..+1 */
+sfi_ring_cmp_length (const SfiRing *head,
+                     guint          test_length)
 {
-  SfiRing *ring = head;
+  const SfiRing *ring = head;
 
   while (test_length && ring)
     {
@@ -1587,44 +1625,40 @@ sfi_ring_test_length (SfiRing *head,
       ring = sfi_ring_walk (ring, head);
     }
 
-  return test_length == 0 && !ring;
+  return test_length > 0 ? -1 : ring != NULL;
 }
 
 SfiRing*
-sfi_ring_find (SfiRing      *head,
+sfi_ring_find (const SfiRing*head,
 	       gconstpointer data)
 {
-  SfiRing *ring;
-
+  const SfiRing *ring;
   for (ring = head; ring; ring = sfi_ring_walk (ring, head))
     if (ring->data == (gpointer) data)
-      return ring;
-
+      return (SfiRing*) ring;
   return NULL;
 }
 
 SfiRing*
-sfi_ring_nth (SfiRing *head,
-	      guint    n)
+sfi_ring_nth (const SfiRing *head,
+	      guint          n)
 {
-  SfiRing *ring = head;
-
+  const SfiRing *ring = head;
   while (n-- && ring)
     ring = sfi_ring_walk (ring, head);
-
-  return ring;
+  return (SfiRing*) ring;
 }
 
 gpointer
-sfi_ring_nth_data (SfiRing *head,
-		   guint    n)
+sfi_ring_nth_data (const SfiRing *head,
+		   guint          n)
 {
-  SfiRing *ring = head;
+  const SfiRing *ring = head;
 
   while (n-- && ring)
     ring = sfi_ring_walk (ring, head);
 
-  return ring ? ring->data : ring;
+  return ring ? ring->data : NULL;
 }
 
 void
@@ -1680,39 +1714,487 @@ sfi_ring_pop_tail (SfiRing **head_p)
 }
 
 SfiRing*
-sfi_ring_insert_sorted (SfiRing	    *head,
-			gpointer     data,
-			GCompareFunc func)
+sfi_ring_from_list (GList *list)
 {
-  gint cmp;
+  SfiRing *ring = NULL;
+  for (; list; list = list->next)
+    ring = sfi_ring_append (ring, list->data);
+  return ring;
+}
 
-  g_return_val_if_fail (func != NULL, head);
+SfiRing*
+sfi_ring_from_list_and_free (GList *list)
+{
+  SfiRing *ring = NULL;
+  GList *free_list = list;
+  for (; list; list = list->next)
+    ring = sfi_ring_append (ring, list->data);
+  g_list_free (free_list);
+  return ring;
+}
 
+SfiRing*
+sfi_ring_from_slist (GSList *slist)
+{
+  SfiRing *ring = NULL;
+  for (; slist; slist = slist->next)
+    ring = sfi_ring_append (ring, slist->data);
+  return ring;
+}
+
+SfiRing*
+sfi_ring_from_slist_and_free (GSList *slist)
+{
+  SfiRing *ring = NULL;
+  GSList *free_slist = slist;
+  for (; slist; slist = slist->next)
+    ring = sfi_ring_append (ring, slist->data);
+  g_slist_free (free_slist);
+  return ring;
+}
+
+SfiRing*
+sfi_ring_insert_sorted (SfiRing	      *head,
+			gpointer       insertion_data,
+                        SfiCompareFunc cmp,
+                        gpointer       cmp_data)
+{
+  g_return_val_if_fail (cmp != NULL, head);
   if (!head)
-    return sfi_ring_prepend (head, data);
+    return sfi_ring_prepend (head, insertion_data);
 
-  /* typedef gint (*GCompareFunc) (gconstpointer a,
-   *                               gconstpointer b);
-   */
-  cmp = func (data, head->data);
-
-  if (cmp >= 0)	/* insert after head */
+  if (cmp (insertion_data, head->data, cmp_data) >= 0)  /* insert after head */
     {
       SfiRing *tmp, *tail = head->prev;
       
       /* make appending an O(1) operation */
-      if (head == tail || func (data, tail->data) >= 0)
-	return sfi_ring_append (head, data);
+      if (head == tail || cmp (insertion_data, tail->data, cmp_data) >= 0)
+	return sfi_ring_append (head, insertion_data);
 
       /* walk forward while data >= tmp (skipping equal nodes) */
       for (tmp = head->next; tmp != tail; tmp = tmp->next)
-	if (func (data, tmp->data) < 0)
+	if (cmp (insertion_data, tmp->data, cmp_data) < 0)
 	  break;
 
-      /* insert before sibling which is greater than data */
-      sfi_ring_prepend (tmp, data);	/* keep current head */
+      /* insert before sibling which is greater than insertion_data */
+      sfi_ring_prepend (tmp, insertion_data); /* keep current head */
       return head;
     }
   else /* cmp < 0 */
-    return sfi_ring_prepend (head, data);
+    return sfi_ring_prepend (head, insertion_data);
+}
+
+SfiRing*
+sfi_ring_merge_sorted (SfiRing        *head1,
+		       SfiRing        *head2,
+                       SfiCompareFunc  cmp,
+                       gpointer        data)
+{
+  if (head1 && head2)
+    {
+      SfiRing *tail1 = head1->prev;
+      SfiRing *tail2 = head2->prev;
+      SfiRing *tmp, *ring = NULL;
+      /* NULL terminate rings */
+      tail1->next = NULL;
+      tail2->next = NULL;
+      while (head1 && head2)
+	{
+	  gint c = cmp (head1->data, head2->data, data);
+	  if (c <= 0)
+	    {
+	      tmp = head1;
+	      head1 = head1->next;
+	    }
+	  else
+	    {
+	      tmp = head2;
+	      head2 = head2->next;
+	    }
+	  ring = sfi_ring_append_link_i (ring, tmp);
+	}
+      /* reform valid rings, concat sorted rest */
+      if (head1)
+	{
+	  tail1->next = head1;
+	  head1->prev = tail1;
+	  return sfi_ring_concat (ring, head1);
+	}
+      if (head2)
+	{
+	  tail2->next = head2;
+	  head2->prev = tail2;
+	  return sfi_ring_concat (ring, head2);
+	}
+      return ring;
+    }
+  else
+    return sfi_ring_concat (head1, head2);
+}
+
+SfiRing*
+sfi_ring_sort (SfiRing        *head,
+               SfiCompareFunc  cmp,
+               gpointer        data)
+{
+  g_return_val_if_fail (cmp != NULL, head);
+
+  if (head && head->next != head)
+    {
+      SfiRing *ring, *tmp, *tail = head->prev;
+      /* find middle node to get log2 recursion depth */
+      ring = tmp = head->next;
+      while (tmp != tail && tmp->next != tail)
+	{
+	  ring = ring->next;
+	  tmp = tmp->next->next;
+	}
+      sfi_ring_split (head, ring);
+      return sfi_ring_merge_sorted (sfi_ring_sort (head, cmp, data),
+				    sfi_ring_sort (ring, cmp, data),
+				    cmp, data);
+    }
+  return head;
+}
+
+SfiRing* /* eliminates duplicate nodes */
+sfi_ring_uniq (SfiRing        *sorted_ring1,
+               SfiCompareFunc  cmp,
+               gpointer        data)
+{
+  const SfiRing *ring = sfi_ring_copy_uniq (sorted_ring1, cmp, data);
+  sfi_ring_free (sorted_ring1);
+  return ring;
+}
+
+SfiRing* /* eliminates duplicate nodes */
+sfi_ring_copy_uniq (const SfiRing  *sorted_ring1,
+                    SfiCompareFunc  cmp,
+                    gpointer        data)
+{
+  const SfiRing *r1 = sorted_ring1;
+  SfiRing *r2 = NULL;
+  if (r1)
+    {
+      gpointer last_data = r1->data;
+      r2 = sfi_ring_append (r2, last_data);
+      for (r1 = sfi_ring_walk (r1, sorted_ring1); r1; r1 = sfi_ring_walk (r1, sorted_ring1))
+        if (cmp (last_data, r1->data, data))
+          {
+            last_data = r1->data;
+            r2 = sfi_ring_append (r2, last_data);
+          }
+    }
+  return r2;
+}
+
+SfiRing* /* merges rings without dups */
+sfi_ring_union (const SfiRing  *sorted_set1,
+                const SfiRing  *sorted_set2,
+                SfiCompareFunc  cmp,
+                gpointer        data)
+{
+  const SfiRing *r1 = sorted_set1, *r2 = sorted_set2;
+  SfiRing *d = NULL;
+  while (r1 && r2)
+    {
+      gint c = cmp (r1->data, r2->data, data);
+      if (c < 0)
+        {
+          d = sfi_ring_append (d, r1->data);
+          r1 = sfi_ring_walk (r1, sorted_set1);
+        }
+      else if (c > 0)
+        {
+          d = sfi_ring_append (d, r2->data);
+          r2 = sfi_ring_walk (r2, sorted_set2);
+        }
+      else
+        {
+          d = sfi_ring_append (d, r1->data);
+          r1 = sfi_ring_walk (r1, sorted_set1);
+          r2 = sfi_ring_walk (r2, sorted_set2);
+        }
+    }
+  return sfi_ring_concat (d, sfi_ring_copy_rest (r1 ? r1 : r2, r1 ? sorted_set1 : sorted_set2));
+}
+
+SfiRing* /* returns nodes contained in both rings */
+sfi_ring_intersection (const SfiRing  *sorted_set1,
+                       const SfiRing  *sorted_set2,
+                       SfiCompareFunc  cmp,
+                       gpointer        data)
+{
+  const SfiRing *r1 = sorted_set1, *r2 = sorted_set2;
+  SfiRing *d = NULL;
+  while (r1 && r2)
+    {
+      gint c = cmp (r1->data, r2->data, data);
+      if (c < 0)
+        r1 = sfi_ring_walk (r1, sorted_set1);
+      else if (c > 0)
+        r2 = sfi_ring_walk (r2, sorted_set2);
+      else
+        {
+          d = sfi_ring_append (d, r1->data);
+          r1 = sfi_ring_walk (r1, sorted_set1);
+          r2 = sfi_ring_walk (r2, sorted_set2);
+        }
+    }
+  return d;
+}
+
+SfiRing* /* produces set1 without the elements of set2 */
+sfi_ring_difference (const SfiRing  *sorted_set1,
+                     const SfiRing  *sorted_set2,
+                     SfiCompareFunc  cmp,
+                     gpointer        data)
+{
+  const SfiRing *r1 = sorted_set1, *r2 = sorted_set2;
+  SfiRing *d = NULL;
+  while (r1 && r2)
+    {
+      gint c = cmp (r1->data, r2->data, data);
+      if (c < 0)
+        {
+          d = sfi_ring_append (d, r1->data);
+          r1 = sfi_ring_walk (r1, sorted_set1);
+        }
+      else if (c > 0)
+        r2 = sfi_ring_walk (r2, sorted_set2);
+      else
+        {
+          r1 = sfi_ring_walk (r1, sorted_set1);
+          r2 = sfi_ring_walk (r2, sorted_set2);
+        }
+    }
+  return sfi_ring_concat (d, sfi_ring_copy_rest (r1, sorted_set1));
+}
+
+SfiRing* /* prduces difference (set1, set2) + difference (set2, set1) */
+sfi_ring_symmetric_difference (const SfiRing  *sorted_set1,
+                               const SfiRing  *sorted_set2,
+                               SfiCompareFunc  cmp,
+                               gpointer        data)
+{
+  const SfiRing *r1 = sorted_set1, *r2 = sorted_set2;
+  SfiRing *d = NULL;
+  while (r1 && r2)
+    {
+      gint c = cmp (r1->data, r2->data, data);
+      if (c < 0)
+        {
+          d = sfi_ring_append (d, r1->data);
+          r1 = sfi_ring_walk (r1, sorted_set1);
+        }
+      else if (c > 0)
+        {
+          d = sfi_ring_append (d, r2->data);
+          r2 = sfi_ring_walk (r2, sorted_set2);
+        }
+      else
+        {
+          r1 = sfi_ring_walk (r1, sorted_set1);
+          r2 = sfi_ring_walk (r2, sorted_set2);
+        }
+    }
+  return sfi_ring_concat (d, sfi_ring_copy_rest (r1 ? r1 : r2, r1 ? sorted_set1 : sorted_set2));
+}
+
+static inline int
+pointerloccmp (const void *pp1,
+               const void *pp2)
+{
+  const gpointer *p1 = pp1;
+  const gpointer *p2 = pp2;
+  return *p1 < *p2 ? -1 : *p1 != *p2;
+}
+
+static inline gboolean
+ring_reorder_lookup (guint          n_items,
+                     gpointer      *items,
+                     gpointer       key,
+                     guint         *indexp)
+{
+  guint offset = 0, n = n_items;
+  while (offset < n)
+    {
+      guint i = (offset + n) >> 1;
+      gint cmp = key < items[i] ? -1 : key != items[i];
+      if (cmp < 0)
+        n = i;
+      else if (cmp > 0)
+        offset = i + 1;
+      else /* match */
+        {
+          *indexp = i;
+          return TRUE;
+        }
+    }
+  return FALSE;
+}
+
+SfiRing* /* reproduce all elements from unordered_ring in the order new_ring_order */
+sfi_ring_reorder (SfiRing        *unordered_ring,
+                  const SfiRing  *new_ring_order)
+{
+  if (!unordered_ring || !new_ring_order)
+    return unordered_ring;
+  const SfiRing *ring;
+
+  /* construct a sorted array for faster lookups */
+  gpointer *items = NULL;
+  guint i, n_items = 0, n_alloced = 0;
+  for (ring = unordered_ring; ring; ring = sfi_ring_walk (ring, unordered_ring))
+    {
+      i = n_items++;
+      if (n_items > n_alloced)
+        {
+          n_alloced = sfi_alloc_upper_power2 (MAX (n_items, 32));
+          items = g_renew (gpointer, items, n_alloced);
+        }
+      items[i] = ring->data;
+    }
+  sfi_ring_free (unordered_ring);
+  unordered_ring = NULL;
+  qsort (items, n_items, sizeof (items[0]), pointerloccmp);
+
+  /* collapse duplicates */
+  guint j = 0, *counts = g_new0 (guint, n_items);
+  for (i = 0; i < n_items; i++)
+    if (items[j] != items[i])
+      {
+        if (++j != i)
+          items[j] = items[i];
+        counts[j] = 1;
+      }
+    else /* catch dups */
+      counts[j]++;
+  n_items = j + 1;      /* shrink to number of different items */
+
+  /* pick unordered_ring members in the order given by new_ring_order */
+  for (ring = new_ring_order; ring; ring = sfi_ring_walk (ring, new_ring_order))
+    if (ring_reorder_lookup (n_items, items, ring->data, &i) && counts[i])
+      {
+        counts[i]--;
+        unordered_ring = sfi_ring_append (unordered_ring, ring->data);
+      }
+
+  /* append left-over members from sorted_ring */
+  for (i = 0; i < n_items; i++)
+    while (counts[i]--)
+      unordered_ring = sfi_ring_append (unordered_ring, items[i]);
+
+  g_free (items);
+  g_free (counts);
+  return unordered_ring;
+}
+
+gboolean
+sfi_ring_includes (const SfiRing  *sorted_super_set,
+                   const SfiRing  *sorted_sub_set,
+                   SfiCompareFunc  cmp,
+                   gpointer        data)
+{
+  const SfiRing *r1 = sorted_super_set, *r2 = sorted_sub_set;
+  while (r1 && r2)
+    {
+      gint c = cmp (r1->data, r2->data, data);
+      if (c > 0)
+        return FALSE;
+      else if (c == 0)
+        r2 = sfi_ring_walk (r2, sorted_sub_set);
+      r1 = sfi_ring_walk (r1, sorted_super_set);
+    }
+  return !r2;
+}
+
+gboolean
+sfi_ring_equals (const SfiRing  *sorted_ring1,
+                 const SfiRing  *sorted_ring2,
+                 SfiCompareFunc  cmp,
+                 gpointer        data)
+{
+  const SfiRing *r1 = sorted_ring1, *r2 = sorted_ring2;
+  while (r1 && r2)
+    {
+      if (cmp (r1->data, r2->data, data))
+        return FALSE;
+      r1 = sfi_ring_walk (r1, sorted_ring1);
+      r2 = sfi_ring_walk (r2, sorted_ring2);
+    }
+  return r1 == r2; /* both need to be NULL */
+}
+
+gboolean
+sfi_ring_mismatch (SfiRing       **sorted_ring1_p,
+                   SfiRing       **sorted_ring2_p,
+                   SfiCompareFunc  cmp,
+                   gpointer        data)
+{
+  SfiRing *head1 = *sorted_ring1_p, *head2 = *sorted_ring2_p;
+  SfiRing *r1 = head1, *r2 = head2;
+  while (r1 && r2)
+    {
+      if (cmp (r1->data, r2->data, data))
+        goto mismatch;
+      r1 = sfi_ring_walk (r1, head1);
+      r2 = sfi_ring_walk (r2, head2);
+    }
+  if (r1 == r2) /* both are NULL */
+    return FALSE;
+ mismatch:
+  *sorted_ring1_p = r1;
+  *sorted_ring2_p = r2;
+  return TRUE;
+}
+
+SfiRing*
+sfi_ring_min_node (const SfiRing  *head,
+                   SfiCompareFunc  cmp,
+                   gpointer        data)
+{
+  const SfiRing *ring = head, *last = NULL;
+  if (ring)
+    {
+      last = ring;
+      for (ring = sfi_ring_walk (ring, head); ring; ring = sfi_ring_walk (ring, head))
+        if (cmp (last->data, ring->data, data) > 0)
+          last = ring;
+    }
+  return (SfiRing*) last;
+}
+
+SfiRing*
+sfi_ring_max_node (const SfiRing  *head,
+                   SfiCompareFunc  cmp,
+                   gpointer        data)
+{
+  const SfiRing *ring = head, *last = NULL;
+  if (ring)
+    {
+      last = ring;
+      for (ring = sfi_ring_walk (ring, head); ring; ring = sfi_ring_walk (ring, head))
+        if (cmp (last->data, ring->data, data) < 0)
+          last = ring;
+    }
+  return (SfiRing*) last;
+}
+
+gpointer
+sfi_ring_min (const SfiRing  *head,
+              SfiCompareFunc  cmp,
+              gpointer        data)
+{
+  SfiRing *ring = sfi_ring_min_node (head, cmp, data);
+  return ring ? ring->data : NULL;
+}
+
+gpointer
+sfi_ring_max (const SfiRing  *head,
+              SfiCompareFunc  cmp,
+              gpointer        data)
+{
+  SfiRing *ring = sfi_ring_max_node (head, cmp, data);
+  return ring ? ring->data : NULL;
 }
