@@ -49,12 +49,16 @@ guint8*	   sfi_bblock_get	   (const SfiBBlock *bblock);
 
 /* --- SfiFBlock primitive type --- */
 struct _SfiFBlock {
-  guint   ref_count;
-  guint   n_values;
-  gfloat *values;
+  guint     ref_count;
+  guint     n_values;
+  gfloat   *values;
+  GFreeFunc freefunc;
 };
 SfiFBlock* sfi_fblock_new	   (void);
 SfiFBlock* sfi_fblock_new_sized	   (guint	     size);
+SfiFBlock* sfi_fblock_new_foreign  (guint            n_values,
+                                    gfloat          *values,
+                                    GFreeFunc        freefunc);
 SfiFBlock* sfi_fblock_ref	   (SfiFBlock	    *fblock);
 void	   sfi_fblock_unref	   (SfiFBlock	    *fblock);
 void	   sfi_fblock_resize	   (SfiFBlock	    *fblock,
@@ -91,6 +95,8 @@ GValue*	 sfi_seq_get		(const SfiSeq	 *seq,
 				 guint		  index);
 gboolean sfi_seq_check		(SfiSeq		 *seq,
 				 GType		  element_type);
+gboolean sfi_seq_validate       (SfiSeq          *seq,
+                                 GParamSpec      *pspec);
 /* convenience */
 void     sfi_seq_append_bool	(SfiSeq          *seq,
 				 SfiBool	  v_bool);
@@ -241,52 +247,65 @@ SfiProxy     sfi_rec_get_proxy	(SfiRec		*rec,
 				 const gchar	*field_name);
 
 
+/* --- basic comparisons --- */
+typedef gint (*SfiCompareFunc)          (gconstpointer   value1,
+                                         gconstpointer   value2,
+                                         gpointer        data);
+gint     sfi_pointer_cmp                (gconstpointer   value1,
+                                         gconstpointer   value2,
+                                         gpointer        dummy);
+
+
 /* --- ring (circular-list) --- */
 typedef gpointer (*SfiRingDataFunc)	(gpointer	 data,
 					 gpointer	 func_data);
-struct _SfiRing		// FIXME: move rings into their own object files
+struct _SfiRing		// FIXME: move SfiRing into its own object file
 {
   SfiRing  *next;
   SfiRing  *prev;
   gpointer  data;
 };
 SfiRing*        sfi_ring_prepend        (SfiRing        *head,
-					 gpointer        data);
+                                         gpointer        data);
 SfiRing*        sfi_ring_prepend_uniq   (SfiRing        *head,
-					 gpointer        data);
+                                         gpointer        data);
 SfiRing*        sfi_ring_append         (SfiRing        *head,
-					 gpointer        data);
+                                         gpointer        data);
 SfiRing*        sfi_ring_append_uniq    (SfiRing        *head,
-					 gpointer        data);
-SfiRing*        sfi_ring_insert_sorted  (SfiRing        *head,
-					 gpointer        data,
-					 GCompareFunc    func);
+                                         gpointer        data);
+SfiRing*        sfi_ring_insert         (SfiRing        *head,
+                                         gpointer        data,
+                                         gint            position);
+SfiRing*        sfi_ring_insert_before  (SfiRing        *head,
+                                         SfiRing        *sibling,
+                                         gpointer        data);
+gint            sfi_ring_position       (const SfiRing  *head,
+                                         const SfiRing  *node);
+gint            sfi_ring_index          (const SfiRing  *head,
+                                         gconstpointer   data);
+SfiRing*        sfi_ring_nth            (const SfiRing  *head,
+                                         guint           n);
+gpointer        sfi_ring_nth_data       (const SfiRing  *head,
+                                         guint           n);
+SfiRing*        sfi_ring_find           (const SfiRing  *head,
+                                         gconstpointer   data);
 SfiRing*        sfi_ring_remove_node    (SfiRing        *head,
-					 SfiRing        *node);
+                                         SfiRing        *node);
 SfiRing*        sfi_ring_remove         (SfiRing        *head,
-					 gpointer        data);
-guint           sfi_ring_length         (SfiRing        *head);
-gboolean        sfi_ring_test_length    (SfiRing        *head,
-					 guint           test_length);
-SfiRing*        sfi_ring_copy           (SfiRing        *head);
-SfiRing*        sfi_ring_copy_deep      (SfiRing        *head,
-					 SfiRingDataFunc copy,
-					 gpointer	 func_data);
+                                         gpointer        data);
+guint           sfi_ring_length         (const SfiRing  *head);
+gint            sfi_ring_cmp_length     (const SfiRing  *head,
+                                         guint           test_length);
+SfiRing*        sfi_ring_copy           (const SfiRing  *head);
+SfiRing*        sfi_ring_copy_deep      (const SfiRing  *head,
+                                         SfiRingDataFunc copy,
+                                         gpointer        func_data);
+SfiRing*        sfi_ring_copy_rest      (const SfiRing  *ring,
+                                         const SfiRing  *head);
 SfiRing*        sfi_ring_concat         (SfiRing        *head1,
-					 SfiRing        *head2);
-SfiRing*        sfi_ring_find           (SfiRing        *head,
-					 gconstpointer   data);
-SfiRing*        sfi_ring_nth            (SfiRing        *head,
-					 guint           n);
-gpointer        sfi_ring_nth_data       (SfiRing        *head,
-					 guint           n);
+                                         SfiRing        *head2);
 SfiRing*        sfi_ring_split          (SfiRing        *head1,
-					 SfiRing        *head2);
-SfiRing*	sfi_ring_merge_sorted	(SfiRing	*head1,
-					 SfiRing	*head2,
-					 GCompareFunc	 func);
-SfiRing*        sfi_ring_sort           (SfiRing        *head,
-					 GCompareFunc    func);
+                                         SfiRing        *head2);
 SfiRing*        sfi_ring_reverse        (SfiRing        *head);
 gpointer        sfi_ring_pop_head       (SfiRing       **head);
 gpointer        sfi_ring_pop_tail       (SfiRing       **head);
@@ -294,11 +313,79 @@ gpointer        sfi_ring_pop_tail       (SfiRing       **head);
 #define         sfi_ring_push_tail      sfi_ring_append
 void            sfi_ring_free           (SfiRing        *head);
 void            sfi_ring_free_deep      (SfiRing        *head,
-					 GDestroyNotify  data_destroy);
+                                         GDestroyNotify  data_destroy);
 #define sfi_ring_tail(head)             ((head) ? (head)->prev : NULL)
 #define sfi_ring_walk(node,head_bound)  ((node)->next != (head_bound) ? (node)->next : NULL)
 #define sfi_ring_next                   sfi_ring_walk
 
+SfiRing* sfi_ring_from_list             (GList          *list);
+SfiRing* sfi_ring_from_list_and_free    (GList          *list);
+SfiRing* sfi_ring_from_slist            (GSList         *slist);
+SfiRing* sfi_ring_from_slist_and_free   (GSList         *slist);
+
+/* ring-modifying cmp-based operations */
+SfiRing* sfi_ring_insert_sorted         (SfiRing        *head,
+                                         gpointer        insertion_data,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        cmp_data);
+SfiRing* sfi_ring_merge_sorted          (SfiRing        *head1,
+                                         SfiRing        *head2,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+SfiRing* sfi_ring_sort                  (SfiRing        *head,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+SfiRing* sfi_ring_uniq                  (SfiRing        *sorted_ring1,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+SfiRing* sfi_ring_reorder               (SfiRing        *unordered_ring,
+                                         const SfiRing  *new_ring_order);
+/* ring-copying cmp-based operations */
+SfiRing* sfi_ring_copy_uniq             (const SfiRing  *sorted_ring1,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+SfiRing* sfi_ring_union                 (const SfiRing  *sorted_set1,
+                                         const SfiRing  *sorted_set2,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+SfiRing* sfi_ring_intersection          (const SfiRing  *sorted_set1,
+                                         const SfiRing  *sorted_set2,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+SfiRing* sfi_ring_difference            (const SfiRing  *sorted_set1,
+                                         const SfiRing  *sorted_set2,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+SfiRing* sfi_ring_symmetric_difference  (const SfiRing  *sorted_set1,
+                                         const SfiRing  *sorted_set2,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+/* const-result cmp-based operations */
+gboolean sfi_ring_includes              (const SfiRing  *sorted_super_set,
+                                         const SfiRing  *sorted_sub_set,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+gboolean sfi_ring_mismatch              (SfiRing       **sorted_ring1_p,
+                                         SfiRing       **sorted_ring2_p,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+gboolean sfi_ring_equals                (const SfiRing  *sorted_set1,
+                                         const SfiRing  *sorted_set2,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+SfiRing* sfi_ring_min_node              (const SfiRing  *head,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+SfiRing* sfi_ring_max_node              (const SfiRing  *head,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+gpointer sfi_ring_min                   (const SfiRing  *head,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+gpointer sfi_ring_max                   (const SfiRing  *head,
+                                         SfiCompareFunc  cmp,
+                                         gpointer        data);
+
 G_END_DECLS
 
 #endif /* __SFI_PRIMITIVES_H__ */
