@@ -956,3 +956,12 @@ sfi_serial_check_parse_null_token (GScanner *scanner)
   else
     return FALSE;
 }
+
+void
+sfi_value_store_stderr (const GValue *value)
+{
+  GString *gstring = g_string_new ("");
+  sfi_value_store_typed (value, gstring);
+  g_printerr ("((GValue*)%p)=%s\n", value, gstring->str);
+  g_string_free (gstring, TRUE);
+}
